/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *	@(#)MUC_scorer.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	main function for MUC scorer.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)MUC_scorer.c	6.1\t09/08/97\tSAIC";
#endif

#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void display_output_file_headers();
static void massage_entry(Catalog_entry *entry);
static void align_entry(Catalog_entry *entry);
static void score_entry(Catalog_entry *entry);
static void display_report_summary_entry(Catalog_entry *entry);
static void display_entry_scores(Catalog_entry *entry);
static void display_output_file_ends();


int
main(int argc, char **argv)
{
	Catalog_entry *entry;
	
	if (argc == 1)
	{
		fprintf(stderr, 
			"Usage: %s <configuration_filename>\n", 
			argv[0]);
		exit(1);
	}

	MUC_configuration_filename = STRALLOC(argv[1]);
    
	initialize_MUC_scorer();        /* configure and parse input */

	printf("\nScoring documents:\n" );fflush(stdout);
	display_output_file_headers();
	table_first(MUC_Catalog);
	for(entry=(Catalog_entry *)table_next(MUC_Catalog);
	    entry;
	    entry=(Catalog_entry *)table_next(MUC_Catalog))
	{
		printf("%s ", entry->docnum);fflush(stdout);
		massage_entry(entry);            /* get ready to map   */
		printf(".");fflush(stdout);
		align_entry(entry);              /* "map" objects      */
		printf(".");fflush(stdout);
		score_entry(entry);              /* sum tallies, add to total*/
		printf(".");fflush(stdout);
		display_map_history_entry(entry);/* output alignment   */
		printf(".");fflush(stdout);
		display_report_summary_entry(entry); /* one-liners   */
		printf(".");fflush(stdout);
		if( IS_CO )
		{
			display_entry_CO_partitions(entry);
			printf(".");fflush(stdout);
		}
		display_entry_scores(entry);     /* output results     */
		printf(" done.\n");fflush(stdout);
	}

	display_output_file_ends();

	fclose(MUC_map_history_file);
	if( IS_CO )
		fclose(MUC_partition_file);
	fclose(MUC_score_report_file);

	printf("Finished scoring documents\n");
	exit(0);
}

static
void
display_output_file_headers()
{
	char *dump_map_history;

	dump_map_history = option_find_first_value(MUC_Options,
						   ":dump_map_history");

	if( ! STRCASEEQ(dump_map_history, "no") )
		fprintf(MUC_map_history_file, "{\n");
	if( IS_CO )
		display_scores_header_CO();
}


static
void
massage_entry( Catalog_entry *entry )
{
	if( IS_ST )
		massage_ST_entry(entry);
	else if( IS_TE )
		massage_TE_entry(entry);
	else if( IS_TR )
		massage_TR_entry(entry);
	else if( IS_CO )
		massage_CO_entry(entry);
	else 
		massage_NE_entry(entry);
}

static
void
align_entry(Catalog_entry *entry)
{
	if( IS_CO )
		align_CO(entry);
	else if( IS_NE )
		align_NE(entry);
	else
		align_IE(entry);
}

static
void
score_entry(Catalog_entry *entry)
{
	if( IS_CO )
		score_entry_coref(entry);
	else
		score_entry_IE(entry);
}

static
void
display_entry_scores(Catalog_entry *entry)
{
	if( IS_CO )
		display_entry_scores_CO(entry);
	else
		display_entry_scores_IE(entry);
}

static
void
display_report_summary_entry(Catalog_entry *entry)
{
	char *use_IE_report_summary;

	use_IE_report_summary = 
		option_find_first_value(MUC_Options,
					":use_IE_report_summary");
	
	if(IS_CO && STRCASEEQ(use_IE_report_summary, "no"))
		display_report_summary_CO(entry);
	else if(IS_NE && STRCASEEQ(use_IE_report_summary, "no"))
		display_report_summary_NE(entry);
	else 
		display_report_summary_IE(entry);
}

static
void
display_output_file_ends()
{
	char *dump_map_history;

	dump_map_history = option_find_first_value(MUC_Options,
						   ":dump_map_history");

	if( ! STRCASEEQ(dump_map_history, "no") )
		fprintf(MUC_map_history_file, "}\n");
	if( IS_CO )
		display_summary_scores_CO();
	else
		display_summary_scores_IE();
}
