/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	@(#)align_CO.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      alignment of coreference "objects"
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)align_CO.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


static void align_CO_insts(TABLE *key_insts, 
			   TABLE *resp_insts,
			   TABLE *inst_pairs);
static void align_CO_insts_aux(Inst *ks[],
			       Inst *rs[],
			       long num_ks,
			       long num_rs,
			       TABLE *inst_pairs);
static Bool EQ( Inst *key, Inst *resp );
static Bool EQ_stringfill(Stringfill *key_stringfill,	
		   Stringfill *key_minstring,
		   Stringfill *response);
static Bool substring( Stringfill *s1, Stringfill *s2 );

static Stringfill *get_first_stringfill_in_min_slot( Inst *inst );
static Stringfill *get_next_stringfill_in_min_slot( Inst *inst );

static void pair_missing( Inst *key_inst, TABLE *inst_pairs );
static void pair_optional( Inst *key_inst, TABLE *inst_pairs );
static void pair_spurious( Inst *resp_inst, TABLE *inst_pairs );
static void pair_correct( Inst *key_inst, Inst *resp_inst, TABLE *inst_pairs );

/*
 *  align_CO -- given a catalog entry, locate the key and response class
 *    ( there's only one class in coreference ), then call the function
 *    that aligns two tables of insts into a table of inst_pairs
 */

void
align_CO(Catalog_entry *entry)
{
	Clas *key_clas;
	Clas *resp_clas;
	Clas_pair *clas_pair;
	
	if( entry == NULL )
	{
		fprintf(stderr,	
			"NULL catalog entry\n");
		exit(1);
	}
	
	table_first(entry->key_classes);
	key_clas = (Clas *) table_next(entry->key_classes);

	table_first(entry->response_classes);
	resp_clas = (Clas *) table_next(entry->response_classes);
	
	table_first(entry->clas_pairs);
	clas_pair = (Clas_pair *) table_next(entry->clas_pairs);

	align_CO_insts(key_clas->insts, 
		       resp_clas->insts, 
		       clas_pair->inst_pairs );
	inst_pair_sum_tallies(entry->total_slot_tallies, 
			      clas_pair->inst_pairs);
}

/* 
 * align_CO_insts -- copy the keys and responses into two arrays, and
 *     sort the arrays by the interval of the text slot stringfill.
 *     Then call the auxiliary alignment function to do the work.
 */


static
void
align_CO_insts(TABLE *key_insts, TABLE *resp_insts, TABLE *inst_pairs)
{
	long i;
	long num_keys;
	long num_resps;
	Inst **ks;
	Inst **rs;
	Inst *key_inst;
	Inst *resp_inst;
	
	num_keys = table_count( key_insts );
	num_resps = table_count( resp_insts );

	ks = (Inst **) calloc(sizeof(Inst *), num_keys);
	rs = (Inst **) calloc(sizeof(Inst *), num_resps);

	table_first(key_insts);
	for(i=0, key_inst=(Inst *)table_next(key_insts);
	    key_inst;
	    i++, key_inst=(Inst *)table_next(key_insts))
	{
		key_inst->is_aligned = False;
		ks[i] = key_inst;
	}
		
	table_first(resp_insts);
	for(i=0, resp_inst=(Inst *)table_next(resp_insts);
	    resp_inst;
	    i++, resp_inst=(Inst *)table_next(resp_insts))
	{
		resp_inst->is_aligned = False;
		rs[i] = resp_inst;
	}
	
	align_CO_insts_aux( ks, rs, num_keys, num_resps, inst_pairs );
	UFREE(ks);
	UFREE(rs);
}

/* align_CO_insts_aux -- given two arrays of instances,
 *     sorted by start offset, with ties broken
 *     by looking at the end offset (eg (2,3) < (3,4) and (2,3) <
 *     (2,2)).  Start at the top of both arrays.  If the current key
 *     comes before the current response, it's a missing.  If the
 *     current response comes before the current key, it's
 *     spurious.  If the current key matches the current response,
 *     its correct.  If none of the above are true, then the two
 *     instances overlap but don't match.  In that case look ahead
 *     one instance (only) to see if the next key or response
 *     matches the current key or response.  If there are no more
 *     keys, all remainng responses are spurious; If there are no
 *     more responses, all remaining keys are 'missing'.
 */

static
void
align_CO_insts_aux(Inst *ks[],
		   Inst *rs[],
		   long num_ks,
		   long num_rs,
		   TABLE *inst_pairs)
{
	long k = 0;
	long r = 0;
	long r2;
	Bool found;
	
	while((k < num_ks) || (r < num_rs))
	{
		/* key already aligned */
		if((k < num_ks) && ks[k]->is_aligned )
			k++;
		/* response already aligned */
		else if((r < num_rs) && rs[r]->is_aligned )
			r++;
		/* key comes before response */
		else if((k < num_ks) && (r < num_rs) && LT(ks[k], rs[r]))
		{
			if( inst_is_optional(k[ks]))
				pair_optional(k[ks], inst_pairs);
			else
				pair_missing( ks[k], inst_pairs );
			k++;
		}
		/* response comes before key */
		else if((k < num_ks) && (r < num_rs) && LT(rs[r], ks[k]))
		{
			pair_spurious( rs[r], inst_pairs );
			r++;
		}
		/* key and response match */
		else if((k < num_ks) && (r < num_rs) && EQ(ks[k], rs[r])) 
		{
			pair_correct( ks[k], rs[r], inst_pairs );
			k++;
			r++;
		} 
		/* key and response overlap but don't match */
		else if((k < num_ks) &&
			(r < num_rs) &&
			!LT(ks[k], rs[r]) &&
			!LT(rs[r], ks[k]) &&
			!EQ(ks[k], rs[r]))
		{
			/* look for matching response farther down */
			r2 = r + 1;
			found = False;
			while((r2 < num_rs) && !LT(ks[k], rs[r2]))
			{
				if( EQ(ks[k], rs[r2]) )
				{
					found = True;
					pair_correct(ks[k], rs[r2], inst_pairs );
					k++;
					break;
				}
				else	
					r2++;
			}
			if( !found )
			{
				if( inst_is_optional(k[ks]))
					pair_optional(k[ks], inst_pairs);
				else
					pair_missing( ks[k], inst_pairs );
				k++;
			}
		}
		else if( (k < num_ks) && !(r < num_rs) )
			/* rest of keys are missing */
			do
			{
				if( !(ks[k]->is_aligned) )
				{
					if( inst_is_optional(k[ks]))
						pair_optional(k[ks], inst_pairs);
					else
						pair_missing( ks[k], inst_pairs );
					k++;
				}
			} while( k < num_ks );
		else
			/* rest of responses are spurious */
			do
			{
				if( !(rs[r]->is_aligned) )
					pair_spurious(rs[r], inst_pairs);
				r++;
			} while( r < num_rs );
	}
}

/*
 * EQ -- two coref instances are equal if either 
 *       (1) their text string offsets match perfectly
 *       (2) If the response has no MIN strings, then 
 *              if the key MIN is a substring of the response TEXT and
 *                 the response TEXT is a substring of the key TEXT
 *		    ====> True
 *	 (3) If the response has MIN strings, then use them in place of the
 *	     response TEXT string in case (2) above.
 *
 */

static
Bool
EQ( Inst *key, Inst *resp )
{
	Stringfill *key_text, *resp_text, *key_min, *resp_min;
	
	key_text = get_first_stringfill_in_text_slot(key);
	resp_text = get_first_stringfill_in_text_slot(resp);
	
	if(key_text == NULL) return False;
	if(resp_text == NULL) return False;
	
	if(key_text->start_offset == resp_text->start_offset 
	   &&
	   key_text->end_offset == resp_text->end_offset )
		return True;
	else
	{
		resp_min = get_first_stringfill_in_min_slot(resp);
		if( resp_min == NULL ) /* no MIN in response */
			for(key_min = get_first_stringfill_in_min_slot(key);
			    key_min;
			    key_min = get_next_stringfill_in_min_slot(key))
			{
				if( EQ_stringfill(key_text,key_min,resp_text) )
					return True;
			}
		else /*response has MIN strings -- use them in place of text */
		{
			for(key_min = get_first_stringfill_in_min_slot(key);
			    key_min;
			    key_min = get_next_stringfill_in_min_slot(key))
				for(resp_min = get_first_stringfill_in_min_slot(resp);
				    resp_min;
				    resp_min = get_next_stringfill_in_min_slot(resp))
					if(EQ_stringfill(key_text,
							 key_min,
							 resp_min))
						return True;
		}
		
	}
	return False;
}

static
Bool
EQ_stringfill(Stringfill *key_stringfill,	
	      Stringfill *key_minstring,
	      Stringfill *response)
{
	Bool result = False;
	Stringfill *trimmed_min, *trimmed_resp;
	
	if( !offsets_overlap(key_stringfill, response) )
		return False;
	else if(substring(response,key_stringfill)
		&&
		substring(key_minstring,response))
		return True;
	else
	{
		trimmed_min =  trim_stringfill(key_minstring);
		trimmed_resp = trim_stringfill(response);
		
		if(substring(trimmed_resp, key_stringfill)
		   &&
		   substring(trimmed_min, response))
			result = True;

		stringfill_release((char *)trimmed_min);
		stringfill_release((char *)trimmed_resp);
		return result;
	}
}

static
Bool
substring( Stringfill *s1, Stringfill *s2 )
{
	if((s1->start_offset >= s2->start_offset)
	   &&
	   (s1->end_offset <= s2->end_offset))
		return True;
	else
		return False;
}


static 
Stringfill *
get_first_stringfill_in_min_slot( Inst *inst )
{
	char *slotname;
	Slot *slot;
	Mfil *mfil;
	Sfil *sfil;
	
	if( inst == NULL ) return NULL;
	if( inst->slots == NULL ) return NULL;
	slotname = option_find_first_value(MUC_Options, ":sgml_MIN_slot");
	if(slotname == NULL) return NULL;
	slot = slot_find(inst->slots, slotname);
	if( slot == NULL ) return NULL;
	if( slot->mfils == NULL ) return NULL;
	table_first( slot->mfils );
	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	if( mfil->sfils == NULL ) return NULL;
	table_first( mfil->sfils );
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}

static 
Stringfill *
get_next_stringfill_in_min_slot( Inst *inst )
{
	char *slotname;
	Slot *slot;
	Mfil *mfil;
	Sfil *sfil;
	
	if( inst == NULL ) return NULL;
	if( inst->slots == NULL ) return NULL;
	slotname = option_find_first_value(MUC_Options, ":sgml_MIN_slot");
	if(slotname == NULL) return NULL;
	slot = slot_find(inst->slots, slotname);
	if( slot == NULL ) return NULL;
	if( slot->mfils == NULL ) return NULL;

	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	if( mfil->sfils == NULL ) return NULL;
	table_first( mfil->sfils );
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}

/* 
 *
 */
	

/* missing insts */

static
void
pair_missing( Inst *key_inst, TABLE *inst_pairs )
{
	Inst_pair *inst_pair;
	
	if( !(key_inst->is_aligned) )
	{
		inst_pair = pair_min(key_inst);
		table_append((char *)inst_pair, inst_pairs);
		key_inst->is_aligned = True;
		key_inst->its_pair = inst_pair;
	}
}

static
void
pair_optional( Inst *key_inst, TABLE *inst_pairs )
{
	Inst_pair *inst_pair;
	
	if( !(key_inst->is_aligned) )
	{
		inst_pair = pair_oin(key_inst);
		table_append((char *)inst_pair, inst_pairs);
		key_inst->is_aligned = True;
		key_inst->its_pair = inst_pair;
	}
}

static
void
pair_spurious( Inst *resp_inst, TABLE *inst_pairs )
{
	Inst_pair *inst_pair;
	
	if( !(resp_inst->is_aligned ) )
	{
		inst_pair = pair_sin(resp_inst);
		table_append((char *)inst_pair, inst_pairs);
		resp_inst->is_aligned = True;
		resp_inst->its_pair = inst_pair;
	}
}

static
void
pair_correct( Inst *key_inst, Inst *resp_inst, TABLE *inst_pairs )
{
	Inst_pair *inst_pair;
	
	if( !(key_inst->is_aligned) )
	{
		inst_pair = pair_cin(key_inst, resp_inst);
		table_append((char *)inst_pair, inst_pairs);
		key_inst->is_aligned = True;
		resp_inst->is_aligned = True;
		key_inst->its_pair = resp_inst->its_pair = inst_pair;
	}
}


