/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	@(#)align_NE.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      alignment of named entity objects
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)align_NE.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void align__cl_NE(TABLE *key_classes, 
			 TABLE *response_classes, 
			 TABLE *clas_pairs);
static void align__in_ccl_NE(TABLE *key_insts, 
			     TABLE *resp_insts, 
			     TABLE *inst_pairs);
static void align_cin_NE(TABLE *key_insts, 
			 TABLE *resp_insts, 
			 TABLE *inst_pairs);
static void align_cin_pair_poss(TABLE *key_insts, 
				TABLE *resp_insts, 
				TABLE *poss_pairs);
static void align_cin_pair_poss_aux(Inst *ks[],
				    Inst *rs[],
				    long num_ks,
				    long num_rs,
				    TABLE *poss_pairs);
static void pair_correct(Inst *key_inst, 
			 Inst *resp_inst, 
			 TABLE *inst_pairs);
/*
 *  align_NE -- 
 */

void
align_NE(Catalog_entry *entry)
{
	if( entry == NULL )
	{
		fprintf(stderr,	
			"NULL catalog entry\n");
		exit(1);
	}

	align__cl_NE(entry->key_classes,
		     entry->response_classes,
		     entry->clas_pairs);

	clas_pair_sum_tallies(entry->total_slot_tallies, 
			      entry->clas_pairs);
	entry->total_slot_score = 
		compute_tallies_f( entry->total_slot_tallies, F_PR );
}

static
void
align__cl_NE(TABLE *key_classes, 
	     TABLE *response_classes, 
	     TABLE *clas_pairs)
{
	Clas *key_clas;
	Clas *resp_clas;
	Clas_pair *clas_pair;
	
	table_first(key_classes);
	table_first(response_classes);
	table_first(clas_pairs);
	for(key_clas=(Clas *)table_next(key_classes),
	    resp_clas=(Clas *)table_next(response_classes),
	    clas_pair=(Clas_pair *)table_next(clas_pairs);

	    (key_clas && resp_clas && clas_pair);

	    key_clas=(Clas *)table_next(key_classes),
	    resp_clas=(Clas *)table_next(response_classes),
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
	{
		align__in_ccl_NE(key_clas->insts, 
				 resp_clas->insts, 
				 clas_pair->inst_pairs );
		inst_pair_sum_tallies(clas_pair->tallies,
				      clas_pair->inst_pairs);
		clas_pair->score = 
			compute_tallies_f(clas_pair->tallies,
					  F_PR);
	}
}

static
void
align__in_ccl_NE(TABLE *key_insts, 
		 TABLE *resp_insts, 
		 TABLE *inst_pairs)
{
	inst_clear_aligned_flags(key_insts);
	inst_clear_aligned_flags(resp_insts);

	align_cin_NE(key_insts, resp_insts, inst_pairs);
	align_oin(key_insts, inst_pairs);
	align_min(key_insts, inst_pairs);
	align_sin(resp_insts, inst_pairs);
}

static
void
align_cin_NE(TABLE *key_insts, TABLE *resp_insts, TABLE *inst_pairs)
{
	TABLE *poss_pairs;
	TABLE *chosen_pairs;
	Inst_pair **pair_array;

	/* first, pair the insts that *can* be aligned */
	poss_pairs = table_list_create(default_compar,
				       NULL,
				       default_destroy);
	
	align_cin_pair_poss(key_insts, resp_insts, poss_pairs);

	/* sort possible pairings by f-score  */
	pair_array = (Inst_pair **)calloc(table_count(poss_pairs),
					  sizeof(Inst_pair *));
	
	inst_pair_table_2_array(pair_array, poss_pairs);
	qsort(pair_array, 
	      table_count(poss_pairs),
	      sizeof(Inst_pair *), 
	      inst_pair_f_compare );

	/* choose the (unchosen) pairs with highest f-score */
	chosen_pairs = table_list_create(default_compar,
					 NULL,
					 default_destroy);
	align_cin_choose_pairs(pair_array, 
			       table_count(poss_pairs),
			       chosen_pairs);

	/* put the chosen pairs back in order by text offset */
	UFREE(pair_array);
	pair_array = (Inst_pair **)calloc(table_count(chosen_pairs),
					  sizeof(Inst_pair *));
	
	inst_pair_table_2_array(pair_array, chosen_pairs);
	qsort(pair_array,
	      table_count(chosen_pairs),
	      sizeof(Inst_pair *),
	      inst_pair_offset_compare );
	
	/* put final result into inst_pairs table */
	inst_pair_array_2_table(inst_pairs, 
				pair_array, 
				table_count(chosen_pairs));

	/* clean up */
	UFREE(pair_array);
	table_destroy(poss_pairs);
	table_destroy(chosen_pairs);
}


static
void
align_cin_pair_poss(TABLE *key_insts, 
		    TABLE *resp_insts, 
		    TABLE *poss_pairs)
{
	long i;
	long num_keys;
	long num_resps;
	Inst **ks;
	Inst **rs;
	Inst *key_inst;
	Inst *resp_inst;
	
	num_keys = table_count( key_insts );
	num_resps = table_count( resp_insts );

	ks = (Inst **) calloc(sizeof(Inst *), num_keys);
	rs = (Inst **) calloc(sizeof(Inst *), num_resps);

	table_first(key_insts);
	for(i=0, key_inst=(Inst *)table_next(key_insts);
	    key_inst;
	    i++, key_inst=(Inst *)table_next(key_insts))
		ks[i] = key_inst;
		
	table_first(resp_insts);
	for(i=0, resp_inst=(Inst *)table_next(resp_insts);
	    resp_inst;
	    i++, resp_inst=(Inst *)table_next(resp_insts))
		rs[i] = resp_inst;
	
	align_cin_pair_poss_aux(ks, rs, num_keys, num_resps, poss_pairs );
	UFREE(ks);
	UFREE(rs);
}

static
void
align_cin_pair_poss_aux(Inst *ks[],
			Inst *rs[],
			long num_ks,
			long num_rs,
			TABLE *poss_pairs)
{
	long k = 0;
	long r = 0;
	long r2;
	
	while((k < num_ks) || (r < num_rs))
	{
		/* key comes before response */
		if((k < num_ks) && (r < num_rs) && LT(ks[k], rs[r]))
			k++;
		/* response comes before key */
		else if((k < num_ks) && (r < num_rs) && LT(rs[r], ks[k]))
			r++;
		/* key and resp overlap - pair all rs that overlap w/ key */
		else if((k < num_ks) && (r < num_rs))
		{
			r2 = r;
			while( (r2 < num_rs) && !LT(ks[k], rs[r2]))
			{
				pair_correct( ks[k], rs[r2], poss_pairs );
				r2++;
			}
			k++;
		} 
		/* no more responses */
		else if( (k < num_ks) && !(r < num_rs) )
				k++;
		/* no more keys */
		else
				r++;
	}
}

static
void
pair_correct( Inst *key_inst, Inst *resp_inst, TABLE *inst_pairs )
{
	Inst_pair *inst_pair;
	inst_pair = pair_cin( key_inst, resp_inst );
	table_append((char *)inst_pair, inst_pairs);
}

