/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      functions common to align_NE and align_CO
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 * AUTHOR
 */


#ifdef SCCSID
static char SccsId[] = "@(#)align_sgml.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


static void trim_stringfill_beginning(Stringfill *stringfill);
static char *find_beginning_to_trim(char *s);
static char *find_beginning_to_trim(char *s);
static char *skip_premodifier(char *string);
static char *skip_leading_corp(char *string);
static void trim_stringfill_end(Stringfill *stringfill);
static char *find_end_to_trim( char *wholestring, char *s );
static char *skip_trailing_whitespace( char *wholestring, char *p );
static char *skip_postmodifier(char *wholestring, char *p);
static char *skip_trailing_designators(char *wholestring, char *p);

/* 
 * LT -- an instance i1 is "less than" another instance i2 if the 
 *       end offset of i1 comes before the start offset of i2.
 *       -assumes that the first mfil in the text slot is the 
 *       "superstring" of all other mfils
 *
 */

Bool
LT( Inst *key, Inst *resp )
{
	Stringfill *key_text, *resp_text;

	key_text = get_first_stringfill_in_text_slot(key);
	resp_text = get_first_stringfill_in_text_slot(resp);

	if(key_text == NULL)
	{
		fprintf(stderr,
			"align_NE:LT() Missing key_text stringfill\n");
		exit(1);
	}

	if(resp_text == NULL)
	{
		fprintf(stderr,
			"align_NE:LT() Missing resp_text stringfill\n");
		exit(1);
	}
	
	return( key_text->end_offset < resp_text->start_offset );
}


Stringfill *
trim_stringfill( Stringfill *stringfill )
{
	Stringfill *result;
	
	result = stringfill_create();
	result->fill = STRALLOC(stringfill->fill);
	result->start_offset = stringfill->start_offset;
	result->end_offset = stringfill->end_offset;
	
	trim_stringfill_beginning(result);
	trim_stringfill_end(result);
	
	return result;
}

static 
void
trim_stringfill_beginning(Stringfill *stringfill)
{
	char *ptr;
	
	ptr = find_beginning_to_trim(stringfill->fill);

	while( ptr != stringfill->fill )
	{
		stringfill->start_offset += (ptr - stringfill->fill);
		/* copying with overlapping objects */
		muc_memmove(stringfill->fill, ptr, (ptr - stringfill->fill) );
		ptr = find_beginning_to_trim(stringfill->fill);
	}
}

static
char *
find_beginning_to_trim(char *s)
{
	char *bufptr;
	
	do
	{
		bufptr = s;

		s += strspn(s, " \t");   /* skip whitespace */
		
		s = skip_premodifier(s);
		
		s = skip_leading_corp(s);
	}
	while( (bufptr != s) && (*s) );
	
	return s;
}

static
char *
skip_premodifier(char *string)
{
	TABLE *premods;
	char *premod;
	int premod_len;
	char *ptr;
	
	premods = option_find_values(MUC_Options, 
				     ":premodifiers");
	
	table_first(premods);
	for(premod=table_next(premods);
	    premod;
	    premod=table_next(premods))
	{
		premod_len = strlen(premod);
		if(case_is_prefix(string, premod)
		   &&
		   (isspace( *( string + premod_len ) )
		    ||
		    ( *( string + premod_len ) == '\0' )))
			return string+premod_len;
	}
	return string;
}

static
char *
skip_leading_corp(char *string)
{
	TABLE *corps;
	char *corp;
	int corp_len;

	corps = option_find_values(MUC_Options, 
				     ":corporate_designators");

	table_first(corps);
	for(corp=table_next(corps);
	    corp;
	    corp=table_next(corps))
	{
		if(case_is_prefix(string, corp))
		{
			corp_len = strlen(corp);
			if((isspace( *( string + corp_len ) ))
			   ||
			   ( *( string + corp_len ) == '\0' ))
				return string + corp_len;
		}
	}
	return string;
}


static
void
trim_stringfill_end(Stringfill *stringfill)
{
	char *ptr;
	char *cur_end;
	
	cur_end = stringfill->fill + strlen(stringfill->fill) - 1; 
	ptr = find_end_to_trim(stringfill->fill, cur_end);
	
	while( ptr != cur_end )
	{
		stringfill->end_offset -= (cur_end - ptr);
		*(ptr+1) = '\0';
		cur_end = ptr;
		ptr = find_end_to_trim(stringfill->fill, cur_end);
	}
}

static
char *
find_end_to_trim( char *wholestring, char *s )
{
	char *bufptr;
	
	do
	{
		bufptr = s;
		
		s = skip_trailing_whitespace(wholestring, s);
		
		s = skip_postmodifier(wholestring, s);
		
		s = skip_trailing_designators(wholestring, s);
	}
	while( (bufptr != s) && (s > wholestring) );
	
	return s;
}

static
char *
skip_trailing_whitespace( char *wholestring, char *p )
{
	while(p > wholestring && (*p == ' ' || *p == '\t'))
		p--;
	return p;
}

static
char *
skip_postmodifier(char *wholestring, char *p)
{
	TABLE *postmods;
	char *postmod;

	postmods = option_find_values(MUC_Options, 
				      ":postmodifiers");
	table_first(postmods);
	for(postmod=table_next(postmods);
	    postmod;
	    postmod=table_next(postmods))
		if(case_is_suffix(wholestring, postmod))
			return p - strlen(postmod);

	return p;
}

static
char *
skip_trailing_designators(char *wholestring, char *p)
{
	TABLE *corps;
	char *corp;
	char *result;

	corps = option_find_values(MUC_Options, 
				      ":corporate_designators");
	table_first(corps);
	for(corp=table_next(corps);
	    corp;
	    corp=table_next(corps))
		if(case_is_suffix(wholestring, corp))
		{
			result = p - strlen(corp);
			/* preceded by space? or is whole string a corp? */
			if(isspace(*result)
			   || 
			   strlen(corp)==(p - wholestring + 1))
				return result; 
		}
	return p;
}

			
			
	

/* 
 * Two intervals overlap iff ones beginning is between the 
 *  beginning and end of the other.
 */
   

Bool
offsets_overlap(Stringfill *s1, Stringfill *s2)
{
	if(((s2->start_offset <= s1->start_offset)
	    &&
	    (s2->end_offset >= s1->start_offset))
	   ||
	   ((s1->start_offset <= s2->start_offset)
	    &&
	    (s1->end_offset >= s2->start_offset)))
		return True;
	else
		return False;
}

	   
	   

/* 
 * get_{first,next}_stringfill_in_{text,min}_slot
 *      these functions look up the slot name of the TEXT or MIN strings
 *      then get the first stringfill of the first mfil of the slot, 
 *      resetting the mfil table of the slot, or get the first stringfill
 *      of the next mfil of the slot
 */
   

Stringfill *
get_first_stringfill_in_text_slot( Inst *inst )
{
	char *slotname;
	Slot *slot;
	Mfil *mfil;
	Sfil *sfil;
	
	if( inst == NULL ) return NULL;
	if( inst->slots == NULL ) return NULL;
	slotname = option_find_first_value(MUC_Options, ":sgml_TEXT_slot");
	if(slotname == NULL) return NULL;
	slot = slot_find(inst->slots, slotname);
	if( slot == NULL ) return NULL;
	if( slot->mfils == NULL ) return NULL;
	table_first( slot->mfils );
	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	if( mfil->sfils == NULL ) return NULL;
	table_first( mfil->sfils );
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}

Stringfill *
get_next_stringfill_in_text_slot( Inst *inst )
{
	char *slotname;
	Slot *slot;
	Mfil *mfil;
	Sfil *sfil;
	
	if( inst == NULL ) return NULL;
	if( inst->slots == NULL ) return NULL;
	slotname = option_find_first_value(MUC_Options, ":sgml_TEXT_slot");
	if(slotname == NULL) return NULL;
	slot = slot_find(inst->slots, slotname);
	if( slot == NULL ) return NULL;
	if( slot->mfils == NULL ) return NULL;

	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	if( mfil->sfils == NULL ) return NULL;
	table_first( mfil->sfils );
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}


