/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	catalog_entry.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Catalog_entry data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)catalog_entry.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  catalog_entry_create						*
 *									*
 *  Initialize a new catalog_entry with DOCNUM as key.
 *									*
 ***********************************************************************/

Catalog_entry *
catalog_entry_create(char *docnum)
{
    TABLE *subtask_names;        /* tables used in named tallies init. */
    TABLE *doc_section_names;
    TABLE *dsgs;
    Catalog_entry *newentry;
    
    if((newentry = UALLOC(Catalog_entry, 1)) == (Catalog_entry *)NULL)
    {
	fprintf(stderr, "Error: [catalog_entry_create]: Allocation failed.\n");
	exit(1);
    }
    
    /* allocate a new string and copy the docnum string */
    newentry->docnum = STRALLOC(docnum);

    newentry->key_classes = table_list_create(clas_compare,
					    clas_release,
					    default_destroy);
    clas_init_table(newentry->key_classes, docnum);

    newentry->response_classes = table_list_create(clas_compare,
					    clas_release,
					    default_destroy);
    clas_init_table(newentry->response_classes, docnum);

    newentry->clas_pairs = table_list_create(clas_pair_compare,
					      clas_pair_release,
					      default_destroy);
    clas_pair_init_table(newentry->clas_pairs, 
			 newentry->key_classes,
			 newentry->response_classes);

    newentry->class_scores = table_list_create(class_score_compare,
					       class_score_release,
					       default_destroy);

    /* create tables of named tallies, with names */
    /* copied from global option table            */

    subtask_names = option_find_values(MUC_Options, ":ne_subtask_names");
    newentry->ne_subtask_tallies = named_tallies_table_create(subtask_names);
							      
    doc_section_names = option_find_values(MUC_Options, ":doc_sections");
    newentry->ne_doc_section_tallies = named_tallies_table_create(doc_section_names);
    
    dsgs = option_find_values(MUC_Options, 
			      ":doc_section_groups");
    if( dsgs )
	    newentry->dsg_tallies = named_tallies_table_create(dsgs);
    else
	    newentry->dsg_tallies = NULL;

    newentry->is_pointed_to = False;
    newentry->total_slot_tallies = tallies_create();
    newentry->text_filtering = tallies_create();
    
    class_score_init_table(newentry->class_scores, docnum);

    newentry->text_filtering = tallies_create();
	
    return(newentry);
}


/************************************************************************
 *									*
 *  catalog_entry_release						*
 *									*
 ***********************************************************************/

void
catalog_entry_release(char *elt)
{
    Catalog_entry *ptr = (Catalog_entry *)elt;
    
    if(ptr == (Catalog_entry *)NULL)
	return;

    UFREE(ptr->docnum);
    table_destroy(ptr->key_classes);
    table_destroy(ptr->response_classes);
    table_destroy(ptr->clas_pairs);
    table_destroy(ptr->class_scores);
    table_destroy(ptr->ne_subtask_tallies);
    table_destroy(ptr->ne_doc_section_tallies);
    table_destroy(ptr->dsg_tallies);

    tallies_release((char *) (ptr->total_slot_tallies));
    tallies_release((char *) (ptr->text_filtering));

    eqclasset_release((char *)ptr->key_eqclasset);
    eqclasset_release((char *)ptr->response_eqclasset);
    partitionset_release((char *)ptr->partitionset);
    
    UFREE(ptr);
}


/************************************************************************
 *									*
 *  catalog_entry_compare						*
 *									*
 ***********************************************************************/

int
catalog_entry_compare(char *elt1, char *elt2)
{
    Catalog_entry *ptr1 = (Catalog_entry *)elt1;
    Catalog_entry *ptr2 = (Catalog_entry *)elt2;

    return(muc_strcasecmp(ptr1->docnum, ptr2->docnum));
}

/************************************************************************
 *									*
 *  catalog_entry_find
 *									*
 ***********************************************************************/

Catalog_entry *
catalog_entry_find(TABLE *catalog_entries, char *docnum)
{
  Catalog_entry *catalog_entry;
  
  table_first(catalog_entries);
  for(catalog_entry = (Catalog_entry *)table_next(catalog_entries);
      catalog_entry && !(STRCASEEQ(docnum, catalog_entry->docnum));
      catalog_entry = (Catalog_entry *)table_next(catalog_entries))
    ;
  
  return(catalog_entry);
}


/************************************************************************
 *									*
 * catalog_entry_print_table
 *									*
 ***********************************************************************/

void
catalog_entry_print_table( TABLE *catalog_entrys, int indent )
{
    Catalog_entry *entry;
    
    printf("\n%*.sList of Catalog entries:\n", indent, "" );
    
    table_first(catalog_entrys);
    for(entry = (Catalog_entry *)table_next(catalog_entrys);
	entry;
	entry = (Catalog_entry *)table_next(catalog_entrys))
	catalog_entry_print(entry, indent+2);
}
/***********************************************************************
 *									*
 *  catalog_entry
 *									*
 ***********************************************************************/

void
catalog_entry_print(Catalog_entry *entry, int indent)
{
	if( entry == NULL ) return;
	
	printf("%*.sDocument %s:\n", indent, "", entry->docnum);
	clas_print_table(entry->key_classes, indent+2);
	clas_print_table(entry->response_classes, indent+2);
}

