/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	clas.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for clas data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)clas.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  clas_create						*
 *									*
 ***********************************************************************/

Clas *
clas_create(char *name, char *docnum)
{
    Clas *clas;
    
    if((clas = UALLOC(Clas, 1)) == NULL)
    {
	fprintf(stderr, "Error: [clas_create]: ");
	fprintf(stderr, "Allocation failed.\n");
	exit(1);
    }
    
    clas->name = STRALLOC(name);
    clas->docnum = STRALLOC(docnum);

    /* create an empty table of instances */
    clas->insts = table_list_create(inst_compare,
				    inst_release,
				    default_destroy);
    
    if(clas->insts == NULL)
    {
	fprintf(stderr, "Error: [clas_create]: ");
	fprintf(stderr, "Unable to create instances table.\n");
	exit(1);
    }
    
    return(clas);
}


/************************************************************************
 *									*
 *  clas_release						*
 *									*
 ***********************************************************************/

void
clas_release(char *elt)
{
    Clas *ptr = (Clas *)elt;
    
    if(ptr == (Clas *)NULL)
	return;
    
    UFREE(ptr->name);
    UFREE(ptr->docnum);
    
    /* free up the table of class instantiations */
    table_destroy(ptr->insts);
    
    UFREE(ptr);
}


/************************************************************************
 *									*
 *  clas_compare						*
 *									*
 ***********************************************************************/

int
clas_compare(char *elt1, char *elt2)
{
    Clas *ptr1 = (Clas *)elt1;
    Clas *ptr2 = (Clas *)elt2;
    
    return(muc_strcasecmp(ptr1->name, ptr2->name));
}


/************************************************************************
 *									*
 *  clas_find
 *									*
 ***********************************************************************/

Clas *
clas_find(TABLE *classes, char *name)
{
  Clas *clas;
  
  table_first(classes);
  for(clas = (Clas *)table_next(classes);
      clas && !(STRCASEEQ(name, clas->name));
      clas = (Clas *)table_next(classes))
    ;
  
  return(clas);
}


/************************************************************************
 *									*
 * clas_init_table -- in one document, create a class structure for
 *                    every defined class (assumes that the class 
 *                    definitions are in the order that they
 *                    should be aligned).
 *
 ***********************************************************************/

void
clas_init_table( TABLE *classes, char *docnum )
{
	Clas *clas;
	Class_def *class_def;
  
	table_first(MUC_Class_defs);
	for(class_def = (Class_def *)table_next(MUC_Class_defs);
	    class_def;
	    class_def = (Class_def *)table_next(MUC_Class_defs))
	{
		clas = clas_create(class_def->name,
				   docnum );
		table_append((char *)clas, classes);
	}
}

/************************************************************************
 *									*
 * clas_count_unaligned
 *
 ***********************************************************************/

int
clas_count_unaligned( TABLE *classes )
{
  Clas *clas;
  int num_unaligned = 0;
  
  table_first(classes);
  for(clas = (Clas *)table_next(classes);
      clas;
      clas = (Clas *)table_next(classes))
    if( !clas->is_aligned )
      num_unaligned++;
  return num_unaligned;
}


/************************************************************************
 *									*
 * clas_print_table
 *									*
 ***********************************************************************/

void
clas_print_table( TABLE *classes, int indent )
{
  Clas *clas;
  
  table_first(classes);
  for(clas = (Clas *)table_next(classes);
      clas;
      clas = (Clas *)table_next(classes))
    clas_print(clas, indent+2);
}

/************************************************************************
 *									*
 *  clas_print						*
 *									*
 ***********************************************************************/

void
clas_print(Clas *clas, int indent)
{
  if (clas == NULL)
    return;
  
  printf("%*.sclass %s instances (%s): (%d instances)\n",
	 indent, "", clas->name, clas->docnum,
	 table_count(clas->insts));
  
  inst_print_table( clas->insts, indent+2 );
}



