/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	clas_pair.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for clas_pair data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)clas_pair.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  clas_pair_create						*
 *									*
 ***********************************************************************/

Clas_pair *
clas_pair_create(Clas *key_clas, Clas *response_clas, char status)
{
    Clas_pair *clas_pair;
    
    if((clas_pair = UALLOC(Clas_pair, 1)) == (Clas_pair *)NULL)
    {
	fprintf(stderr, "Error: [clas_pair_create]: ");
	fprintf(stderr, "Allocation failed.\n");
	exit(1);
    }

    if(key_clas)
	    clas_pair->name = STRALLOC(key_clas->name);
    else if(response_clas)
	    clas_pair->name = STRALLOC(response_clas->name);
    else
	    clas_pair->name = NULL;
    
    clas_pair->key_clas = key_clas;
    clas_pair->response_clas = response_clas;
    clas_pair->status = status;
    clas_pair->tallies = tallies_create();
    clas_pair->score = 0.0;
    clas_pair->inst_pairs = table_list_create(default_compar,
					      inst_pair_release,
					      default_destroy);
    return(clas_pair);
}


/************************************************************************
 *									*
 *  clas_pair_release						*
 *									*
 ***********************************************************************/

void
clas_pair_release(char *elt)
{
    Clas_pair *ptr = (Clas_pair *)elt;
    
    if(ptr == (Clas_pair *)NULL)
	return;
    
    UFREE(ptr->name);
    tallies_release((char *) (ptr->tallies));
    
    /* free up the table of clas_pairs instantiations */
    table_destroy(ptr->inst_pairs);
    
    UFREE(ptr);
}

/************************************************************************
 *									*
 *  clas_pair_find
 *									*
 ***********************************************************************/

Clas_pair *
clas_pair_find(TABLE *clas_pairs, char *name)
{
  Clas_pair *clas_pair;
  
  table_first(clas_pairs);
  for(clas_pair = (Clas_pair *)table_next(clas_pairs);
      clas_pair && !(STRCASEEQ(name, clas_pair->name));
      clas_pair = (Clas_pair *)table_next(clas_pairs))
    ;
  
  return(clas_pair);
}
/************************************************************************
 *									*
 * clas_pair_init_table
 *									*
 ***********************************************************************/

void
clas_pair_init_table(TABLE *clas_pairs,
		     TABLE *key_classes,
		     TABLE *response_classes)
{
	Clas_pair *clas_pair;
	Clas *key_clas;
	Clas *response_clas;
	
	table_first(key_classes);
	table_first(response_classes);
	for(key_clas=(Clas *)table_next(key_classes), 
	    response_clas=(Clas *)table_next(response_classes);

	    key_clas && response_clas;

	    key_clas=(Clas *)table_next(key_classes), 
	    response_clas=(Clas *)table_next(response_classes))
	{
		clas_pair = clas_pair_create(key_clas, response_clas, COR);
		table_append((char *)clas_pair, clas_pairs);
	}
}

/************************************************************************
 *									*
 * clas_pair_sum_tallies
 *									*
 ***********************************************************************/

void
clas_pair_sum_tallies( Tallies *tallies, TABLE *clas_pairs )
{
  Clas_pair *clas_pair;
  
  table_first(clas_pairs);
  for(clas_pair = (Clas_pair *)table_next(clas_pairs);
      clas_pair;
      clas_pair = (Clas_pair *)table_next(clas_pairs))
    tallies_combine( tallies, clas_pair->tallies );
}

/************************************************************************
 *									*
 *  clas_pair_compare							*
 *									*
 ***********************************************************************/

int
clas_pair_compare(char *item1, char *item2)
{
    return ((int)((char *) item1 - (char *) item2));
}

/************************************************************************
 *									*
 * clas_pair_print_table
 *									*
 ***********************************************************************/

void
clas_pair_print_table( TABLE *clas_pairs, int indent )
{
  Clas_pair *clas_pair;
  
  table_first(clas_pairs);
  for(clas_pair = (Clas_pair *)table_next(clas_pairs);
      clas_pair;
      clas_pair = (Clas_pair *)table_next(clas_pairs))
    clas_pair_print(clas_pair, indent+2);
}

/************************************************************************
 *									*
 *  clas_pair_print						*
 *									*
 ***********************************************************************/

void
clas_pair_print(Clas_pair *clas_pair, int indent)
{
  if (clas_pair == NULL)
    return;
  
  printf( "\n%*.skey:", indent, "" );
  clas_print( clas_pair->key_clas, indent );

  printf( "\n%*.sresponse:", indent, "" );
  clas_print( clas_pair->response_clas, indent );

  tallies_print( clas_pair->tallies );
  printf("\n");

  printf("\n%*.sscore: %f", indent, "", clas_pair->score );

  inst_pair_print_table( clas_pair->inst_pairs, indent+2 );
}
