/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	class_def.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Class_def data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)class_def.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  class_def_create							*
 *									*
 *  Initialize a new class_def with name as key
 *  									*
 ************************************************************************/

Class_def *
class_def_create(char *name)
{
	Class_def *class_def;
	
    
	if((class_def = UALLOC(Class_def, 1)) == (Class_def *)NULL)
	{
		fprintf(stderr,	"Error: [class_def_create]: ");
		fprintf(stderr, "Allocation failed for '%s'.\n", name);
		exit(1);
	}
	
	class_def->name = STRALLOC(name);
	class_def->score_report_name = (char *) NULL;
	class_def->map_threshold = 0;
	class_def->is_scored_class = False;
	class_def->total_tallies = tallies_create();
	
	/* create an empty table for slot_defs */
	class_def->slot_defs = table_list_create(slot_def_compare,
						 slot_def_release,
						 default_destroy);
	return(class_def);
}


/************************************************************************
 *									*
 *  class_def_release							*
 *									*
 ***********************************************************************/

void
class_def_release(char *elt)
{
    Class_def *class_def = (Class_def *)elt;
    
    if(class_def == (Class_def *)NULL)
	return;

    UFREE(class_def->name);
    UFREE(class_def->score_report_name);
    tallies_release((char *) (class_def->total_tallies));
    table_destroy(class_def->slot_defs);
    
    UFREE(class_def);
}


/************************************************************************
 *									*
 *  class_def_compare							*
 *									*
 ***********************************************************************/

int
class_def_compare(char *elt1, char *elt2)
{
    Class_def *class_def1 = (Class_def *)elt1;
    Class_def *class_def2 = (Class_def *)elt2;
    
    return(muc_strcasecmp(class_def1->name, class_def2->name));
}


/************************************************************************
 *									*
 *  class_def_find						*
 *									*
 *  checks to see if there exists a Class_def by name of CNAME.		*
 *									*
 ***********************************************************************/

Class_def *
class_def_find(TABLE *class_defs, char *name)
{
	Class_def  *class_def;
      
	table_first(class_defs);
	for(class_def = (Class_def *)table_next(class_defs);
	    class_def && !(STRCASEEQ(name, class_def->name));
	    class_def = (Class_def *)table_next(class_defs))
		;
	return(class_def);
}


/************************************************************************
 *									*
 *  class_def_print_table
 *									*
 ***********************************************************************/

void
class_def_print_table(TABLE *class_defs, int indent)
{
    Class_def *class_def;
    
    if( class_defs == NULL ) return;

    printf("\nList of Class Definitions:\n");
    
    table_first(class_defs);
    for(class_def = (Class_def *)table_next(class_defs);
	class_def;
	class_def = (Class_def *)table_next(class_defs))
	    print_class_def(class_def, indent+2);
}


/************************************************************************
 *									*
 *  print_class_def							*
 *									*
 ***********************************************************************/

void
print_class_def(Class_def *class_def, int indent)
{
    if( class_def == NULL ) return;
    
    printf("%*.s%s-%s:\n", indent, "", class_def->name, class_def->score_report_name);
    printf("%*.sSlot_defs:\n", indent, "");
    slot_def_print_table(class_def->slot_defs, indent+2);
}
