/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	class_score.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Class_score data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)class_score.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  class_score_create							*
 *									*
 ***********************************************************************/

Class_score *
class_score_create(Class_def *class_def, char *docnum)
{
    Class_score *newscore;
    char *cname = class_def->name;
    
    if((newscore = UALLOC(Class_score, 1)) == (Class_score *)NULL)
    {
	fprintf(stderr, "Error: [class_score_create]: Allocation failed.\n");
	exit(1);
    }
    
    newscore->name = STRALLOC(cname);
    newscore->docnum = STRALLOC(docnum);
    newscore->tallies = tallies_create();
    
    /* create an empty table for the slot scores for this class
       and build it by calling initialize_slot_scores. */
    newscore->slot_scores = table_list_create(slot_score_compare,
					      slot_score_release,
					      default_destroy);

    if(newscore->slot_scores == (TABLE *)NULL)
    {
	fprintf(stderr, "Error: [class_score_create]: '%s' - ", cname);
	fprintf(stderr, "unable to create slot_scores table.\n");
	exit(1);
    }
    return(newscore);
}


/************************************************************************
 *									*
 *  class_score_release							*
 *									*
 ***********************************************************************/

void
class_score_release(char *elt)
{
    Class_score *ptr = (Class_score *)elt;
    
    if(ptr == (Class_score *)NULL)
	return;
    
    UFREE(ptr->name);
    UFREE(ptr->docnum);
    tallies_release((char *) ptr->tallies);
    
    /* free up the table of slot_scores */
    table_destroy(ptr->slot_scores);
    
    UFREE(ptr);
}


/************************************************************************
 *									*
 *  class_score_compare							*
 *									*
 ***********************************************************************/

int
class_score_compare(char *elt1, char *elt2)
{
    Class_score *ptr1 = (Class_score *)elt1;
    Class_score *ptr2 = (Class_score *)elt2;
    
    return(muc_strcasecmp(ptr1->name, ptr2->name));
}

/************************************************************************
 *									*
 *  class_score_find
 *									*
 ***********************************************************************/

Class_score *
class_score_find(TABLE *scores, char *cname)
{
    Class_score  *this_score;
    
    /* search the table for a class_score having the same
       name as cname. */
    table_first(scores);
    for(this_score = (Class_score *)table_next(scores);
	this_score && !(STRCASEEQ(cname, this_score->name));
	this_score = (Class_score *)table_next(scores))
	;
    
    return(this_score);
}

/************************************************************************
 *									*
 * class_score_init_table -- in one document, create a class_score
 *                    structure for every defined class
 *
 ***********************************************************************/

void
class_score_init_table( TABLE *class_scores, char *docnum )
{
	Class_score *class_score;
	Class_def *class_def;
	
	table_first(MUC_Class_defs);
	for(class_def = (Class_def *)table_next(MUC_Class_defs);
	    class_def;
	    class_def = (Class_def *)table_next(MUC_Class_defs))
	{
		class_score = class_score_create( class_def, docnum );
		slot_score_init_table(class_score->slot_scores, class_def, docnum);
		table_append((char *)class_score, class_scores);
	}
}

