%{
/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	config.y
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	MUC configuration file parser.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)config.y	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


extern void yyerror();
extern int yyleng;
extern int configlineno;

Option  *newoption;
TABLE   *newvalues;
char	*unqstr;

%}


%union 
{
    int   ival;
    char  *sval;
}

%token <ival> NUMBER
%token <sval> STRING
%token <sval> OPTION
%token <sval> VALUE
%token <sval> QSTRING

%%


config_specs:	option_spec
        |	config_specs option_spec
	;

option_spec:	option value_list
	|	option
	;

option:		OPTION
        {
		newoption = option_find(MUC_Options, yylval.sval);

		if( newoption == NULL )    /* a truly "new" option */
		{
			newoption = option_create(yylval.sval);
			table_append( (char *)newoption, MUC_Options );
		}
		else                       /* option exists -- overwrite its values */
		{
			table_destroy(newoption->values);
			newoption->values = table_list_create(cstring_compare,
							      default_release,
							      default_destroy);
		}
		newvalues = newoption->values;
		UFREE(yylval.sval);
        }
	;

value_list:	value
	|	value_list value
	;

value:		VALUE
	{
	    if (newvalues != NULL)
	    {
		    unqstr = STRALLOC(yylval.sval);
		    table_append(unqstr, newvalues);
	    }
	    UFREE(yylval.sval);
	}
	|	QSTRING
	{
	    if (newvalues != NULL)
	    {
		    unqstr = trim_quotes(yylval.sval);
		    table_append(unqstr, newvalues);
	    }
	    UFREE(yylval.sval);
	}
	;
		
%%

