/*
 *  Copyright 1997 Science Applications International Corporation.
 */

/*
 * NAME
 * 
 * FILE 
 *      cstring.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *	support routines for tables (see table.h) of character strings
 *
 * DIAGNOSTICS
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)cstring.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 * cstring_print_table
 *									*
 ***********************************************************************/

void
cstring_print_table(TABLE *char_table, int indent)
{
  char *cstr;
  table_first( char_table );
  for(cstr = (char *)table_next(char_table);
      cstr;
      cstr = (char *)table_next(char_table))
    printf("%*.s%s\n", indent, "", cstr);
}

/************************************************************************
 *
 * cstring_compare -- wrapper for strcmp which can be 
 *                passed to table_list_create
 *									*
 ***********************************************************************/

int
cstring_compare(char *c1, char *c2)
{
  return muc_strcasecmp(c1,c2);
}

TABLE *
cstring_make_prefix_table(TABLE *multi_tokens)
{
	char *p;
	char *token;
	char *multi_token;
	TABLE *result;

	result = table_list_create(cstring_compare,
				   default_release,
				   default_destroy);
	
	table_first(multi_tokens);
	for(multi_token=table_next(multi_tokens);
	    multi_token;
	    multi_token=table_next(multi_tokens))
	{
		p = multi_token + strcspn(multi_token, " \t");
		table_append(alloc_between(multi_token,p), result);
	}
	return result;
}

		

TABLE *
cstring_lower_table(TABLE *cstrings)
{
	char *cstring;
	char *lowered_string;
	
	TABLE *result = table_list_create(cstring_compare,
					  default_release,
					  default_destroy);
	table_first(cstrings);
	for(cstring=table_next(cstrings);
	    cstring;
	    cstring=table_next(cstrings))
	{
		lowered_string = string_lower(cstring);
		table_append(lowered_string, result);
	}
	return result;
}


/***********************************************************************
 *
 * cstring_case_hash -- this is just the default hash function
 *                      of libtable with a cast to lowercase
 *                      on the chars of the (string) key
 *
 **********************************************************************
 */

long
cstring_case_hash(char *k)
{
	long		h;

	h = 0;
	/*
	 * * Convert string to integer
	 */
	while (*k)
	{
		h += muc_tolower(*k);
		k++;
	}
	h %= 1048583;
	return h;
}
