/*
 *  Copyright 1997 Science Applications International Corporation.
 */

/*
 * FILE 
 *	eqclass.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *	support routines for Eqclass data structure.
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)eqclass.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  eqclass_create
 *									*
 ***********************************************************************/

Eqclass *
eqclass_create()
{
    Eqclass *eqclass;
    
    if((eqclass = UALLOC(Eqclass, 1)) == (Eqclass *)NULL)
    {
	fprintf(stderr, "Error: [eqclass_create]: Allocation failed.\n");
	exit(1);
    }
    
    eqclass->class_num = 0;
    eqclass->insts = table_list_create(inst_compare,
				       NULL,
				       default_destroy);
    return eqclass;
}


/************************************************************************
 *									*
 *  eqclass_release 
 *									*
 ***********************************************************************/

void
eqclass_release(char *elt)
{
	Eqclass *eqclass = (Eqclass *) elt;
	
	table_destroy(eqclass->insts);
	UFREE(eqclass);
}

/************************************************************************
 *									*
 *  eqclass_add_member
 *									*
 ***********************************************************************/

void
eqclass_add_member(Inst *inst, Eqclass *eqclass)
{
	if( inst == NULL || eqclass == NULL ) 
		return;
	
	table_append( (char *) inst, eqclass->insts );
}

/************************************************************************
 *									*
 *  eqclass_compute_indices
 *									*
 ***********************************************************************/

void
eqclass_compute_indices( TABLE *eqclasses )
{
	Eqclass *eqclass;
	int i;
	
	table_first(eqclasses);
	for(i=0, eqclass=(Eqclass *)table_next(eqclasses);
	    eqclass;
	    i++, eqclass=(Eqclass *)table_next(eqclasses))
		eqclass->class_num = i;
}		

void
eqclass_print_table(TABLE *eqclasses, int indent)
{
	Eqclass *eqclass;
	
	if( eqclasses == NULL ) return;
	
	printf("%*.sEquivalence classes:\n", indent, "");

	table_first(eqclasses);
	for(eqclass=(Eqclass *)table_next(eqclasses);
	    eqclass;
	    eqclass=(Eqclass *)table_next(eqclasses))
		eqclass_print(eqclass, indent+2);
}

void
eqclass_print(Eqclass *eqclass, int indent)
{
	if( eqclass == NULL ) return;

	printf("%*.sEquivalence class:\n", indent, "" );
	inst_print_table(eqclass->insts, indent+2);
}
