/*
 *  Copyright 1997 Science Applications International Corporation.
 */

/*
 * FILE 
 *	eqmap.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *	support routines for Eqmap data structure.
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)eqmap.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  eqmap_create
 *									*
 ***********************************************************************/

Eqmap *
eqmap_create(Inst *inst, Eqclass *eqclass)
{
    Eqmap *eqmap;
    
    if((eqmap = UALLOC(Eqmap, 1)) == (Eqmap *)NULL)
    {
	fprintf(stderr, "Error: [eqmap_create]: Allocation failed.\n");
	exit(1);
    }
    
    eqmap->inst = inst;
    eqmap->eqclass = eqclass;
    return eqmap;
}

/************************************************************************
 *									*
 *  eqmap_release 
 *									*
 ***********************************************************************/

void
eqmap_release(char *elt)
{
	Eqmap *eqmap = (Eqmap *)elt;
	UFREE(eqmap);
}

/************************************************************************
 *									*
 *  eqmap_compare -- cast eqmap->inst to int and return difference
 *									*
 ***********************************************************************/

int
eqmap_compare(char *elt1, char *elt2)
{
    Eqmap *eqmap1 = (Eqmap *)elt1;
    Eqmap *eqmap2 = (Eqmap *)elt2;
    
    return (int) (eqmap1->inst) - (int) (eqmap2->inst);
}


/************************************************************************
 *									*
 *  eqmap_hash -- cast eqmap->inst to long -- this may need changing
 *									*
 ***********************************************************************/

long
eqmap_hash( char *elt )
{
	Eqmap *eqmap = (Eqmap *) elt;
	if( elt == NULL )
		return 0;
	else
		return (long) (eqmap->inst);
}

void 
eqmap_print_table(TABLE *eqmaps, int indent)
{
	Eqmap *eqmap;
	
	table_first(eqmaps);
	for(eqmap=(Eqmap *)table_next(eqmaps);
	    eqmap;
	    eqmap=(Eqmap *)table_next(eqmaps))
		eqmap_print(eqmap, indent+2);
}


void 
eqmap_print( Eqmap *eqmap, int indent )
{
	if(eqmap == NULL) 
		printf("%*.sNULL eqmap\n", indent, "");
	else if(eqmap->inst==NULL)
		printf("%*.sNULL eqmap->inst\n", indent, "");
	else if(eqmap->eqclass==NULL)
		printf("%*.sNULL eqmap->eqclass\n", indent, "");
	else
		printf("%*.sID: %s eqclass size: %d\n", 
		       indent, 
		       "", 
		       eqmap->inst->id,
		       table_count(eqmap->eqclass->insts));
}

	
