/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	inst.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)inst.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  inst_create						*
 *									*
 ***********************************************************************/

Inst *
inst_create(char *name, char *docnum, char *id)
{
    Inst *inst;
    
    /* create a new Inst */

    if((inst = UALLOC(Inst, 1)) == NULL)
    {
	fprintf(stderr, "Error: [inst_create]: Allocation failed.\n");
	exit(1);
    }

    /* initialize */
    inst->name = STRALLOC(name);
    inst->docnum = STRALLOC(docnum);
    inst->id = STRALLOC(id);

    /* create an empty table for the slot_fills */
    inst->slots = table_list_create(slot_compare,
				    slot_release,
				    default_destroy);
    
    inst->is_key = False;
    inst->its_class_def = NULL;

    inst->is_aligned = False;
    inst->its_pair = NULL;
    inst->aligned_with = NULL;

    inst->its_rep_inst = inst;

    inst->is_instantiated = False;
    inst->is_pointed_to = False;

    inst->is_optional = False;
    inst->start_offset = 0;
    inst->end_offset = 0;
    inst->doc_section = NULL;
    inst->eqclass_index = 0;

    return(inst);
}


/************************************************************************
 *									*
 *  inst_release						*
 *									*
 ***********************************************************************/

void
inst_release(char *elt)
{
    Inst *ptr = (Inst *)elt;
    
    if(ptr == (Inst *)NULL)
	return;
    
    UFREE(ptr->name);
    UFREE(ptr->docnum);
    UFREE(ptr->id);

    table_destroy(ptr->slots);

    UFREE(ptr->doc_section);

    UFREE(ptr);
}


/************************************************************************
 *									*
 *  inst_compare						*
 *									*
 ***********************************************************************/

int
inst_compare(char *elt1, char *elt2)
{
    Inst *ptr1 = (Inst *)elt1;
    Inst *ptr2 = (Inst *)elt2;
    
    return(muc_strcasecmp(ptr1->id, ptr2->id));
}


/************************************************************************
 *									*
 * inst_find
 *									*
 ***********************************************************************/

Inst *
inst_find(TABLE *insts, char *instid)
{
	Inst *inst;
  
	table_first(insts);
	for(inst = (Inst *)table_next(insts);
	    inst && !(STRCASEEQ(instid, inst->id));
	    inst = (Inst *)table_next(insts))
		;
  
	return(inst);
}

/************************************************************************
 *									*
 * inst_clear_aligned_flags
 *									*
 ***********************************************************************/

int
inst_clear_aligned_flags( TABLE *insts )
{
  Inst *inst;
  
  table_first(insts);
  for(inst = (Inst *)table_next(insts);
      inst;
      inst = (Inst *)table_next(insts))
    inst->is_aligned = False;
}

/************************************************************************
 *									*
 * inst_count_unaligned
 *									*
 ***********************************************************************/

int
inst_count_unaligned( TABLE *insts )
{
  Inst *inst;
  int num_unaligned = 0;
  
  table_first(insts);
  for(inst = (Inst *)table_next(insts);
      inst;
      inst = (Inst *)table_next(insts))
    if( !inst->is_aligned )
      num_unaligned++;
  return num_unaligned;

}



/************************************************************************
 *									*
 *  inst_is_removed -- if every slot of an instance is either unscored
 *                     or "removed" (has no content except except 
 *                     pointers to unaligned optional objects), then
 *                     the instance is also "removed".
 *									*
 ***********************************************************************/

Bool
inst_is_removed(Inst *inst)
{
  Slot *slot;
  TABLE *slots;
  if( inst )
    {
      slots = inst->slots;
      table_first( slots );
      for(slot=(Slot *)table_next(slots);
	  slot;
	  slot=(Slot *)table_next(slots))
	if( !slot_is_removed(slot) && !slot_is_unscored(slot) )
	  return False;
    }
  return True;
}


/************************************************************************
 *									*
 *  inst_offsets_overlap
 *
 *  for NE instances, mapping can occur only between two instances	*
 *  whose text offset ranges have some overlap.				*
 *									*
 *  this is a pretty generous overlap check -- the only fail		*
 *  condition is if there is no overlap at all.  this occurs if		*
 *  there is a gap between the two ranges				*
 *  i.e., (k_end < r_start) or (r_end < k_start)			*
 *									*
 ***********************************************************************/

Bool
inst_offsets_overlap(Inst *kinst, Inst *rinst)
{
  if(   (kinst->end_offset < rinst->start_offset)
     || (rinst->end_offset < kinst->start_offset))
    return False;
  else
    return True;
}

/************************************************************************
 *									*
 * inst_find_aligned -- 
 *									*
 ***********************************************************************/

Inst *
inst_find_aligned(Inst *inst)
{
	if( inst )
		return inst->aligned_with;
	else
		return NULL;
}

Bool
inst_is_key(Inst *inst)
{
	if( inst )
		return inst->is_key;
	else
		return False;
}

Bool
inst_is_optional(Inst *inst)
{
	if( inst )
		return inst->is_optional;
	else
		return False;
}


/************************************************************************
 *									*
 * inst_follow_REF_link  -- used in coreference scoring
 *									*
 ***********************************************************************/


Inst *
inst_follow_REF_link(Inst *inst)
{
	Pointerfill *pointerfill;
	
	if( inst )
	{
		pointerfill = get_inst_REF_slot_fill(inst);
		if( pointerfill )
			return pointerfill->fill;
	}
	return NULL;
}


/************************************************************************
 *									*
 * inst_print_table
 *									*
 ***********************************************************************/

void
inst_print_table( TABLE *insts, int indent )
{
  Inst *inst;
  
  table_first(insts);
  for(inst = (Inst *)table_next(insts);
      inst;
      inst = (Inst *)table_next(insts))
    inst_print(inst, indent);
}


/************************************************************************
 *									*
 *  inst_print						*
 *									*
 ***********************************************************************/

void
inst_print(Inst *inst, int indent)
{
    if (inst == NULL)
	return;
    
    printf("%*.s%s-%s-%s:\n", indent, "", inst->name, inst->docnum, inst->id);
    slot_print_table( inst->slots, indent+2 );
}
