/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * FILE 
 *     @(#)map_history.c	6.1
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *    output of alignment
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * SEE ALSO
 *
 * AUTHOR
 * */

#ifdef SCCSID
static char SccsId[] = "@(#)map_history.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void display_clas_pairs(TABLE *clas_pairs);
static void display_clas_pair(Clas_pair *clas_pair);
static void display_inst_pairs(TABLE *inst_pairs);
static void display_inst_pair(Inst_pair *inst_pair);
static void display_slot_pairs(TABLE *slot_pairs);
static void display_slot_pair(Slot_pair *slot_pair);
static void display_mfil_pairs(TABLE *mfil_pairs);
static void display_mfil_pair(Mfil_pair *mfil_pair);
static void display_sfil_pairs(TABLE *sfil_pairs);
static void display_sfil_pair(Sfil_pair *sfil_pair);
static void display_sfil(Sfil *sfil);
static void display_setfill(Setfill *setfill);
static void display_pointerfill(Pointerfill *pointerfill);
static void display_stringfill(Stringfill *stringfill);
static void display_tallies_list(Tallies *tallies);
static void display_status(char status);
static void display_tcl_string(char *s);
static void display_long(long n);

#ifdef SCCSID
static char SccsId[] = "@(#)map_history.c	6.1 SAIC";
#endif

#include "muc.h"
#include "muc_protos.h"
#include "muc_globals.h"

void
display_map_history_entry(Catalog_entry *entry)
{
	char *dump_map_history;

	dump_map_history = option_find_first_value(MUC_Options,
						   ":dump_map_history");

	if( STRCASEEQ(dump_map_history, "no") )
		return;
	

	fprintf(MUC_map_history_file, "{\n");
	if( entry )
	{	
		fprintf(MUC_map_history_file, "-docnum ");
		display_tcl_string(entry->docnum);
		fprintf(MUC_map_history_file, "\n");

		fprintf(MUC_map_history_file, "-doctallies ");
		display_tallies_list(entry->total_slot_tallies);
		fprintf(MUC_map_history_file, "\n");

		fprintf(MUC_map_history_file, "-class_pairs {\n");
		display_clas_pairs(entry->clas_pairs);
		fprintf(MUC_map_history_file, "}\n");
	}
	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_clas_pairs(TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
		display_clas_pair(clas_pair);
}

static
void
display_clas_pair(Clas_pair *clas_pair)
{
	fprintf(MUC_map_history_file, "{\n");

	fprintf(MUC_map_history_file, "-class_name ");
	display_tcl_string(clas_pair->name);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-class_tallies ");
	display_tallies_list(clas_pair->tallies);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-obj_pairs {\n");
	display_inst_pairs(clas_pair->inst_pairs);
	fprintf(MUC_map_history_file, "}\n");

	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_inst_pairs(TABLE *inst_pairs)
{
	Inst_pair *inst_pair;
	
	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
		display_inst_pair(inst_pair);
}

static
void
display_inst_pair(Inst_pair *inst_pair)
{
	fprintf(MUC_map_history_file, "{\n");

	fprintf(MUC_map_history_file, "-obj_pair_status ");
	display_status(inst_pair->status);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-obj_pair_tallies ");
	display_tallies_list(inst_pair->tallies);
	fprintf(MUC_map_history_file, "\n");

	if(inst_pair->key_inst)
	{
		fprintf(MUC_map_history_file, "-key_obj_id ");
		display_tcl_string(inst_pair->key_inst->id);
		fprintf(MUC_map_history_file, "\n");

		if(inst_pair->key_inst->is_optional)
			fprintf(MUC_map_history_file, "-key_obj_optional\n");

		if(inst_pair->key_inst->its_rep_inst)
		{
			fprintf(MUC_map_history_file, "-key_obj_rep_id ");
			display_tcl_string(inst_pair->key_inst->its_rep_inst->id);
			fprintf(MUC_map_history_file, "\n");
		}

		if(inst_pair->key_inst->end_offset)
		{
			fprintf(MUC_map_history_file, "-key_obj_start_offset ");
			display_long(inst_pair->key_inst->start_offset);
			fprintf(MUC_map_history_file, "\n");
			fprintf(MUC_map_history_file, "-key_obj_end_offset ");
			display_long(inst_pair->key_inst->end_offset);
			fprintf(MUC_map_history_file, "\n");
		}
	}

	if(inst_pair->response_inst)
	{
		fprintf(MUC_map_history_file, "-rsp_obj_id ");
		display_tcl_string(inst_pair->response_inst->id);
		fprintf(MUC_map_history_file, "\n");

		if(inst_pair->response_inst->is_optional)
			fprintf(MUC_map_history_file, "-rsp_obj_optional\n");

		if(inst_pair->response_inst->its_rep_inst)
		{
			fprintf(MUC_map_history_file, "-rsp_obj_rep_id ");
			display_tcl_string(inst_pair->response_inst->its_rep_inst->id);
			fprintf(MUC_map_history_file, "\n");
		}

		if(inst_pair->response_inst->end_offset)
		{
			fprintf(MUC_map_history_file, "-rsp_obj_start_offset ");
			display_long(inst_pair->response_inst->start_offset);
			fprintf(MUC_map_history_file, "\n");
			fprintf(MUC_map_history_file, "-rsp_obj_end_offset ");
			display_long(inst_pair->response_inst->end_offset);
			fprintf(MUC_map_history_file, "\n");
		}
	}

	if(inst_pair->key_inst
	   && 
	   inst_pair->key_inst->doc_section )
	{
		fprintf(MUC_map_history_file, "-doc_section ");
		display_tcl_string(inst_pair->key_inst->doc_section);
		fprintf(MUC_map_history_file, "\n");
	}
	else if(inst_pair->response_inst
		&& 
		inst_pair->response_inst->doc_section )
	{
		fprintf(MUC_map_history_file, "-doc_section ");
		display_tcl_string(inst_pair->response_inst->doc_section);
		fprintf(MUC_map_history_file, "\n");
	}


	fprintf(MUC_map_history_file, "-slot_pairs {\n");
	display_slot_pairs(inst_pair->slot_pairs);
	fprintf(MUC_map_history_file, "}\n");

	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_slot_pairs(TABLE *slot_pairs)
{
	Slot_pair *slot_pair;
	
	table_first(slot_pairs);
	for(slot_pair=(Slot_pair *)table_next(slot_pairs);
	    slot_pair;
	    slot_pair=(Slot_pair *)table_next(slot_pairs))
		display_slot_pair(slot_pair);
}

static
void
display_slot_pair(Slot_pair *slot_pair)
{
	fprintf(MUC_map_history_file, "{\n");

	fprintf(MUC_map_history_file, "-slot_name ");
	display_tcl_string(slot_pair->name);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-slot_tallies ");
	display_tallies_list(slot_pair->tallies);
	fprintf(MUC_map_history_file, "\n");

	if(slot_pair->key_slot
	   &&
	   slot_pair->key_slot->is_optional)
		fprintf(MUC_map_history_file, "-key_slot_optional\n");
		
	if(slot_pair->response_slot
	   &&
	   slot_pair->response_slot->is_optional)
		fprintf(MUC_map_history_file, "-rsp_slot_optional\n");
		
		
	fprintf(MUC_map_history_file, "-multi_fill_pairs {\n");
	display_mfil_pairs(slot_pair->mfil_pairs);
	fprintf(MUC_map_history_file, "}\n");

	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_mfil_pairs(TABLE *mfil_pairs)
{
	Mfil_pair *mfil_pair;
	
	table_first(mfil_pairs);
	for(mfil_pair=(Mfil_pair *)table_next(mfil_pairs);
	    mfil_pair;
	    mfil_pair=(Mfil_pair *)table_next(mfil_pairs))
		display_mfil_pair(mfil_pair);
}

static
void
display_mfil_pair(Mfil_pair *mfil_pair)
{
	fprintf(MUC_map_history_file, "{\n");

	fprintf(MUC_map_history_file, "-multi_fill_tallies ");
	display_tallies_list(mfil_pair->tallies);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-single_fill_pairs {\n");
	display_sfil_pairs(mfil_pair->sfil_pairs);
	fprintf(MUC_map_history_file, "}\n");

	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_sfil_pairs(TABLE *sfil_pairs)
{
	Sfil_pair *sfil_pair;
	
	table_first(sfil_pairs);
	for(sfil_pair=(Sfil_pair *)table_next(sfil_pairs);
	    sfil_pair;
	    sfil_pair=(Sfil_pair *)table_next(sfil_pairs))
		display_sfil_pair(sfil_pair);
}

static
void
display_sfil_pair(Sfil_pair *sfil_pair)
{
	fprintf(MUC_map_history_file, "{\n");

	fprintf(MUC_map_history_file, "-single_fill_pair_status ");
	display_status(sfil_pair->status);
	fprintf(MUC_map_history_file, "\n");

	fprintf(MUC_map_history_file, "-single_fill_pair_tallies ");
	display_tallies_list(sfil_pair->tallies);
	fprintf(MUC_map_history_file, "\n");

	if( sfil_pair->key_sfil )
	{
		fprintf(MUC_map_history_file, "-key_single_fill {\n");
		display_sfil(sfil_pair->key_sfil);
		fprintf(MUC_map_history_file, "}\n");
	}
	
	if( sfil_pair->response_sfil )
	{
		fprintf(MUC_map_history_file, "-rsp_single_fill {\n");
		display_sfil(sfil_pair->response_sfil);
		fprintf(MUC_map_history_file, "}\n");
	}
	
	fprintf(MUC_map_history_file, "}\n");
}

static
void
display_sfil(Sfil *sfil)
{
	if(sfil)
	{
		switch(sfil->type)
		{
		case SET_FILL:
			display_setfill(sfil->fill.set);
			break;
		case STRING_FILL:
			display_stringfill(sfil->fill.str);
			break;
		case POINTER_FILL:
			display_pointerfill(sfil->fill.ptr);
			break;
		default:
			break;
		}
	}
}

static
void
display_setfill(Setfill *setfill)
{
	if(setfill)
	{
		fprintf(MUC_map_history_file, "-type ");
		display_tcl_string("SET");
		fprintf(MUC_map_history_file, "\n");

		fprintf(MUC_map_history_file, "-fill ");
		display_tcl_string(setfill->fill);
		fprintf(MUC_map_history_file, "\n");

		if( setfill->end_offset )
		{
			fprintf(MUC_map_history_file, "-start_offset ");
			display_long(setfill->start_offset);
			fprintf(MUC_map_history_file, "\n");

			fprintf(MUC_map_history_file, "-end_offset ");
			display_long(setfill->end_offset);
			fprintf(MUC_map_history_file, "\n");
		}
	}
}

static
void
display_pointerfill(Pointerfill *pointerfill)
{
	if(pointerfill)
	{
		fprintf(MUC_map_history_file, "-type ");
		display_tcl_string("PTR");
		fprintf(MUC_map_history_file, "\n");

		fprintf(MUC_map_history_file, "-fill ");
		display_tcl_string(pointerfill->string_rep);
		fprintf(MUC_map_history_file, "\n");

		if( pointerfill->end_offset )
		{
			fprintf(MUC_map_history_file, "-start_offset ");
			display_long(pointerfill->start_offset);
			fprintf(MUC_map_history_file, "\n");

			fprintf(MUC_map_history_file, "-end_offset ");
			display_long(pointerfill->end_offset);
			fprintf(MUC_map_history_file, "\n");
		}
	}
}

static
void
display_stringfill(Stringfill *stringfill)
{
	if(stringfill)
	{
		fprintf(MUC_map_history_file, "-type ");
		display_tcl_string("STR");
		fprintf(MUC_map_history_file, "\n");

		fprintf(MUC_map_history_file, "-fill ");
		display_tcl_string(stringfill->fill);
		fprintf(MUC_map_history_file, "\n");

		if( stringfill->clean_fill )
		{
			fprintf(MUC_map_history_file, "-clean_fill ");
			display_tcl_string(stringfill->clean_fill);
			fprintf(MUC_map_history_file, "\n");
		}

		if( stringfill->end_offset )
		{
			fprintf(MUC_map_history_file, "-start_offset ");
			display_long(stringfill->start_offset);
			fprintf(MUC_map_history_file, "\n");

			fprintf(MUC_map_history_file, "-end_offset ");
			display_long(stringfill->end_offset);
			fprintf(MUC_map_history_file, "\n");
		}
	}
}

static
void
display_tallies_list(Tallies *tallies)
{
	fprintf(MUC_map_history_file, "{");
	if(tallies)
	{
		fprintf(MUC_map_history_file, "%d ", tallies->possible);
		fprintf(MUC_map_history_file, "%d ", tallies->actual);
		fprintf(MUC_map_history_file, "%d ", tallies->correct);
		fprintf(MUC_map_history_file, "%d ", tallies->partial);
		fprintf(MUC_map_history_file, "%d ", tallies->incorrect);
		fprintf(MUC_map_history_file, "%d ", tallies->missing);
		fprintf(MUC_map_history_file, "%d ", tallies->spurious);
		fprintf(MUC_map_history_file, "%d", tallies->noncommittal);
	}
	fprintf(MUC_map_history_file, "}");
}	

static
void
display_status(char status)
{
	switch(status)
	{
	case COR:
		display_tcl_string("COR");
		break;
	case INC:
		display_tcl_string("INC");
		break;
	case PAR:
		display_tcl_string("PAR");
		break;
	case MIS:
		display_tcl_string("MIS");
		break;
	case SPU:
		display_tcl_string("SPU");
		break;
	case OPT:
		display_tcl_string("OPT");
		break;
	case NON:
		display_tcl_string("NON");
		break;
	case REMOVED:
		display_tcl_string("REM");
		break;
	case UNSCORED:
		display_tcl_string("UNS");
		break;
	default:
		display_tcl_string(NULL);
		break;
	}
}

static
void
display_tcl_string(char *s)
{
	fprintf(MUC_map_history_file, "\"");
	if(s)
		while(*s)
		{
			if((*s == '$') || (*s == '\"')) /* " */
				fprintf(MUC_map_history_file, "\\%c", *s);
			else
				fprintf(MUC_map_history_file, "%c", *s);
			s++;
		}
	fprintf(MUC_map_history_file, "\"");
}

static
void
display_long(long n)
{
	fprintf(MUC_map_history_file, "%d", n);
}
