%{
/*
 *  Copyright 1997 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	@(#)markup.l	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      parser for NE/CO keys and responses
 *     
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)markup.l	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


static char *cur_docnum = NULL;
static char *cur_classname = NULL;
static long cur_instnum = 0;
static char *cur_slotname;

static Catalog_entry *cur_catalog_entry;
static Clas *cur_clas;
static Inst *cur_inst;
static Slot *cur_slot = NULL;
static Mfil *cur_mfil;
static Sfil *cur_sfil;

static TABLE *doc_section_stack;
static TABLE *cur_inst_stack;
static TABLE *all_doc_insts;

static Class_def *cur_class_def;
static Slot_def *cur_slot_def;

static Bool in_document = False;
static Bool in_task_text = False;
static Bool in_docnum_text = False;

static long cur_doc_start;
static long cur_doc_finish;

/* in case we're using a version of flex that */
/* defines yywrap() as a macro...             */
#undef yywrap
int yywrap();
void yyerror(const char *msg);


/* local functions */

static Bool is_task_tag( char *lexeme );
static Bool is_doc_tag( char *lexeme );
static Bool is_docnum_tag( char *lexeme );
static Bool is_doc_section_tag( char *lexeme );

static void finish_cur_docnum();
static void insert_all_insts_into_cur_entry();
static void start_cur_inst( char *lexeme );
static void finish_cur_inst( char *lexeme );

static void start_cur_slot(char *lexeme );
static void parse_cur_mfil_quoted(char *lexeme);
static void parse_cur_mfil_unquoted(char *lexeme);
static void parse_sfil(char *tokenstart, char *bufptr);

static void add_text_to_cur_docnum( char *lexeme );
static void add_text_to_cur_insts( char *lexeme );
static void add_text_to_inst_text_slot(Inst *inst, char *lexeme);

static void push_doc_section( char *lexeme );
static void pop_doc_section( char *lexeme );


%}

%s TASKOPEN TASKCLOSE

%%
%{

/* initialize tables  */

doc_section_stack = table_list_create(cstring_compare,
				      default_release,
				      default_destroy);

cur_inst_stack = table_list_create(inst_compare,
				   NULL,
				   default_destroy);

all_doc_insts = table_list_create(inst_compare,
				  NULL,
				  default_destroy);
%}
%{

   /* SGML open tag beginning */

%}
<INITIAL>"<"[^ /\t>]*	          {

	if( is_task_tag( yytext + 1 ) )
	{
		start_cur_inst( yytext + 1 );  /* pushes inst onto inst stack*/
		in_task_text = False;
		MUC_cur_task_charno += yyleng;
		BEGIN TASKOPEN;
	}
	else 
	{
		if( is_doc_tag( yytext + 1 ) )    /* start of new document */
		{
			cur_doc_start = MUC_cur_text_charno + MUC_cur_task_charno;
			in_document = True;
		}

		if( is_docnum_tag( yytext + 1 ) )
		{
			UFREE(cur_docnum);
			in_docnum_text = True;
		}

		if( is_doc_section_tag( yytext + 1 ) )
			push_doc_section( yytext + 1 );

		if( in_task_text )
			add_text_to_cur_insts( yytext );

		MUC_cur_text_charno += yyleng;
	}
}
%{

   /* SGML close tag beginning */

%}
<INITIAL>"</"[^ \t>]*                  {

	if( is_task_tag( yytext + 2 ) )
	{
		finish_cur_inst( yytext + 2 );  /* pops inst from inst stack */
		MUC_cur_task_charno += yyleng;
		BEGIN TASKCLOSE;
	}
	else 
	{
		if( is_docnum_tag( yytext + 2 ) )
		{
			in_docnum_text = False;
			finish_cur_docnum(); /* cleans up cur_docnum */
			printf("%s ", cur_docnum);
			fflush(stdout);
		}

		if( is_doc_section_tag( yytext + 2 ) ) /* end of doc section */
			pop_doc_section( yytext + 2 );

		if( in_task_text )
			add_text_to_cur_insts( yytext );

		if( is_doc_tag( yytext + 2 ) )     /* end of document! */
		{
			cur_doc_finish = MUC_cur_text_charno + MUC_cur_task_charno + 1;
			in_document = False;
			insert_all_insts_into_cur_entry();
		}

		MUC_cur_text_charno += yyleng;
	}
}
%{

   /* anything with no open/close angle brackets (or newlines) */

%}
<INITIAL>[^<>\n]*            {

		if( in_docnum_text )
			add_text_to_cur_docnum( yytext );
		
		if( in_task_text )
			add_text_to_cur_insts( yytext );

		MUC_cur_text_charno += yyleng;
}
%{

   /* single close angle in INITIAL state */

%}
<INITIAL>">"     {

		if( in_docnum_text )
			add_text_to_cur_docnum( yytext );
		
		if( in_task_text )
			add_text_to_cur_insts( yytext );

                MUC_cur_text_charno += yyleng;
}
%{

   /* newline in INITIAL state */

%}
<INITIAL>\n               {

	if( in_docnum_text )
		add_text_to_cur_docnum( " " ); /* change newline to space */
		
	if( in_task_text )
		add_text_to_cur_insts( " " );
	MUC_cur_text_charno++;
	MUC_cur_lineno++;
}
%{

  /* whitespace in a task open tag */

%}
<TASKOPEN>[ \t]*               {

	MUC_cur_task_charno += yyleng;
}
%{

   /* attribute (slot) name in task open tag */

%}
<TASKOPEN>[^ \t=<>]*"="     {

	MUC_cur_task_charno += yyleng;
	start_cur_slot(yytext);
}
%{

   /* unquoted attribute value (mfils) in task open tag        */
   /* (could be attribute name, except no equals sign at end)  */

%}
<TASKOPEN>[^ \t=\"\'<>]*       { 	/* " */

	MUC_cur_task_charno += yyleng;
	if( cur_slot )
	{
		parse_cur_mfil_unquoted(yytext);
		cur_slot = NULL;
	}
	else
	{
		fprintf(stderr,
			"Parse error in tag attribute, line %d\n",
			MUC_cur_lineno,
			yytext);
		exit(1);
	}
}
%{

   /* quoted attribute value (mfils) in task open tag */

%}
<TASKOPEN>\"[^\"]*\"     |       
<TASKOPEN>\'[^\']*\'       {   	/* " */

	MUC_cur_task_charno += yyleng;
	if( cur_slot )
		parse_cur_mfil_quoted(yytext);
}
%{

   /* close angle bracket in task open tag */

%}
<TASKOPEN>">"       {

	MUC_cur_task_charno += yyleng;
	in_task_text = True;
	BEGIN INITIAL;
}
%{

   /* anything else in task open tag */

%}
<TASKOPEN>.       {

	fprintf(stderr,
		"syntax error in SGML task tag, line %d",
		MUC_cur_lineno);
	exit(1);
}
%{

   /* everything upto and including close of task close tag */

%}
<TASKCLOSE>[^>]*">"       {

	MUC_cur_task_charno += yyleng;

	/* we've already popped the inst from the stack, so... */
	if( table_count(cur_inst_stack) == 0 )
		in_task_text = False;
	else
		in_task_text = True;
	BEGIN INITIAL;
}
%{
	/* anything else in task close tag - missing a tag close */
%}
<TASKCLOSE>.    {

	fprintf(stderr,
		"Incomplete task close tag, line %d",
		MUC_cur_lineno);
	exit(1);
}

%%

/**************************************************/
/**************************************************/

static
Bool
is_task_tag( char *lexeme )
{
	if( class_def_find( MUC_Class_defs, lexeme ))
		return True;
	else
		return False;
}

/**************************************************/
/**************************************************/

static
Bool
is_doc_tag( char *lexeme )
{
	char *doc_tag_string = option_find_first_value(MUC_Options, 
						       ":sgml_DOC_gid");

	if( STRCASEEQ( doc_tag_string, lexeme ) )
		return True;
	else
		return False;
}

/**************************************************/
/**************************************************/

static
Bool
is_docnum_tag( char *lexeme )
{
	char *docnum_tag_string = option_find_first_value(MUC_Options,
							  ":sgml_DOCNUM_gid");
	if( STRCASEEQ( docnum_tag_string, lexeme ) )
		return True;
	else
		return False;
}

/**************************************************/
/**************************************************/

static
Bool
is_doc_section_tag( char *lexeme )
{
	TABLE *doc_sections = option_find_values(MUC_Options,
						 ":doc_sections");
	if( table_find( lexeme, doc_sections ) )
		return True;
	else
		return False;
}

/**************************************************/
/**************************************************/

static
void
add_text_to_cur_docnum( char *lexeme )
{
	if( lexeme == NULL )  /* should never happen */
	{
		fprintf(stderr,
			"add_text_to_cur_docnum: NULL text, line %d\n",
			MUC_cur_lineno);
		exit(1);
	}

	if( cur_docnum == NULL )
		cur_docnum = STRALLOC(lexeme);
	else
	{
		cur_docnum = 
			(char *)realloc((void *)cur_docnum, 
					strlen( cur_docnum ) + 
					strlen( lexeme ) + 1 );
		strcat( cur_docnum, lexeme );
	}
}

static
void
finish_cur_docnum()
{
	char *c;     /* holds cur_docnum, temporarily */

	c = nice_docnum( cur_docnum );
	UFREE( cur_docnum );
	cur_docnum = c;
}


/**************************************************
 *
 * insert_all_insts_into_cur_entry
 *   transfer all inst's from the all_doc_insts to the 
 *   appropriate class of the current catalog entry, and
 *   reset tables.
 *
 **************************************************/

static
void
insert_all_insts_into_cur_entry()
{
	Inst *inst;
	Clas *clas;
	TABLE *classes;

	if( cur_docnum == NULL )
	{
		fprintf(stderr,
			"Document number element missing from document\n");
		exit(1);
	}
		
	/* get entry from catalog */
	cur_catalog_entry = catalog_entry_find(MUC_Catalog, cur_docnum);
	if( cur_catalog_entry == NULL )
	{
		cur_catalog_entry = catalog_entry_create(cur_docnum);
		table_append((char *)cur_catalog_entry, MUC_Catalog);
	}

	/* set class list, based on whether parsing a key or response file */
	if( MUC_key_or_response == KEY )
		classes = cur_catalog_entry->key_classes;
	else
		classes = cur_catalog_entry->response_classes;
	
	/* take insts from all_doc_insts, add some final info, */
	/* and put them in the right class                     */
	table_first(all_doc_insts);
	for(inst = (Inst *)table_next(all_doc_insts);
	    inst;
	    inst = (Inst *)table_next(all_doc_insts))
	{
		/* put proper document number in inst */
		UFREE( inst->docnum );
		inst->docnum = STRALLOC(cur_catalog_entry->docnum);

		/* put inst in its class */
		clas = clas_find( classes, inst->name );
		table_append((char *)inst, clas->insts);
	}

	/* reset the "helper" tables -- shouldn't hurt insts */
	table_destroy( doc_section_stack );
	doc_section_stack = table_list_create(cstring_compare,
					      default_release,
					      default_destroy);

	table_destroy( cur_inst_stack );
	cur_inst_stack = table_list_create(inst_compare,
					   NULL,
					   default_destroy);

	table_destroy( all_doc_insts ); 
	all_doc_insts = table_list_create(inst_compare,
					  NULL,
					  default_destroy);

	cur_catalog_entry = NULL;
	cur_instnum = 0;
}

/*************************************************
 *
 * start_cur_inst -- since some insts may come before
 *     the document number in the text (not in current
 *     texts, but maybe in the future),  delay putting
 *     the instances in the catalog until the end of the
 *     document, when the docnum has (surely!) been found.
 *
 **************************************************/

static
void
start_cur_inst( char *lexeme )
{
	char instnum[BUFLENGTH];
	char *doc_section;
	
	/* are we in a doc section that we're interested in? */
	table_first(doc_section_stack);
	doc_section = table_next(doc_section_stack);
	if( doc_section )
	{
		cur_class_def = class_def_find( MUC_Class_defs, lexeme );
		if(  cur_class_def == NULL )
		{
			fprintf(stderr,
				"Unknown instance type: \'%s\', line %d",
				lexeme,
				MUC_cur_lineno);
			exit(1);
		}
		
		sprintf(instnum, "%d", cur_instnum++ );
		cur_inst = inst_create( lexeme, "", instnum );
		if( MUC_key_or_response == KEY )
			cur_inst->is_key = True;
		cur_inst->its_class_def = cur_class_def;
		cur_inst->is_instantiated = True;
		cur_inst->start_offset = MUC_cur_text_charno;
		cur_inst->doc_section = STRALLOC(doc_section);

		table_append((char *) cur_inst, all_doc_insts );
		table_prepend((char *)cur_inst, cur_inst_stack);
	}
}

/**************************************************/
/**************************************************/

static
void
finish_cur_inst( char *lexeme )
{
	Inst *inst;
	char *doc_section;
	
	/* inst was parsed only if it's in an interesting doc section */
	table_first(doc_section_stack);
	doc_section = table_next(doc_section_stack);
	if(doc_section)
	{
		/* get top inst of stack */
		table_first(cur_inst_stack);
		inst = (Inst *)table_next(cur_inst_stack);
		if( (inst == NULL) || !STRCASEEQ(lexeme,inst->name) )
		{
			/* empty stack or the wrong inst at top of stack */
			fprintf(stderr,
				"Unexpected \'%s\' close tag, line %d\n",
				lexeme,
				MUC_cur_lineno);
			exit(1);
		}

		/* set cur_inst */
		cur_inst = (Inst *)table_next(cur_inst_stack);
		
		/* record where in the text the inst text ends */
		inst->end_offset = MUC_cur_text_charno;

		/* ensure there's a TEXT slot in the inst */
		add_text_to_cur_insts("");

		/* "pop" the inst from the stack */
		table_delete((char *)inst, cur_inst_stack);
	}
}


/**************************************************/
/**************************************************/

static
void
add_text_to_cur_insts(char *lexeme)
{
	Inst *inst;
	table_first(cur_inst_stack);
	for(inst=(Inst *)table_next(cur_inst_stack);
	    inst;
	    inst=(Inst *)table_next(cur_inst_stack))
		add_text_to_inst_text_slot(inst, lexeme);
}

/**************************************************/
/**************************************************/

static
void
add_text_to_inst_text_slot(Inst *inst, char *lexeme)
{
	char *slotname;
	Slot *slot;
	Mfil *mfil;
	Sfil *sfil;
	Slot_def *slot_def;

	/* Get the name of the slot into which text will go. (For  */
	/* now, use just one option for all types of instance)   */
	slotname = option_find_first_value(MUC_Options,
					   ":sgml_TEXT_slot");
	slot_def = 
		slot_def_find(inst->its_class_def->slot_defs,
			      slotname);

	/* find the text slot in the inst, or create it */
	slot = slot_find(inst->slots, slotname);
	if(slot == NULL)
	{
		slot = slot_create(slotname);
		slot->its_slot_def = slot_def;
		table_append((char *)slot, inst->slots);
	}
	
	/* find the first mfil in the slot, or create it */
	table_first(slot->mfils);
	mfil = (Mfil *)table_next(slot->mfils);
	if(mfil == NULL)
	{
		mfil = mfil_create();
		table_append((char *)mfil, slot->mfils);
	}
	
	/* find the first sfil in the mfil, or create it.   */
	/* (for now, it will always be of type STRING_FILL) */
	table_first(mfil->sfils);
	sfil = (Sfil *)table_next(mfil->sfils);
	if(sfil == NULL)
	{
		sfil = sfil_create(STRING_FILL);
		sfil->fill.str->fill = STRALLOC(lexeme);
		sfil->fill.str->start_offset = MUC_cur_text_charno;
		sfil->fill.str->end_offset = MUC_cur_text_charno + strlen(lexeme) - 1;
		table_append((char *)sfil, mfil->sfils);
	}
	else if( sfil->fill.str->fill == NULL )
	{
		fprintf(stderr,
			"add_text_to_inst_text_slot: NULL string fill\n");
		exit(0);
        }		
	else
	{
		sfil->fill.str->fill = 
			(char *)realloc((void *)sfil->fill.str->fill, 
					strlen( sfil->fill.str->fill ) + 
					strlen(lexeme) + 1 );
		strcat( sfil->fill.str->fill, lexeme );
		sfil->fill.str->end_offset += strlen(lexeme);		
	}
}

/**************************************************/
/**************************************************/

static
void
start_cur_slot( char *lexeme )
{
	UFREE( cur_slotname );
	cur_slotname = STRALLOC( lexeme );
	cur_slotname[strlen(cur_slotname)-1] = '\0';  /* chop off '=' */
	
	/* If we're not in a good doc section, cur_inst will be null. */
	/* Just return. */
	if( cur_inst == NULL )
		return;

	if( cur_class_def == NULL )
	{
		fprintf(stderr,
			"[start_cur_slot]:current class def NULL, line %d\n",
			MUC_cur_lineno);
		exit(1);
	}

	/* find current slot definition */
	cur_slot_def = slot_def_find( cur_class_def->slot_defs, cur_slotname );
	if( cur_slot_def == NULL )
	{
		fprintf(stderr,
			"[start_cur_slot]:Unknown slot, \'%s\' ",
			cur_slotname);
		fprintf(stderr,
			"in class \'%s\', line %d\n",
			cur_class_def->name,
			MUC_cur_lineno);
		exit(1);
	}

	/* check if slot has already been filled */
	cur_slot = slot_find(cur_inst->slots, cur_slotname);
	if(cur_slot)
	{
		fprintf(stderr,
			"WARNING: redefining attribute \'%s\', line %d\n",
			cur_slotname,
			MUC_cur_lineno);
		table_delete((char *)cur_slot, cur_inst->slots);
	}
	cur_slot = slot_create(cur_slotname);
	cur_slot->its_slot_def = cur_slot_def;
	table_append((char *)cur_slot, cur_inst->slots);
}

/**************************************************/
/**************************************************/

static
void
parse_cur_mfil_quoted(char *lexeme)
{
	char *trimmed_string;
	
	trimmed_string = trim_quotes(lexeme);
	parse_cur_mfil_unquoted(trimmed_string);
	UFREE(trimmed_string);
}


/**************************************************/
/**************************************************/

static
void
parse_cur_mfil_unquoted(char *lexeme)
{
	char *mfil_sep_chars;    /* separator char got from options */
	char *bufptr = lexeme;
	char *tokenstart;
	
	if( cur_slot == NULL )
		return;
	
	mfil_sep_chars =
		option_find_first_value(MUC_Options,
					":sgml_alternative_separator");

	while( *bufptr )
	{
		tokenstart = bufptr;        /* save start of mfil  */
		                            /* go past end of mfil */
		bufptr += strcspn( bufptr, mfil_sep_chars );
		
		cur_mfil = mfil_create();   /* create mfil and insert */
		table_append((char *)cur_mfil, cur_slot->mfils );

		parse_sfil(tokenstart,bufptr);  /* insert sfil in mfil */
		if( *bufptr )                   /* go past separator */
			bufptr++;
	}
}

	

/**************************************************/
/**************************************************/

static
void
parse_sfil(char *tokenstart, char *tokenfinish)
{
	if( cur_mfil == NULL )
		return;
	
	switch( slot_def_which_type(cur_slot_def) ){	
	case STRING_FILL:
		cur_sfil = sfil_create(STRING_FILL);
		cur_sfil->fill.str->fill = alloc_between(tokenstart,
							 tokenfinish);
		cur_sfil->fill.str->start_offset = 0;
		cur_sfil->fill.str->end_offset = 0;
		break;
	case SET_FILL:
		cur_sfil = sfil_create(SET_FILL);
		cur_sfil->fill.set->fill = alloc_between(tokenstart,
							 tokenfinish);
		cur_sfil->fill.set->start_offset = 0;
		cur_sfil->fill.set->end_offset = 0;
		break;
	case POINTER_FILL:
		cur_sfil = sfil_create(POINTER_FILL);
		cur_sfil->fill.ptr->string_rep = alloc_between(tokenstart,
							       tokenfinish);
		cur_sfil->fill.ptr->start_offset = 0;
		cur_sfil->fill.ptr->end_offset = 0;
		break;
	default:
		fprintf(stderr,
			"Unknown slot fill type, line %d\n",
			MUC_cur_lineno);
		exit(1);
	}
	table_append((char *)cur_sfil, cur_mfil->sfils);
}

/**************************************************/
/**************************************************/

static
void
push_doc_section( char *lexeme )
{
	table_prepend(STRALLOC(lexeme),
		      doc_section_stack);
}

/**************************************************/
/**************************************************/

static
void
pop_doc_section( char *lexeme )
{
	char *top_doc_section;
	
	table_first( doc_section_stack );
	top_doc_section = table_next(doc_section_stack);

	/* check for unclosed document section tags */
	while( top_doc_section && !STRCASEEQ(top_doc_section, lexeme))
	{
		fprintf(stderr,
			"WARNING:no \'%s\' close tag before \'%s\', line %d\n",
			top_doc_section,
			lexeme,
			MUC_cur_lineno);
		table_delete(top_doc_section, 
			     doc_section_stack);
		table_first( doc_section_stack );
		top_doc_section = table_next(doc_section_stack);
	}
	
	/* either the document section stack is emptied out, or we've */
	/* found the matching close section tag */
	if( top_doc_section )
		table_delete(top_doc_section, 
			     doc_section_stack);
	else
		fprintf(stderr,
			"WARNING:Unbalanced SGML tag, line %d\n",
			MUC_cur_lineno);
}



/**************************************************/
/**************************************************/

/*
 * yyerror 
 */

void
yyerror(const char *msg)
{
    printf("(markup) %d: %s at '%s'\n", MUC_cur_lineno, msg, yytext);
}

/* 
 * yywrap -- what to do at the end of the file
 */

int
yywrap(void)
{
	if( in_document )
		fprintf(stderr,
			"WARNING: unexpected end of file, line %d\n",
			MUC_cur_lineno);
	UFREE(cur_docnum);
	UFREE(cur_classname);
	UFREE(cur_slotname);
	table_destroy(doc_section_stack);
	table_destroy(cur_inst_stack);
	table_destroy(all_doc_insts);
	return(1);
}
