/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *	@(#)massage_CO.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      stuff to do after parsing but before aligning keys and responses
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)massage_CO.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void adjust_inst_IDs_in_entry( Catalog_entry *entry );
static void adjust_inst_IDs_in_classes( TABLE *classes );
static void adjust_inst_IDs_in_insts( TABLE *insts );

static void make_missing_MIN_strings_in_classes( TABLE *classes );
static void make_missing_MIN_strings_in_insts( TABLE *insts );
static Slot *text_slot_copy( Slot *slot, char *slotname );

static void calc_REF_slots_in_entry(Catalog_entry *entry);
static void calc_REF_slots_in_classes(TABLE *classes);
static void calc_REF_slots_in_insts( TABLE *insts );

void
massage_CO_entry(Catalog_entry *entry)
{
	adjust_inst_IDs_in_entry( entry );
	calc_REF_slots_in_entry( entry );

	change_cr_to_spc_in_entry( entry );
	trim_TEXT_whitespace_in_entry( entry  );
	calc_substring_offsets_in_entry( entry );

	make_missing_MIN_strings_in_classes( entry->key_classes );

	check_status_slots_in_entry( entry );
	make_clean_stringfills_entry(entry);
}

static
void
adjust_inst_IDs_in_entry( Catalog_entry *entry )
{
	adjust_inst_IDs_in_classes( entry->key_classes );
	adjust_inst_IDs_in_classes( entry->response_classes );
}

static
void
adjust_inst_IDs_in_classes( TABLE *classes )
{
	Clas *clas;

	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		adjust_inst_IDs_in_insts( clas->insts );
}
	
static
void
adjust_inst_IDs_in_insts( TABLE *insts )
{
	Inst *inst;
	char *id_string;
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		id_string = STRALLOC(get_inst_ID_slot_fill(inst));
		UFREE(inst->id);
		inst->id = id_string;
	}
}


/*
 *****************************************
 * *****************************************
 * *****************************************
 * *****************************************
 * *****************************************
 */

static
void
make_missing_MIN_strings_in_classes( TABLE *classes )
{
	Clas *clas;

	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		make_missing_MIN_strings_in_insts( clas->insts );
}

static
void
make_missing_MIN_strings_in_insts( TABLE *insts )
{
	Inst *inst;
	Slot *text_slot;
	Slot *min_slot;
	char *text_slotname;
	char *min_slotname;
	Class_def *class_def;
	Slot_def *slot_def;
	
	text_slotname = option_find_first_value(MUC_Options, 
						":sgml_TEXT_slot");
	min_slotname  = option_find_first_value(MUC_Options, 
						":sgml_MIN_slot");
	table_first(MUC_Class_defs);
	class_def = (Class_def *)table_next(MUC_Class_defs);
	slot_def = slot_def_find(class_def->slot_defs,
				 min_slotname);

	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		text_slot = slot_find(inst->slots, text_slotname);
		min_slot = slot_find(inst->slots, min_slotname);
	
		if( min_slot ) continue;
	
		min_slot = text_slot_copy(text_slot, min_slotname);
		min_slot->its_slot_def = slot_def;
		table_append( (char *) min_slot, inst->slots);
	}
}

static
Slot *
text_slot_copy( Slot *slot, char *slotname )	
{
	Slot *newslot;
	Mfil *mfil, *newmfil;
	Sfil *sfil, *newsfil;
	Stringfill *stringfill, *newstringfill;
	
	table_first( slot->mfils );
	mfil = (Mfil *) table_next(slot->mfils);
	table_first( mfil->sfils );
	sfil = (Sfil *) table_next(mfil->sfils);
	stringfill = sfil->fill.str;
	
	newslot = slot_create( slotname );
	newmfil = mfil_create();
	newsfil = sfil_create(STRING_FILL);

	newstringfill = newsfil->fill.str;
	newstringfill->fill = STRALLOC(stringfill->fill);
	newstringfill->start_offset = stringfill->start_offset;
	newstringfill->end_offset = stringfill->end_offset;
	
	table_append((char *)newsfil, newmfil->sfils);
	table_append((char *)newmfil, newslot->mfils);
	
	return newslot;
}

/*
 *****************************************
 * *****************************************
 * *****************************************
 * *****************************************
 * *****************************************
 */

static
void
calc_REF_slots_in_entry(Catalog_entry *entry)
{
	calc_REF_slots_in_classes(entry->key_classes);
	calc_REF_slots_in_classes(entry->response_classes);
}

static
void
calc_REF_slots_in_classes(TABLE *classes)
{
	Clas *clas;
	
	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		calc_REF_slots_in_insts( clas->insts );
}

static
void
calc_REF_slots_in_insts( TABLE *insts )
{
	Inst *inst;
	TABLE *refmaps = table_hash_create(refmap_compare,
					   refmap_release,
					   default_destroy,
					   refmap_hash,
					   4);
	Refmap *refmap;
	Pointerfill *pointerfill;
	char *new_stringrep;

	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		refmap = refmap_create(get_inst_ID_slot_fill(inst), inst);
		table_add((char *)refmap, refmaps);
	}
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		pointerfill = get_inst_REF_slot_fill(inst);
		if( pointerfill )
		{
			pointerfill->fill = 
				refmap_find_inst(refmaps, 
						 pointerfill->string_rep);
			new_stringrep = (char *)calloc(5 + 
						       strlen(inst->name) +
						       strlen(inst->docnum) +
						       strlen(pointerfill->string_rep),
						       sizeof(char *));
			sprintf(new_stringrep, 
				"<%s-%s-%s>",
				inst->name,
				inst->docnum,
				pointerfill->string_rep);
			UFREE(pointerfill->string_rep);
			pointerfill->string_rep = new_stringrep;
		}
	}
	table_destroy(refmaps);
}

char *
get_inst_ID_slot_fill(Inst *inst)
{
	Slot *slot;
	char *slotname;
	Mfil *mfil;
	Sfil *sfil;
	
	slotname = option_find_first_value(MUC_Options, ":sgml_ID_slot");
	if(slotname == NULL) return NULL;
	if(inst == NULL) return NULL;
	if( inst->slots == NULL ) return NULL;
	slot = slot_find(inst->slots, slotname);
	if(slot == NULL) return NULL;
	if(slot->mfils == NULL) return NULL;
	table_first(slot->mfils);
	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	table_first(mfil->sfils);
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	
	if( sfil->type != SET_FILL ) return NULL;
	if( sfil->fill.set == NULL) return NULL;
	return sfil->fill.set->fill;
}

Pointerfill *
get_inst_REF_slot_fill(Inst *inst)
{
	Slot *slot;
	char *slotname;
	Mfil *mfil;
	Sfil *sfil;
	
	slotname = option_find_first_value(MUC_Options, ":sgml_REF_slot");
	if(slotname == NULL) return NULL;
	if(inst == NULL) return NULL;
	if( inst->slots == NULL ) return NULL;
	slot = slot_find(inst->slots, slotname);
	if(slot == NULL) return NULL;
	if(slot->mfils == NULL) return NULL;
	table_first(slot->mfils);
	mfil = (Mfil *)table_next(slot->mfils);
	if( mfil == NULL ) return NULL;
	table_first(mfil->sfils);
	sfil = (Sfil *)table_next(mfil->sfils);
	if( sfil == NULL ) return NULL;
	
	if( sfil->type != POINTER_FILL ) return NULL;
	return sfil->fill.ptr;
}


