/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *      @(#)massage_IE.c	6.1\t09/08/97
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	functions common to the ST, TR, and TE tasks which 
 *      preprocess data before alignment/scoring
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)massage_IE.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void make_clean_stringfills_classes(TABLE *classes);
static void make_clean_stringfills_insts(TABLE *insts);
static void make_clean_stringfills_slots(TABLE *slots);
static void make_clean_stringfills_mfils(TABLE *mfils);
static void make_clean_stringfills_sfils(TABLE *sfils);
static void make_clean_stringfill(Stringfill *stringfill);
static char *trim_premodifiers(char *src);
static char *trim_postmodifiers(char *src);
static char *trim_designators(char *src);

/********
 ********
 ********
 */

void
make_clean_stringfills_entry(Catalog_entry *entry)
{
	make_clean_stringfills_classes(entry->key_classes);
	make_clean_stringfills_classes(entry->response_classes);
}

static
void
make_clean_stringfills_classes(TABLE *classes)
{
	Clas *clas;
	
	table_first(classes);
	for(clas=(Clas *)table_next(classes);
	    clas;
	    clas=(Clas *)table_next(classes))
		make_clean_stringfills_insts(clas->insts);
}

static
void
make_clean_stringfills_insts(TABLE *insts)
{
	Inst *inst;
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
		make_clean_stringfills_slots(inst->slots);
}

static
void
make_clean_stringfills_slots(TABLE *slots)
{
	Slot *slot;
	
	table_first(slots);
	for(slot=(Slot *)table_next(slots);
	    slot;
	    slot=(Slot *)table_next(slots))
		if( !slot_is_unscored(slot) )
			make_clean_stringfills_mfils(slot->mfils);
}

static
void
make_clean_stringfills_mfils(TABLE *mfils)
{
	Mfil *mfil;
	
	table_first(mfils);
	for(mfil=(Mfil *)table_next(mfils);
	    mfil;
	    mfil=(Mfil *)table_next(mfils))
		make_clean_stringfills_sfils(mfil->sfils);
}

static
void
make_clean_stringfills_sfils(TABLE *sfils)
{
	Sfil *sfil;
	
	table_first(sfils);
	for(sfil=(Sfil *)table_next(sfils);
	    sfil;
	    sfil=(Sfil *)table_next(sfils))
		if( sfil->type == STRING_FILL )
			make_clean_stringfill(sfil->fill.str);
}

static
void
make_clean_stringfill(Stringfill *stringfill)
{
	char *s1, *s2, *s3, *s4, *s5;
	UFREE( stringfill->clean_fill );
	UFREE( stringfill->straight_fill );
	
	s1 = string_lower(stringfill->fill);
	s2 = trim_intertoken_whitespace(s1);
	s3 = trim_premodifiers(s2);
	s4 = trim_postmodifiers(s3);
	s5 = trim_designators(s4);

	stringfill->straight_fill = s2;  
	stringfill->clean_fill = s5;

	UFREE(s1);
	UFREE(s3);
	UFREE(s4);
}

static
char *
trim_premodifiers(char *src)
{
	char *buf;
	char *result;
	TABLE *premods;
	char *premod, *start, *end, *pos;
	Bool found=False;

	buf = STRALLOC(src);
	premods = option_find_values(MUC_Options, 
				     ":premodifiers");

	do
	{
		found=False;
		table_first(premods);
		for(premod=table_next(premods);
		    premod;
		    premod=table_next(premods))
			if((strstr(buf, premod) == buf)
			   &&
			   isspace(*(buf+strlen(premod))))
			{
				found=True;
				start=buf+strlen(premod)+1;
				end=buf+strlen(buf);
				muc_memmove(buf, start, (end-start)+1);
			}
	}
	while( found );

	return buf;
}

/* 
   trim_postmodifiers -- change all postmodifiers to spaces, then 
   trim the intertoken whitespace
 */

static
char *
trim_postmodifiers(char *src)
{
	char *s1, *s2;
	char *postmod;
	char *pos;
	int i;
	TABLE *postmods;
	
	postmods = option_find_values(MUC_Options,
				      ":postmodifiers");
	s1 = STRALLOC(src);
	
	table_first(postmods);
	for(postmod=table_next(postmods);
	    postmod;
	    postmod=table_next(postmods))
	{
		pos=strstr(s1, postmod);
		while(pos)
		{
			for(i=0;i<strlen(postmod);i++)
			{
				(*pos)=' ';
				pos++;
			}
			pos=strstr(pos, postmod);
		}
	}
	s2 = trim_intertoken_whitespace(s1);
	UFREE(s1);
	return s2;
}


static
char *
trim_designators(char *src)
{
	char *buf;
	TABLE *designators;
	char *designator, *start, *end, *pos, *pos2;
	Bool found=False;

	buf = STRALLOC(src);
	designators = 
		option_find_values(MUC_Options, 
				   ":corporate_designators");
	do
	{
		found=False;
		table_first(designators);
		for(designator=table_next(designators);
		    designator;
		    designator=table_next(designators))
		{
			pos=strstr(buf, designator);
			if( pos == NULL )
				continue;
			if(((pos == buf)
			    ||
			    *(pos-1) == ' ')
			   &&
			   ((*(pos+strlen(designator)) == '\0')
			    ||
			    (*(pos+strlen(designator)) == ' ')))
			{
				found=True;
				if(*(pos+strlen(designator)) == '\0')
				{
					start=pos+strlen(designator);
					if(pos == buf)
						pos2=pos;
					else	
						pos2=pos-1;
				}
				else
				{
					start=pos+strlen(designator)+1;
					pos2=pos;
				}
				end=buf+strlen(buf);
				muc_memmove(pos2, start, (end-start)+1);
			}
		}
	}
	while( found );
	
	return STRALLOC(buf);
}
