/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *      @(#)massage_ST.c	6.1\t09/08/97
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	functions to preprocess ST data before alignment/scoring
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)massage_ST.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void mark_implicit_ST_optionals_entry(Catalog_entry *entry);
static void mark_implicit_ST_optionals_classes(TABLE *classes);
static void mark_implicit_ST_optionals_insts(TABLE *insts);
static void mark_implicit_ST_optionals_slots(TABLE *slots);
static void mark_implicit_ST_optionals_mfils(TABLE *mfils);
static void mark_implicit_ST_optionals_sfils(TABLE *sfils);

static void mfil_pointer_and(TABLE *sfils, TABLE *mfils);
static void init_cbo_flags(TABLE *classes, TABLE *reverse_map_order);

static void find_identical_insts_entry(Catalog_entry *entry);
static void find_identical_insts_classes(TABLE *classes);
static void find_identical_insts_insts(TABLE *insts);
static Bool insts_are_equatable(Inst *inst1, Inst *inst2);
static long num_scored_slots(TABLE *slots);
static Bool slots_are_equatable(TABLE *slots1, TABLE *slots2);
static Bool mfils_are_equatable(TABLE *mfils1, TABLE *mfils2);
static Bool sfils_are_equatable(TABLE *sfils1, TABLE *sfils2);
static Bool sfils_are_equal(Sfil *sfil1, Sfil *sfil2);
static void make_equivalent(Inst *inst1, Inst *inst2);
static Inst *find_inst_rep(Inst *inst);

/********
 ********
 ********
 */

void
massage_ST_entry(Catalog_entry *entry)
{
	check_status_slots_in_entry(entry);
	if( IS_K2R )
		mark_implicit_ST_optionals_entry(entry);
	find_identical_insts_entry(entry);
	make_clean_stringfills_entry(entry);
}

static
void
mark_implicit_ST_optionals_entry(Catalog_entry *entry)
{
	if( entry == NULL )
	{
		fprintf(stderr,
			"\nWARNING: null catalog entry.\n");
		return;
	}

	mark_implicit_ST_optionals_classes(entry->key_classes);
}

static
void
mark_implicit_ST_optionals_classes(TABLE *classes)
{
	TABLE *map_order;
	TABLE *reverse_map_order;
	char *clas_name;
	Clas *clas;
	
	map_order = option_find_values(MUC_Options, ":map_order");
	reverse_map_order = table_list_create(cstring_compare,
					      NULL, 
					      default_destroy);
	table_reverse(reverse_map_order, map_order);
	
	/* cbo - "could be optional" */
	init_cbo_flags(classes, reverse_map_order);
	
	table_first(reverse_map_order);
	for(clas_name=table_next(reverse_map_order);
	    clas_name;
	    clas_name=table_next(reverse_map_order))
	{
		clas = clas_find(classes, clas_name );
		if( clas )
			mark_implicit_ST_optionals_insts( clas->insts );
	}
	table_destroy(reverse_map_order);
}

static
void
mark_implicit_ST_optionals_insts( TABLE *insts )
{
	Inst *inst;
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		if(!(inst->is_optional) && 
		   inst->could_be_optional )
		{
			fprintf(stderr,
				"\nWARNING: making <%s-%s-%s> optional\n",
				inst->name,
				inst->docnum,
				inst->id);
			inst->is_optional = True;
		}

		/* if optional, this inst can't make another */
		/* inst non-optional by pointing at it */
		if( inst->is_optional )
			; 
		else
			mark_implicit_ST_optionals_slots( inst->slots );
	}
}

static
void
mark_implicit_ST_optionals_slots( TABLE *slots )
{
	Slot *slot;
	
	table_first(slots);
	for(slot=(Slot *)table_next(slots);
	    slot;
	    slot=(Slot *)table_next(slots))
	{
		/* if optional, this slot can't make another */
		/* inst optional by pointing at it */
		if( slot->is_optional )
			;
		else
			mark_implicit_ST_optionals_mfils( slot->mfils );
	}
}


static
void
mark_implicit_ST_optionals_mfils( TABLE *mfils )
{
	TABLE *sfils;

	/* a table of sfils which are in all mfil's in mfils */
	sfils = table_list_create(sfil_compare,
				  NULL,
				  default_destroy);

	mfil_pointer_and(sfils, mfils); 
	mark_implicit_ST_optionals_sfils( sfils );
	table_destroy(sfils);
}

static
void
mark_implicit_ST_optionals_sfils( TABLE *sfils )
{
	Sfil *sfil;
	
	table_first(sfils);
	for(sfil=(Sfil *)table_next(sfils);
	    sfil;
	    sfil=(Sfil *)table_next(sfils))
	{
		if((sfil->type == POINTER_FILL) && (sfil->fill.ptr) && (sfil->fill.ptr->fill))
			sfil->fill.ptr->fill->could_be_optional = False;
	}
}	

/* cbo -- "could be optional" -- the objects at the top of the
   scenario DAG (i.e., the template objects) should have their
   could_be_optional flags set to the same as their is_optional flags.
   The other objects (all of which have pointers pointing to them)
   should have their could_be_optional flags initialized to "True"
   (and later checking will clear these flags if some non-optional
   thing points to them).*/

static
void
init_cbo_flags(TABLE *classes, TABLE *reverse_map_order)
{
	char *clas_name;
	Clas *clas;
	Inst *inst;
	TABLE *insts;
	
	/* first set the "template" objects' flags */
	table_first(reverse_map_order);
	clas_name = table_next(reverse_map_order);
	
	if( clas_name == NULL )
	{
		fprintf(stderr,
			"ERROR: reverse of map order is empty.\n" );
		exit(1);
	}
	
	clas = clas_find(classes, clas_name);
	if( clas == NULL )
	{
		fprintf(stderr,
			"[init_cbo_flags]: class \'%s\' not found.\n",
			clas_name);
		exit(1);
	}

        insts = clas->insts;
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
		inst->could_be_optional = inst->is_optional;
		

	/* now set the flags of the lower objects to True */
	while( (clas_name = table_next(reverse_map_order)) )
	{
		clas = clas_find(classes, clas_name );
		if( clas == NULL )
		{
			fprintf(stderr,
				"ERROR: class \'%s\' not found.\n",
				clas_name);
			exit(1);
		}

		insts = clas->insts;
		table_first(insts);
		for(inst=(Inst *)table_next(insts);
		    inst;
		    inst=(Inst *)table_next(insts))
			inst->could_be_optional = True;
	}
}	

/* mfil_pointer_and -- "sfils" is an already-initialized table of 
   sfils.  This function finds the sfils that are in all lists
   of sfils in mfils, and puts them in the table "sfils" (the sfils
   must also be pointerfills).

   EXAMPLE: 

   mfils == ((a b c d)
	       (b   d)
	       (b c d)
	     (a b   d))

   then at the end, 

   sfils == (b d)

*/

static
void
mfil_pointer_and(TABLE *sfils, TABLE *mfils)
{
	Mfil *mfil;
	Sfil *sfil1, *sfil2;
	Bool found;
	TABLE *sfils_copy;

	/* copy the first mfil->sfils to sfils */
	table_first(mfils);
	mfil = (Mfil *)table_next(mfils);
	if( mfil == NULL ) return;
	table_concat(sfils, mfil->sfils);

	/* now go through a copy of the first list, and
	   see if each sfil is in all lists of sfils in 
	   mfils (if not, take that sfil out of sfils) */
	sfils_copy = table_list_create(sfil_compare,
				       NULL,
				       default_destroy);
	table_concat(sfils_copy, sfils);
	table_first(sfils_copy);
	for(sfil1=(Sfil *)table_next(sfils_copy);
	    sfil1;
	    sfil1=(Sfil *)table_next(sfils_copy))
	{
		if( sfil1->type != POINTER_FILL )
			continue;

		table_first(mfils);
		table_next(mfils); /* skip first mfil */
		while( 	(mfil = (Mfil *)table_next(mfils) ) )
		{
			found = False;
			table_first(mfil->sfils);
			while(!found && 
			      (sfil2=(Sfil *)table_next(mfil->sfils)) )
			{
				if(sfil2->type == POINTER_FILL
				   &&
				   (sfil2->fill.ptr->fill
				    == 
				    sfil1->fill.ptr->fill))
					found = True;
			}
			if( !found )
				table_delete((char *)sfil1, sfils);
		}
	}
	table_destroy(sfils_copy);
}

/********
 ********
 ********
 */

static
void
find_identical_insts_entry(Catalog_entry *entry)
{
	find_identical_insts_classes(entry->key_classes);
	find_identical_insts_classes(entry->response_classes);
}

static
void
find_identical_insts_classes(TABLE *classes)
{
	char *clasname;
	Clas *clas;
	TABLE *clasnames;
	
	clasnames = option_find_values(MUC_Options,
				       ":equatable_objects");
	table_first(clasnames);
	for(clasname=table_next(clasnames);
	    clasname;
	    clasname=table_next(clasnames))
	{
		clas = clas_find(classes, clasname );
		if(clas == NULL)
		{
			fprintf(stderr,
				"no equatable class named \'s\' found\n",
				clasname);
			exit(1);
		}
		find_identical_insts_insts(clas->insts);
	}
}

static
void
find_identical_insts_insts(TABLE *insts)
{
	TABLE *insts_copy;
	Inst *inst1, *inst2;
	
	insts_copy = table_list_create(inst_compare,
				       NULL,
				       default_destroy);
	table_concat(insts_copy, insts);


	table_first(insts);
	for(inst1=(Inst *)table_next(insts);	
	    inst1;
	    inst1=(Inst *)table_next(insts))
	{
		/* skip over comparisons already made */
		table_first(insts_copy);
		while((inst2=(Inst *)table_next(insts_copy))
		      &&
		      !(STRCASEEQ(inst1->id, inst2->id)));
			
		/* look for equal insts */
		for(inst2=(Inst *)table_next(insts_copy);
		    inst2;
		    inst2=(Inst *)table_next(insts_copy))
			if( insts_are_equatable(inst1, inst2) )
				make_equivalent(inst1, inst2);
	}
	/* go through once more and make sure every inst
	   points directly to its inst rep.
	 */
	table_first(insts);
	for(inst1=(Inst *)table_next(insts);	
	    inst1;
	    inst1=(Inst *)table_next(insts))
		find_inst_rep(inst1);
}

static
Bool
insts_are_equatable(Inst *inst1, Inst *inst2)
{
	if(num_scored_slots(inst1->slots) 
	   !=
	   num_scored_slots(inst2->slots))
		return False;
	
	return slots_are_equatable(inst1->slots,
				   inst2->slots);
}

static
long
num_scored_slots(TABLE *slots)
{
	long result = 0;
	Slot *slot;
	
	table_first(slots);
	for(slot=(Slot *)table_next(slots);
	    slot;
	    slot=(Slot *)table_next(slots))
		if( ! slot_is_unscored(slot) )
			result++;
	return result;
}

static
Bool
slots_are_equatable(TABLE *slots1, TABLE *slots2)
{
	Slot *slot1;
	Slot *slot2;
	
	table_first(slots1);
	for(slot1=(Slot *)table_next(slots1);
	    slot1;
	    slot1=(Slot *)table_next(slots1))
	{
		if(slot_is_unscored(slot1))
			continue;

		slot2 = slot_find(slots2, slot1->name);
		if(slot2 == NULL)
			return False;

		
		if(!mfils_are_equatable(slot1->mfils,
					slot2->mfils))
			return False;
	}
	return True;
}

static
Bool
mfils_are_equatable(TABLE *mfils1, TABLE *mfils2)
{
	Mfil *mfil1;
	Mfil *mfil2;
	Bool found;
	
	if(table_count(mfils1)
	   !=
	   table_count(mfils2))
		return False;
	
	table_first(mfils1);
	for(mfil1=(Mfil *)table_next(mfils1);
	    mfil1;
	    mfil1=(Mfil *)table_next(mfils1))
	{
		found = False;
		table_first(mfils2);
		for(mfil2=(Mfil *)table_next(mfils2);
		    mfil2;
		    mfil2=(Mfil *)table_next(mfils2))
			if(sfils_are_equatable(mfil1->sfils,
					       mfil2->sfils))
			{
				found = True;
				break;
			}
		if( ! found )
			return False;
	}
	return True;
}

static
Bool
sfils_are_equatable(TABLE *sfils1, TABLE *sfils2)
{
	Sfil *sfil1;
	Sfil *sfil2;
	Bool found;
	
	if(table_count(sfils1)
	   !=
	   table_count(sfils2))
		return False;
	
	table_first(sfils1);
	for(sfil1=(Sfil *)table_next(sfils1);
	    sfil1;
	    sfil1=(Sfil *)table_next(sfils1))
	{
		found = False;
		table_first(sfils2);
		for(sfil2=(Sfil *)table_next(sfils2);
		    sfil2;
		    sfil2=(Sfil *)table_next(sfils2))
			if(sfils_are_equal(sfil1, sfil2))
			{
				found = True;
				break;
			}
		if( ! found )
			return False;
	}
	return True;
}
	
static
Bool
sfils_are_equal(Sfil *sfil1, Sfil *sfil2)
{
	if( sfil1->type != sfil2->type )
		return False;
	
	switch(sfil1->type)
	{
	case SET_FILL:
		return STRCASEEQ(sfil1->fill.set->fill,
				 sfil2->fill.set->fill);
	case STRING_FILL:
		return STRCASEEQ(sfil1->fill.str->fill,
				 sfil2->fill.str->fill);
	case POINTER_FILL:
		return (sfil1->fill.ptr->fill
			==
			sfil2->fill.ptr->fill);
	}
}

static
void
make_equivalent(Inst *inst1, Inst *inst2)
{
	Inst *inst1_rep;
	Inst *inst2_rep;

	/* inst1 is equatable to inst2.
	   Find the "root" members of their
	   "equatable classes," and make
	   whichever root that is lexically
	   greater point to the root that is
	   lexically less in the table.
	 */

	inst1_rep = find_inst_rep(inst1);
	inst2_rep = find_inst_rep(inst2);
	
	if( muc_strcasecmp(inst1_rep->id, inst2_rep->id) < 0 )
	{
/*
		printf("\nEquating <%s-%s-%s> and <%s-%s-%s>\n",
		       inst1->name,
		       inst1->docnum,
		       inst1->id,
		       inst2->name,
		       inst2->docnum,
		       inst2->id);
*/
		inst2_rep->its_rep_inst = inst1_rep;
	}	
	
	else if( muc_strcasecmp(inst2_rep->id, inst1_rep->id) < 0 )
	{
/*
		printf("\nEquating <%s-%s-%s> and <%s-%s-%s>\n",
		       inst1->name,
		       inst1->docnum,
		       inst1->id,
		       inst2->name,
		       inst2->docnum,
		       inst2->id);
*/
		inst1_rep->its_rep_inst = inst2_rep;
	}
}

static
Inst *
find_inst_rep(Inst *inst)
{
	if( inst->its_rep_inst == inst )
		;   /* found */
	else
		inst->its_rep_inst = 
			find_inst_rep(inst->its_rep_inst);
					 
	return inst->its_rep_inst;
}

