/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *      @(#)massage_TR.c	6.1\t09/08/97
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	functions to preprocess TR data before alignment/scoring
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)massage_TR.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void mark_implicit_TR_optionals_entry(Catalog_entry *entry);
static void mark_implicit_TR_optionals_classes(TABLE *classes);
static void mark_implicit_TR_optionals_insts(TABLE *insts);
static void mark_implicit_TR_optionals_slots(Inst *inst, TABLE *slots);
static void mark_implicit_TR_optionals_mfils(Inst *inst, TABLE *mfils);
static void mark_implicit_TR_optionals_sfils(Inst *inst, TABLE *sfils);

/********
 ********
 ********
 */

void
massage_TR_entry(Catalog_entry *entry)
{
	check_status_slots_in_entry(entry);
	if( IS_K2R )
		mark_implicit_TR_optionals_entry(entry);
	make_clean_stringfills_entry(entry);
}

static
void
mark_implicit_TR_optionals_entry(Catalog_entry *entry)
{
	if( entry == NULL )
	{
		fprintf(stderr,
			"\nWARNING: null catalog entry.\n");
		return;
	}

	mark_implicit_TR_optionals_classes(entry->key_classes);
}

static
void
mark_implicit_TR_optionals_classes(TABLE *classes)
{
	TABLE *map_order;
	TABLE *reverse_map_order;
	char *clas_name;
	Clas *clas;
	
	map_order = option_find_values(MUC_Options, ":map_order");
	reverse_map_order = table_list_create(cstring_compare,
					      NULL, 
					      default_destroy);
	table_reverse(reverse_map_order, map_order);
	
	table_first(reverse_map_order);
	for(clas_name=table_next(reverse_map_order);
	    clas_name;
	    clas_name=table_next(reverse_map_order))
	{
		clas = clas_find(classes, clas_name );
		if( clas )
			mark_implicit_TR_optionals_insts( clas->insts );
	}
	table_destroy(reverse_map_order);
}

static
void
mark_implicit_TR_optionals_insts( TABLE *insts )
{
	Inst *inst;
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		if( inst->is_optional )
			; 
		else
			mark_implicit_TR_optionals_slots(inst,
							 inst->slots);
	}
}

static
void
mark_implicit_TR_optionals_slots(Inst *inst, TABLE *slots )
{
	Slot *slot;
	
	table_first(slots);
	for(slot=(Slot *)table_next(slots);
	    slot;
	    slot=(Slot *)table_next(slots))
		mark_implicit_TR_optionals_mfils(inst,
						 slot->mfils);
}


static
void
mark_implicit_TR_optionals_mfils(Inst *inst, TABLE *mfils )
{
	Mfil *mfil;

	table_first(mfils);
	for(mfil=(Mfil *)table_next(mfils);
	    mfil;
	    mfil=(Mfil *)table_next(mfils))
		mark_implicit_TR_optionals_sfils(inst, 
						 mfil->sfils);
}

static
void
mark_implicit_TR_optionals_sfils(Inst *inst, TABLE *sfils )
{
	Sfil *sfil;
	
	table_first(sfils);
	for(sfil=(Sfil *)table_next(sfils);
	    sfil;
	    sfil=(Sfil *)table_next(sfils))
	{
		if((sfil->type == POINTER_FILL)
		   && 
		   (sfil->fill.ptr) 
		   && 
		   (sfil->fill.ptr->fill)
		   && 
		   inst_is_optional(sfil->fill.ptr->fill)
		   &&
		   ( ! inst_is_optional(inst)) )
		{
			fprintf(stderr,	
				"\nMarking key object <%s-%s-%s> optional\n",
				inst->name,
				inst->docnum,
				inst->id);
			inst->is_optional = True;
		}
	}
}	
