/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * FILE 
 *	@(#)massage_sgml.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *      stuff to do after parsing but before aligning keys and responses
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)massage_sgml.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/********/

static void change_cr_to_spc_in_classes( TABLE *classes );
static void change_cr_to_spc_in_insts( TABLE *insts );
static void change_cr_to_spc_in_mfils( TABLE *mfils );
static void change_cr_to_spc_in_slots( TABLE *slots );
static void change_cr_to_spc_in_sfils( TABLE *sfils );
static void change_cr_to_spc( char *fill );

/********/

static void trim_TEXT_whitespace_in_classes( TABLE *classes );
static void trim_TEXT_whitespace_in_insts( TABLE *insts );
static void trim_TEXT_whitespace_in_mfils(TABLE *mfils);
static void trim_TEXT_whitespace_in_sfils(TABLE *sfils);
static void trim_TEXT_whitespace(Stringfill *stringfill);

/********/

static void calc_substring_offsets_in_classes( TABLE *classes );
static void calc_substring_offsets_in_insts( TABLE *insts );
static void calc_substring_offsets_in_mfils(TABLE *text_mfils,
					    TABLE *other_mfils);
static void calc_substring_offsets(Stringfill *text_stringfill, 
				   Stringfill *other_stringfill,
				   char *ptr);
static char *find_other_string_in_text_string(char *text_str, 
					      char *other_str);
static void change_quote_chars(char *ptr, char *quote_subst);
static Mfil *make_new_substring_mfil(Stringfill *text_stringfill, 
				     Stringfill *other_stringfill, 
				     char *ptr);
static Stringfill *first_stringfill_in_mfils( TABLE *mfils );
static Stringfill *next_stringfill_in_mfils( TABLE *mfils );


void
change_cr_to_spc_in_entry( Catalog_entry *entry )
{
	change_cr_to_spc_in_classes( entry->key_classes );
	change_cr_to_spc_in_classes( entry->response_classes );
}


static
void
change_cr_to_spc_in_classes( TABLE *classes )
{
	Clas *clas;

	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		change_cr_to_spc_in_insts( clas->insts );
}

static
void
change_cr_to_spc_in_insts( TABLE *insts )	
{
	Inst *inst;
	
	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
		change_cr_to_spc_in_slots(inst->slots);
}

static
void
change_cr_to_spc_in_slots( TABLE *slots )
{
	Slot *slot;
	
	table_first(slots);
	for(slot=(Slot *)table_next(slots);
	    slot;
	    slot=(Slot *)table_next(slots))
		change_cr_to_spc_in_mfils(slot->mfils);
}

static
void
change_cr_to_spc_in_mfils( TABLE *mfils )
{
	Mfil *mfil;
	
	table_first(mfils);
	for(mfil=(Mfil *)table_next(mfils);
	    mfil;
	    mfil=(Mfil *)table_next(mfils))
		change_cr_to_spc_in_sfils(mfil->sfils);
}		

static
void
change_cr_to_spc_in_sfils( TABLE *sfils )
{
	Sfil *sfil;
	Stringfill *stringfill;
	char *fill;
	
	table_first(sfils);
	for(sfil=(Sfil *)table_next(sfils);
	    sfil;
	    sfil=(Sfil *)table_next(sfils))
		if( sfil->type == STRING_FILL )
		{
			stringfill = sfil->fill.str;
			if( stringfill )
			{
				fill = stringfill->fill;
				change_cr_to_spc( fill );
			}
		}
}

static
void
change_cr_to_spc( char *fill )
{
	char *ptr = fill;
	
	if(ptr)
		while(*ptr)
		{
			if( *ptr == '\n' )
				*ptr = ' ';
			ptr++;
		}
}

/*
 ******************************************
 ******************************************
 ******************************************
 ******************************************
 ******************************************
 */

void
trim_TEXT_whitespace_in_entry( Catalog_entry *entry )
{
	trim_TEXT_whitespace_in_classes( entry->key_classes );
	trim_TEXT_whitespace_in_classes( entry->response_classes );
}

static
void
trim_TEXT_whitespace_in_classes( TABLE *classes )
{
	Clas *clas;

	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		trim_TEXT_whitespace_in_insts( clas->insts );
}

static
void
trim_TEXT_whitespace_in_insts( TABLE *insts )
{
	Inst *inst;
	Slot *slot;
	char *slotname;
	
	slotname = option_find_first_value(MUC_Options, 
					   ":sgml_TEXT_slot");

	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		slot = slot_find(inst->slots, slotname);
		if( slot )
			trim_TEXT_whitespace_in_mfils(slot->mfils);
		else
			fprintf(stderr,
			       "WARNING: no text in %s object with id %s\n",
			       (inst->is_key ? "key" : "response"),
			       inst->id);
	}
}
	
static
void
trim_TEXT_whitespace_in_mfils(TABLE *mfils)
{
	Mfil *mfil;
	
	table_first(mfils);
	for(mfil=(Mfil *)table_next(mfils);
	    mfil;
	    mfil=(Mfil *)table_next(mfils))
		trim_TEXT_whitespace_in_sfils(mfil->sfils);
}

static
void
trim_TEXT_whitespace_in_sfils(TABLE *sfils)
{
	Sfil *sfil;
	
	table_first(sfils);
	for(sfil=(Sfil *)table_next(sfils);
	    sfil;
	    sfil=(Sfil *)table_next(sfils))
		if( sfil->type == STRING_FILL )
			trim_TEXT_whitespace(sfil->fill.str);
}
	
static
void
trim_TEXT_whitespace(Stringfill *stringfill)
{
	char *f = stringfill->fill;
	char *f_end = stringfill->fill + strlen(stringfill->fill) - 1;
	
	/* find first non-ws in fill */
	f = f + strspn(f, " \t");   

	/* update start_offset and beginning of fill */
	stringfill->start_offset = 
		stringfill->start_offset + (f - stringfill->fill);
	muc_memmove(stringfill->fill, f, (strlen(f)+1));
	
	/* find last non-ws in fill */
	f = f_end;
	while( ((*f == ' ') || (*f == '\t')) && (f > stringfill->fill) )
		f--;
	
	/* update end_offset and end of fill */
	stringfill->end_offset =
		stringfill->end_offset - (f_end - f);
	*(f + 1) = '\0';
}
		
/*
 ******************************************
 ******************************************
 ******************************************
 ******************************************
 ******************************************
 */
	
void
calc_substring_offsets_in_entry( Catalog_entry *entry )
{
	calc_substring_offsets_in_classes( entry->key_classes );
	calc_substring_offsets_in_classes( entry->response_classes );
}


static
void
calc_substring_offsets_in_classes( TABLE *classes )
{
	Clas *clas;

	table_first(classes);
	for(clas = (Clas *)table_next(classes);	
	    clas;
	    clas = (Clas *)table_next(classes))
		calc_substring_offsets_in_insts( clas->insts );
}

static
void
calc_substring_offsets_in_insts( TABLE *insts )
{
	Inst *inst;
	char *text_slotname;
	char *other_slotname;
	Slot *text_slot;
	Slot *other_slot;
	
	text_slotname = option_find_first_value( MUC_Options, 
						":sgml_TEXT_slot");
	if( IS_NE )
		other_slotname = option_find_first_value(MUC_Options, 
							 ":sgml_ALT_slot");
	else /* CO */
		other_slotname = option_find_first_value(MUC_Options, 
							 ":sgml_MIN_slot");

	table_first(insts);
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
	{
		text_slot = slot_find( inst->slots, text_slotname );
		if( text_slot == NULL )
		{
			fprintf(stderr, 	
				"calc_substring_offsets_in_insts: no text slot.\n");
			exit(1);
		}
		other_slot = slot_find( inst->slots, other_slotname );

		/* responses usually don't have MIN's */
		if( other_slot )  
			calc_substring_offsets_in_mfils(text_slot->mfils, 
							other_slot->mfils);
	}
}

/*
 * calc_substring_offsets_in_mfils

   ASSUMPTIONS:
   -- only one mfil in text)mfils, which has only one sfil, a stringfill
   -- every mfil in other_mfils has only one sfil, a stringfill
   -- a stringifll in other mfils is a sbustringof the stringfill in text mfils
   RESULTS
   -- the offsets of eacch string in other mfils are set to match the first
      occurence of the substring in the stringfill of text_mfils,
   -- if there are other occurences of the substring in the text_mfil 
      substring, an new mfil->sfil->stringfill is created, with the
      right offsets, and added to other_mfils
*/

static
void
calc_substring_offsets_in_mfils(TABLE *text_mfils,
				TABLE *other_mfils)
{
	Mfil *new_mfil;  /* if more than one substring of text, new mfils created */
	TABLE *new_mfils;             
	TABLE *other_mfils_copy;      /* copy of the original list of other mfils */
	Stringfill *text_stringfill;
	Stringfill *other_stringfill;
	char *ptr;
	
	new_mfils = table_list_create(mfil_compare, 
				      NULL, 
				      default_destroy);
	other_mfils_copy = table_list_create(mfil_compare, 
					     NULL, 
					     default_destroy);

	/* find the stringfill */
	text_stringfill = first_stringfill_in_mfils( text_mfils );

	/* make a copy of the original table of other mfils (MIN or ALT) */
	table_concat( other_mfils_copy, other_mfils );

	/* go through the copy of the original table... */
	for(other_stringfill = first_stringfill_in_mfils( other_mfils_copy );
	    other_stringfill;
	    other_stringfill = next_stringfill_in_mfils( other_mfils_copy ))
	{
		/* find first position of substring in text string */
		ptr = find_other_string_in_text_string(text_stringfill->fill,
						       other_stringfill->fill);
		if( ptr )
		{
			calc_substring_offsets(text_stringfill, other_stringfill, ptr);
			/* advance pointer one char and look for other copies */
			ptr++;
			ptr = find_other_string_in_text_string(ptr, 
							       other_stringfill->fill);
			while(ptr)
			{	
				/* make a new mfil for each additional occurence of */
				/* substring in text string */
				new_mfil = make_new_substring_mfil(text_stringfill,
								   other_stringfill,
								   ptr);	
				table_append((char *)new_mfil, new_mfils);
				ptr++;
				ptr = find_other_string_in_text_string(ptr, 
								       other_stringfill->fill);
			}
		}
		else
			fprintf(stderr,
				"WARNING: substring \'%s\' not found in \'%s\'\n",
				other_stringfill->fill,
				text_stringfill->fill);
		/* add any new mfils created to the ALT/MIN table */
		table_concat(other_mfils, new_mfils);
		/* reset the table of new mfils */
		table_destroy(new_mfils);
		new_mfils = table_list_create(mfil_compare, 
					      NULL, 
					      default_destroy);
	}
	table_destroy(other_mfils_copy);
	table_destroy(new_mfils);
}

static
char *
find_other_string_in_text_string(char *text_str, char *other_str)
{
	char *text_cpy;
	char *text_cpy_ptr;
	char *result;
	char *quote_subst;
	
	if( *text_str == '\0' )
		return NULL;

	quote_subst = 
		option_find_first_value(MUC_Options, 
					":sgml_attribute_quote_char");

	/* if no quote char in options, just look in text string */
	if( quote_subst == NULL )
		result = strstr(text_str, other_str);

	/* If quote char is in the other string, change chars in text */
	/* copy and look for other string in (changed) text string.   */
	else if( strstr(other_str, quote_subst) )
	{
		text_cpy = STRALLOC(text_str);
		change_quote_chars(text_cpy, quote_subst);
		text_cpy_ptr = strstr(text_cpy, other_str);
		if( text_cpy_ptr == NULL )
			result = NULL;
		else
			result = text_str + (text_cpy_ptr - text_cpy);
		UFREE(text_cpy);
	}
 	else
		result = strstr(text_str, other_str);
	return result;
}


static
void
change_quote_chars(char *ptr, char *quote_subst)
{
	char q = *quote_subst;
	
	while( *ptr )
	{
		if( *ptr == '\"' )  /* " */
			*ptr = q;
		ptr++;
	}
}
	

static
Stringfill *
first_stringfill_in_mfils( TABLE *mfils )
{
	Mfil *mfil;
	TABLE *sfils;
	Sfil *sfil;
	
	table_first(mfils);
	mfil =(Mfil *)table_next(mfils);
	if( mfil == NULL ) return NULL;
	sfils = mfil->sfils;
	if( sfils == NULL ) return NULL;
	table_first(sfils);
	sfil = (Sfil *)table_next(sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}

static
Stringfill *
next_stringfill_in_mfils( TABLE *mfils )
{
	Mfil *mfil;
	TABLE *sfils;
	Sfil *sfil;
	
	mfil =(Mfil *)table_next(mfils);
	if( mfil == NULL ) return NULL;
	sfils = mfil->sfils;
	if( sfils == NULL ) return NULL;
	table_first(sfils);
	sfil = (Sfil *)table_next(sfils);
	if( sfil == NULL ) return NULL;
	if( sfil->type != STRING_FILL ) return NULL;
	return sfil->fill.str;
}

static
void
calc_substring_offsets(Stringfill *text_stringfill, 
		       Stringfill *other_stringfill,
		       char *ptr)
{
	other_stringfill->start_offset = 
		text_stringfill->start_offset + (ptr - text_stringfill->fill);
	other_stringfill->end_offset = 
		other_stringfill->start_offset + 
			strlen(other_stringfill->fill) - 1;
}

static
Mfil *
make_new_substring_mfil(Stringfill *text_stringfill, 
			Stringfill *other_stringfill, 
			char *ptr)
{
	Mfil *mfil;
	Sfil *sfil;
	
	sfil = sfil_create(STRING_FILL);
	sfil->fill.str->fill = STRALLOC(other_stringfill->fill);
	calc_substring_offsets(text_stringfill, sfil->fill.str, ptr);
	
	mfil = mfil_create();
	table_append((char *) sfil, mfil->sfils);
	return mfil;
}
