/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	mfil.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *	support routines for mfil data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)mfil.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  mfil_create
 *									*
 ***********************************************************************/

Mfil *
mfil_create(void)
{
  Mfil *mfil;
  
  if((mfil = UALLOC(Mfil, 1)) == (Mfil *)NULL)
    {
      fprintf(stderr, "Error: [mfil_create]: Allocation failed.\n");
      exit(1);
    }
  
  mfil->sfils = table_list_create(sfil_compare,
				  sfil_release,
				  default_destroy);
  return(mfil);
}


/************************************************************************
 *									*
 *  mfil_release							*
 *									*
 ***********************************************************************/

void
mfil_release(char *c_ptr)
{
    Mfil *mfil = (Mfil *)c_ptr;
    
    if(mfil == (Mfil *)NULL)
	return;
    
    table_destroy( mfil->sfils );
    UFREE(mfil);
}

/************************************************************************
 *									*
 *  mfil_compare							*
 *									*
 ***********************************************************************/

int
mfil_compare(char *item1, char *item2)
{
    return ((int)((char *) item1 - (char *) item2));
}


/************************************************************************
 *									*
 * mfil_clear_aligned_flags
 *									*
 ***********************************************************************/

int
mfil_clear_aligned_flags( TABLE *mfils )
{
  Mfil *mfil;
  
  table_first(mfils);
  for(mfil = (Mfil *)table_next(mfils);
      mfil;
      mfil = (Mfil *)table_next(mfils))
    mfil->is_aligned = False;
}

/************************************************************************
 *									*
 *  mfil_is_removed -- if every single fill of a multi-fill is "removed"
 *                     (ie, points to an unaligned optional instance), 
 *                     then the multi-fill itself is removed.
 *									*
 ***********************************************************************/

Bool
mfil_is_removed(Mfil *mfil)
{
  Sfil *sfil;
  TABLE *sfils;
  if( mfil )
    {
      sfils = mfil->sfils;
      table_first( sfils );
      for(sfil=(Sfil *)table_next(sfils);
	  sfil;
	  sfil=(Sfil *)table_next(sfils))
	if( !sfil_is_removed(sfil) )
	  return False;
    }
  return True;
}


/************************************************************************
 *									*
 * mfil_count_unaligned
 *									*
 ***********************************************************************/

int
mfil_count_unaligned( TABLE *mfils )
{
  Mfil *mfil;
  int num_unaligned = 0;
  
  table_first(mfils);
  for(mfil = (Mfil *)table_next(mfils);
      mfil;
      mfil = (Mfil *)table_next(mfils))
    if( !mfil->is_aligned )
      num_unaligned++;
  return num_unaligned;

}



/************************************************************************
 *									*
 * mfil_print_table
 *									*
 ***********************************************************************/

void
mfil_print_table(TABLE *mfil_table, int indent)
{
  Mfil *mfil;
  table_first( mfil_table );
  for(mfil = (Mfil *)table_next(mfil_table);
      mfil;
      mfil = (Mfil *)table_next(mfil_table))
    mfil_print( mfil, indent );
}


/************************************************************************
 *									*
 * mfil_print
 *									*
 ***********************************************************************/

void
mfil_print(Mfil *mfil, int indent)
{
	if( mfil == NULL ) return;
	
	printf("%*.s/\n", indent, "");

	sfil_print_table( mfil->sfils, indent+1);
}
