/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 * FILE 
 *	mfil_pair.c
 *
 * SYNOPSIS
 * DESCRIPTION
 *	support routines for Mfil_pair data structure.
 *
 * DIAGNOSTICS
 * FILES
 * NOTES
 *
 * SEE ALSO
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)mfil_pair.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  mfil_pair_create							*
 *									*
 ***********************************************************************/

Mfil_pair *
mfil_pair_create(Mfil *key_mfil, Mfil *response_mfil, char status)
{
    Mfil_pair *mfil_pair;
    
    if((mfil_pair = UALLOC(Mfil_pair, 1)) == (Mfil_pair *) NULL)
    {
	fprintf(stderr, "Error: allocation for mfil_pair failed.\n");
	exit(1);
    }

    mfil_pair->key_mfil      = key_mfil;
    mfil_pair->response_mfil = response_mfil;
    mfil_pair->status        = status;
    mfil_pair->sfil_pairs    = table_list_create(default_compar,
						 sfil_pair_release,
						 default_destroy );
    mfil_pair->tallies =  tallies_create();
    mfil_pair->score =    0.0 ;
    
    return(mfil_pair);
}


/************************************************************************
 *									*
 *  mfil_pair_release						*
 *									*
 ***********************************************************************/

void
mfil_pair_release(char *elt)
{
    Mfil_pair *ptr = (Mfil_pair *) elt;

    UFREE(ptr->tallies);
    table_destroy( ptr->sfil_pairs );
    UFREE(ptr);

    return;
}


/************************************************************************
 *									*
 *  mfil_pair_sum_tallies
 *									*
 ***********************************************************************/

int
mfil_pair_sum_tallies(Tallies *tallies, TABLE *mfil_pairs )
{
  Mfil_pair *mfil_pair;
  
  table_first(mfil_pairs);
  for(mfil_pair=(Mfil_pair *)table_next(mfil_pairs);
      mfil_pair;
      mfil_pair=(Mfil_pair *)table_next(mfil_pairs))
    tallies_combine( tallies, mfil_pair->tallies );
}



/************************************************************************
 *									*
 *  mfil_pair_f_compare
 *									*
 *  used to compare mfil_pair structures during "qsort"-ing.
 *       of "correct" mfils.
 *									*
 ***********************************************************************/

int
mfil_pair_f_compare(const void *elt1, const void *elt2)
{
    Mfil_pair *ptr1 = *(Mfil_pair **)elt1;
    Mfil_pair *ptr2 = *(Mfil_pair **)elt2;

    int dif;
    
    /* should sort in descending order by score,
     * so add a minus 
     */

    dif = (int) -(ptr1->score - ptr2->score);

    return(dif);
}


/************************************************************************
 *									*
 *  mfil_pair_wrong_compare
 *									*
 *  used to compare mfil_pair structures during "qsort"-ing of
 *       "incorrect" mfils -- we want incorrect mfil pairs sorted
 *       in increasing order by the number of wrong tallies
 *       (incorrect + missing + spurious).
 *									*
 ***********************************************************************/

int
mfil_pair_wrong_compare(const void *elt1, const void *elt2)
{
    Mfil_pair *ptr1 = *(Mfil_pair **)elt1;
    Mfil_pair *ptr2 = *(Mfil_pair **)elt2;

    int dif;
    
    dif = (int) (  ptr1->tallies->incorrect 
		 + ptr1->tallies->missing
		 + ptr1->tallies->spurious
		 - ptr2->tallies->incorrect 
		 - ptr2->tallies->missing
		 - ptr2->tallies->spurious);

    return(dif);
}
