/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 *  SccsId:  @(#)muc.h	6.1
 */


#ifndef _MUC_H_
#define _MUC_H_  1

#ifdef SCCSID
static char StructsSccsId[] = "@(#)muc.h	6.1\tSAIC";
#endif

/************************************************************************
 *									*
 *  Scoring structure used for tallying all the scoring categories.	*
 *									*
 ***********************************************************************/

typedef struct tallies
{
    int    possible;
    int    actual;
    int    correct;
    int    partial;
    int    incorrect;
    int    missing;
    int    spurious;
    int    noncommittal;
} Tallies;

typedef struct namedtallies
{
	char *name;
	struct tallies *tallies;
} Named_tallies;


/************************************************************************
 *									*
 *  MUC structure declarations						*
 *									*
 ***********************************************************************/

/************************************************************************
 *									*
 *  Structures for general class and slot definitions.  Class and slot	*
 *  definitions will be parsed from the slot-configuration file.	*
 *  Each class_def will be defined with its own set of slot_defs.	*
 *									*
 ***********************************************************************/

typedef struct class_def
{
    char	    *name;
    char	    *score_report_name;	/* the string pattern as it appears
					   in the document or template file */
    int		    map_threshold;   	/* default to 0 */
    Bool	    is_scored_class;	/* are instances of this class type
					   to be scored?  (true or false) */
    struct tallies  *total_tallies;	/* accumulated scores for all
					   instantiations of this class
					   type */
    TABLE	    *slot_defs;		/* a table of slot definitions */
} Class_def;


typedef struct slot_def
{
    char          *name;
    char          *score_report_name;	/* the string pattern as it appears
					   in the document or template file */
    TABLE         *fill_types;		/* in case of alternate fill types */
    TABLE         *set_fill_values;
    int           map_weight;		/* default to 0 */
    Bool	  is_scored_slot;	/* are instances of this slot type
					   to be scored?  (true or false) */
    struct tallies  *total_tallies;	/* accumulated scores for all
					   instantiations of this slot
					   type (for a given class type) */
} Slot_def;


/************************************************************************
 *									*
 *  catalog_entry - a structural representation of a sectionized	*
 *		    document:						*
 *									*
 *	- text_doc, k_doc, r_doc point to the input files, with		*
 *	  beginning and ending file positions for this particular	*
 *	  document.							*
 *	- k_root, r_root point to the root instantiation of the input	*
 *	  data.								*
 *	- class_mappings_list is a table of class_mappings.		*
 *	- class_connections_list is a table of class_connections.	*
 *									*
 ***********************************************************************/

typedef struct catalog_entry
{
    char		 *docnum;

    TABLE                *key_classes;
    TABLE                *response_classes;
    TABLE		 *clas_pairs;     /* alignment results */
    TABLE		 *class_scores;	
    TABLE                *ne_subtask_tallies;
    TABLE                *ne_doc_section_tallies;
    TABLE                *dsg_tallies;
    Bool                 is_pointed_to;
    struct tallies       *total_slot_tallies;
    float                total_slot_score;
    struct tallies	 *text_filtering;

    struct eqclasset     *key_eqclasset;
    struct eqclasset     *response_eqclasset;
    struct partitionset  *partitionset;
    long                  recall_numerator;
    long                  recall_denominator;
    long                  precision_numerator;
    long                  precision_denominator;
} Catalog_entry;

/************************************************************************
 *									*
 *  class_score - scoring data for all instances of this class type
 *		  in one document.  points to a table of slot_scores.
 *									*
 ***********************************************************************/

typedef struct class_score
{
    char	     *name;
    char	     *docnum;
    struct tallies   *tallies;
    TABLE	     *slot_scores;    /* a table containing the scoring
					 data for the slots of this class,
					 for this document */
} Class_score;

/************************************************************************
 *									*
 *  slot_score - scoring data for one slot of a given class, for all	*
 *		 the instances of that class type in one document.	*
 *									*
 ***********************************************************************/

typedef struct slot_score
{
    char	     *name;
    char	     *docnum;
    struct tallies   *tallies;
} Slot_score;

    
/************************************************************************
 *									*
 *  a "class" contains a pointer to a table of instantiated	*
 *  objects of that class type, with each instance being of the same	*
 *  docnum, but having a unique id (1up_number).			*
 *									*
 ***********************************************************************/

typedef struct clas
{
    char	*name;
     char	*docnum;
    Bool        is_aligned;
    TABLE	*insts;            /* a list of class instantiations
				   for this document */
} Clas;


/************************************************************************
 *									*
 *  insts are also sometimes known as objects.  Each is a	*
 *  singular instantiation of a defined class type.  Each instance	*
 *  contains a pointer to a table of slot_fill structures.		*
 *									*
 ***********************************************************************/

typedef struct inst
{
    char		   *name;
    char		   *docnum;
    char		   *id;
    TABLE		   *slots;
    Bool                   is_key;
    struct class_def       *its_class_def;
    /*
     *  When comparing pointer fills, it's necessary
     *  to look at what an inst is aligned with.
     */
    Bool                   is_aligned;
    struct inst_pair       *its_pair; 
    struct inst            *aligned_with;

    /* For in_and_out instances -
       mappings between instances may be affected if there are identical
       insts, in which case the 'wrong' key may be mapped to the 'wrong'
       response.  Set this pointer to the representative in_and_out instance,
       so that pointers to this instance can still be scored correct.
    */
       
    struct inst            *its_rep_inst;

    /* set the following two booleans when this instance is created
       as a pointer or as a 'true' object instantiation. */
    Bool		   is_instantiated;
    Bool		   is_pointed_to;

    /* for named_entity and coreference tasks */
    Bool		   is_optional;
    Bool                   could_be_optional;
    long			   start_offset;
    long			   end_offset;
    char                   *doc_section;
    long                   eqclass_index;
    struct inst            *eqclass_rep;

} Inst;

/************************************************************************
 *									*
 *  slot
 *     where information about an instance is kept:  a "field" of a
 *     "record".  Since keys can have alternate fills, slots have a 
 *     list of multi-fills (mfil), which each element in the list
 *     being an alternative from the key instances.
 *									*
 ***********************************************************************/

typedef struct slot
{
    char		   *name;           /* name of the slot */
    Slot_def               *its_slot_def;
    Bool 		   is_optional;	    /* true or false */
    Bool                   is_aligned;
    TABLE                  *mfils;
} Slot;

/************************************************************************
 *									*
 *  mfil -- multi-fills
 *      multi_fills are just lists of single fills (sfil)
 *									*
 ***********************************************************************/

typedef struct multi_fill
{
    Bool        is_aligned;
    TABLE       *sfils;
} Mfil;

/************************************************************************
 *									*
 *  sfil -- single fills 
 *      are polymorphic;
 *      type tells which type a single_fill should be processed as;
 *      fill is the pointer to the actual fill
 *									*
 ***********************************************************************/

typedef struct sfil
{
    int                    type;  /* SET_FILL, STRING_FILL, POINTER_FILL */
    Bool        is_aligned;
    struct sfil_pair *its_pair;
    union 
    {
      struct stringfill  *str;
      struct pointerfill *ptr;
      struct setfill     *set;
    }                      fill;
} Sfil;


/* stringfill, pointerfill and setfill are "subtypes" of the
 * polymorphic sfil.  Hence they don't have is_aligned flags.
 */

typedef struct stringfill
{
    char	*fill;	       /* text */
    char        *straight_fill;/* lc text, with one space between words */
    char	*clean_fill;   /* lc text with pre- and post-modifiers and */
                               /* corporate designators removed         */
    long         start_offset;
    long         end_offset;
} Stringfill;

typedef struct pointerfill
{
  Inst   *fill;
  char             *string_rep;     /* printable string */
  long start_offset;
  long end_offset;
} Pointerfill;

typedef struct setfill
{
  char *fill;
  long start_offset;
  long end_offset;
} Setfill;


/************************************************************************
 *									*
 *  configuration options
 *									*
 ***********************************************************************/

typedef struct option
{
    char    *name;
    TABLE   *values;  /* c-strings only */
} Option;


/************************************************************************
 *									*
 *  pairs - to align two things, create a "thing_pair", with pointers
 *          to the two things, and tallies holding the result of
 *          comparing the two things.
 *									*
 ***********************************************************************/

typedef struct clas_pair
{
  char *name;
  char status;           /* COR, INC, OPT, MIS, SPU, NON */
  Clas *key_clas;
  Clas *response_clas;
  TABLE *inst_pairs;
  struct tallies *tallies;
  float score;
} Clas_pair;

typedef struct inst_pair
{
  char status;           /* COR, INC, OPT, MIS, SPU, NON */
  Inst *key_inst;
  Inst *response_inst;
  TABLE *slot_pairs;
  struct tallies *tallies;
  float score;
  float weighted_score;   /* dot product of slots' scores with */
                            /* slots' map_weights                */
} Inst_pair;

typedef struct slot_pair
{
  char *name;
  char status;           /* COR, INC, OPT, MIS, SPU, NON */
  Slot *key_slot;
  Slot *response_slot;
  TABLE *mfil_pairs;
  struct tallies *tallies;
  float score;
} Slot_pair;

typedef struct mfil_pair
{
  char status;           /* COR, INC, OPT, MIS, SPU, NON */
  Mfil *key_mfil;
  Mfil *response_mfil;
  TABLE *sfil_pairs;
  struct tallies *tallies;
  float score;
} Mfil_pair;

typedef struct sfil_pair
{
  char status;           /* COR, INC, OPT, MIS, SPU, NON */
  Sfil *key_sfil;
  Sfil *response_sfil;
  struct tallies *tallies;
  struct tallies *minstr_tallies;
  TABLE *minstr_pairs;
  float score;
} Sfil_pair;

/***********************************************************
 *  
 *    Coref stuff
 *
 ***********************************************************
 */

/*
 * Refmap - a hash-table way to find the inst that a REF slot
 *          refers to.
 */

typedef struct refmap
{
	char *ref;
	Inst *inst;
} Refmap;

/*
 *
 *   Eqmap- when computing coref scores, we need
 *        a quick way to associate an "inst" with the 
 *        equivalence class that it's in.  We'll use a 
 *        (hash) table of eqmaps, with the inst's as
 *        the hash keys (Equivalence classes are just 
 *        tables of insts).  -- see score_coref.c
 *
 */

typedef struct eqmap
{
	Inst *inst;
	struct eqclass *eqclass;
} Eqmap;

/*
 *
 *   Eqclass - 
 *       a set of instances which have be linked together by
 *       the key or response.  It also holds an integer, which
 *       is the index into the 2-dimensional array holding the 
 *       partitions.
 *
 */


typedef struct eqclass
{
	int class_num;    /* index into partition matrix */
	TABLE *insts;
} Eqclass;


/*
 *
 *   Eqclasset - a set of equivalence classes.  The 
 *        classes themselves are a linked-list table
 *	  of linked-list tables of insts.  An eqclasset
 *	  also has a hash table, mapping insts to equivalence
 *	  classes of insts (see eqmap above)
 *
 */

typedef struct eqclasset
{
	TABLE *eqclasses;
	TABLE *eqmaps;
} Eqclasset;

typedef struct partitionset
{
	int num_key_classes;
	int num_resp_classes;
	TABLE **matched;
	TABLE **missing;
	TABLE **spurious;
} Partitionset;


#endif   /* _MUC_H_ */
