/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 *  muc_protos.h include file.
 *
 *  scorer global function prototypes.
 *
 */


#ifndef  _MUC_PROTOS_H_
#define  _MUC_PROTOS_H_   1

#ifdef SCCSID
static char ProtosSccsId[] = "@(#)muc_protos.h	6.1\tSAIC";
#endif

/*********************/
/* "top-level" stuff */
/*********************/

/* init_muc.c */
void initialize_MUC_scorer();

/* massage.c */
void check_status_slots_in_entry(Catalog_entry *entry);

/* massage_sgml.c */
void change_cr_to_spc_in_entry( Catalog_entry *entry );
void trim_TEXT_whitespace_in_entry( Catalog_entry *entry );
void calc_substring_offsets_in_entry( Catalog_entry *entry );

/* massage_NE.c */
void massage_NE_entry(Catalog_entry *entry);

/* massage_CO.c */
void massage_CO_entry(Catalog_entry *entry);
char *get_inst_ID_slot_fill(Inst *inst);
Pointerfill *get_inst_REF_slot_fill(Inst *inst);

/* massage_IE.c */
void make_clean_stringfills_entry(Catalog_entry *entry);

/* massage_ST.c */
void massage_ST_entry(Catalog_entry *entry);

/* massage_TR.c */
void massage_TR_entry(Catalog_entry *entry);

/* align_sgml.c */
Bool LT( Inst *key, Inst *resp );
Stringfill *trim_stringfill( Stringfill *stringfill );
Bool offsets_overlap(Stringfill *s1, Stringfill *s2);
Stringfill *get_first_stringfill_in_text_slot( Inst *inst );
Stringfill *get_next_stringfill_in_text_slot( Inst *inst );


/* align_CO.c */
void align_CO(Catalog_entry *entry);

/* align_NE.c */
void align_NE(Catalog_entry *entry);

/* align_IE.c */

void align_IE(Catalog_entry *entry);
Inst_pair *pair_min(Inst *key_inst );
Inst_pair *pair_oin(Inst *inst );
Inst_pair *pair_sin(Inst *inst );
Inst_pair *pair_cin(Inst *key_inst, Inst *response_inst );
void align_oin(TABLE *key_insts, TABLE *inst_pairs);
void align_min(TABLE *key_insts, TABLE *inst_pairs );
void align_sin(TABLE *insts, TABLE *inst_pairs );
void align_cin_choose_pairs(Inst_pair **all_pairs, 
			    int num_all_pairs, 
			    TABLE *inst_pairs );


/* map_history.c */
void display_map_history_entry(Catalog_entry *entry);

/* report_summary_NE.c */
void display_report_summary_NE(Catalog_entry *entry);

/* report_summary_CO.c */
void display_report_summary_CO(Catalog_entry *entry);

/* report_summary_IE.c */
void display_report_summary_IE(Catalog_entry *entry);

/* score.c */
void score_entry_IE(Catalog_entry *entry);

/* score_coref.c */
Eqclass *lookup_eqclass(Inst *member, Eqclasset *eqclasset);
void score_entry_coref(Catalog_entry *entry);

/* output_coref.c */
void display_scores_header_CO();
void display_entry_scores_CO(Catalog_entry *entry);
void display_entry_CO_partitions(Catalog_entry *entry);
void display_summary_scores_CO();

/****************/
/* parser stuff */
/****************/

/* parsers.c */
void parse_config(char *fname);
void parse_keys();
void parse_responses();

/* config.y --> y.config.c */
int configparse();

/* mapconfig.y --> y.mapconfig.c */
int mapconfigparse();

/* slotconfig.y --> y.slotconfig.c */
int slotconfigparse();

/* parse_templates.c */
/*
void parse_templates(char whichfile);
*/

/* markup.l --> l.markup.c */
int markup_lex();

/*******************/
/* structure stuff */
/*******************/

/* catalog_entry.c */
Catalog_entry *catalog_entry_create(char *docnum);
void catalog_entry_release(char *elt);
int catalog_entry_compare(char *elt1, char *elt2);
Catalog_entry *catalog_entry_find(TABLE *catalog_entries, char *docnum);
void catalog_entry_print_table( TABLE *catalog_entrys, int indent );
void catalog_entry_print(Catalog_entry *entry, int indent);

/* clas.c */
Clas *clas_create(char *name, char *docnum);
void clas_release(char *elt);
int clas_compare(char *elt1, char *elt2);
Clas *clas_find(TABLE *classes, char *name);
void clas_init_table( TABLE *classes, char *docnum );
int clas_count_unaligned( TABLE *classes );
void clas_print_table( TABLE *classes, int indent );
void clas_print(Clas *clas, int indent);

/* clas_pair.c */
Clas_pair *clas_pair_create(Clas *key_clas, Clas *response_clas, char status);
void clas_pair_release(char *elt);
void clas_pair_init_table(TABLE *clas_pairs,
			  TABLE *key_classes,
			  TABLE *response_classes);
void clas_pair_sum_tallies(Tallies *tallies, TABLE *clas_pairs );
int clas_pair_compare(char *item1, char *item2);
Clas_pair *clas_pair_find(TABLE *clas_pairs, char *name);
void clas_pair_print_table(TABLE *clas_pairs, int indent );
void clas_pair_print(Clas_pair *clas_pair, int indent);

/* class_def.c */
Class_def *class_def_create(char *name);
void class_def_release(char *elt);
int class_def_compare(char *elt1, char *elt2);
Class_def *class_def_find(TABLE *class_defs, char *name);
void class_def_print_table(TABLE *class_defs, int indent);
void print_class_def(Class_def *class_def, int indent);

/* class_score.c */
Class_score *class_score_create(Class_def *class_def, char *docnum);
void class_score_release(char *elt);
int class_score_compare(char *elt1, char *elt2);
Class_score *class_score_find(TABLE *scores, char *cname);
void class_score_init_table(TABLE *class_scores, char *docnum );

/* cstring.c */
void cstring_print_table(TABLE *char_table, int indent);
int cstring_compare(char *c1, char *c2);
TABLE *cstring_lower_table(TABLE *cstrings);
long cstring_case_hash(char *key);

/* refmap.c */
Refmap *refmap_create(char *ref, Inst *inst);
void refmap_release( char *elt );
int refmap_compare(char *elt1, char *elt2);
long refmap_hash( char *elt );
Inst *refmap_find_inst(TABLE *refmaps, char *ref);
void refmap_print_table(TABLE *refmaps, int indent);
void refmap_print(Refmap *refmap, int indent);

/* eqclass.c */
Eqclass *eqclass_create();
void eqclass_release(char *elt);
void eqclass_add_member(Inst *inst, Eqclass *eqclass);
void eqclass_compute_indices( TABLE *eqclasses );
void eqclass_print_table(TABLE *eqclasses, int indent);
void eqclass_print(Eqclass *eqclass, int indent);

/* eqclasset.c */
Eqclasset *eqclasset_create();
void eqclass_release(char *elt);
void eqclasset_release(char *elt);
void eqclasset_print_table(TABLE *eqclassets, int indent);
void eqclasset_print(Eqclasset *eqclasset, int indent);


/* eqmap.c */
Eqmap *eqmap_create(Inst *inst, Eqclass *eqclass);
void eqmap_release(char *elt);
int eqmap_compare(char *elt1, char *elt2);
long eqmap_hash(char *elt );
void eqmap_print_table(TABLE *eqmaps, int indent);
void eqmap_print( Eqmap *eqmap, int indent );

/* inst.c */
Inst *inst_create(char *name, char *docnum, char *id);
void inst_release(char *elt);
int inst_compare(char *elt1, char *elt2);
int inst_clear_aligned_flags(TABLE *insts );
int inst_count_unaligned(TABLE *insts );
Inst *inst_find(TABLE *insts, char *instid);
Bool inst_is_removed(Inst *inst);
Bool inst_offsets_overlap(Inst *kinst, Inst *rinst);
Inst *inst_find_aligned(Inst *inst);
Bool inst_is_key(Inst *inst);
Bool inst_is_optional(Inst *inst);
Inst *inst_follow_REF_link(Inst *inst);
void inst_print_table(TABLE *insts, int indent );
void inst_print(Inst *inst, int indent);

/* inst_pair.c */
Inst_pair *inst_pair_create(Inst *key_inst, Inst *response_inst, char status);
void inst_pair_release(char *elt);
void inst_pair_sum_tallies(Tallies *tallies, TABLE *inst_pairs );
int inst_pair_f_compare(const void *elt1, const void *elt2);
int inst_pair_wrong_compare(const void *elt1, const void *elt2);
int inst_pair_offset_compare(const void *elt1, const void *elt2);
void inst_pair_table_2_array(Inst_pair **array, TABLE *inst_pairs);
void inst_pair_array_2_table(TABLE *inst_pairs, Inst_pair **array, int n);
void inst_pair_print_table(TABLE *inst_pairs, int indent);
void inst_pair_print(Inst_pair *inst_pair, int indent);

/* mfil.c */
Mfil *mfil_create(void);
void mfil_release(char *c_ptr);
int mfil_compare(char *item1, char *item2);
int mfil_clear_aligned_flags(TABLE *mfils );
Bool mfil_is_removed(Mfil *mfil);
int mfil_count_unaligned(TABLE *mfils );
void mfil_print_table(TABLE *mfil_table, int indent);
void mfil_print(Mfil *mfil, int indent);

/* mfil_pair.c */
Mfil_pair *mfil_pair_create(Mfil *key_mfil, Mfil *response_mfil, char status);
void mfil_pair_release(char *elt);
int mfil_pair_sum_tallies(Tallies *tallies, TABLE *mfil_pairs );
int mfil_pair_f_compare(const void *elt1, const void *elt2);
int mfil_pair_wrong_compare(const void *elt1, const void *elt2);

/* named_tallies.c */
Named_tallies *named_tallies_create(char *name);
void named_tallies_release(char *elt);
int named_tallies_compare(char *elt1, char *elt2);
Named_tallies *named_tallies_find(TABLE *named_tallies_table, char *name);
void named_tallies_table_combine(TABLE *t1, TABLE *t2);
TABLE *named_tallies_table_create(TABLE *name_table);
void named_tallies_print_table(TABLE *named_tallies_table, int indent);
void named_tallies_print(Named_tallies *named_tallies, int indent);

/* option.c */
Option *option_create(char *name);
void option_release(char *elt);
int option_compare(char *elt1, char *elt2);
Option *option_find(TABLE *option_table, char *optname);
TABLE *option_find_values(TABLE *option_table, char *optname);
char *option_find_first_value(TABLE *option_table, char *optname);
void option_print_table(TABLE *option_table, int indent);
void option_print(Option *option, int indent);

/* partitionset.c */
Partitionset *partitionset_create(Eqclasset *key_eqclasset, 
				  Eqclasset *resp_eqclasset, 
				  TABLE *inst_pairs);
void partitionset_score_recall(Partitionset *ps,
			       long *numerator,
			       long *denominator);
void partitionset_score_precision(Partitionset *ps,
				  long *numerator,
				  long *denominator);
void partitionset_release(char *c_ptr);
int partitionset_compare(char *elt1, char *elt2);

/* pointerfill.c */
Pointerfill *pointerfill_create(void);
void pointerfill_release(char *c_ptr);
char pointerfill_match(Pointerfill *key_pointerfill,
		       Pointerfill *response_pointerfill,
		       Tallies *tallies);
void pointerfill_print(Pointerfill *pointerfill, int indent);

/* setfill.c */
Setfill *setfill_create(void);
void setfill_release(char *c_ptr);
char setfill_match(Setfill *key_setfill,
		   Setfill *response_setfill,
		   Tallies *tallies);
void setfill_print(Setfill *setfill, int indent);

/* sfil.c */
Sfil *sfil_create(int type);
void sfil_release(char *c_ptr);
int sfil_compare(char *item1, char *item2);
int sfil_clear_aligned_flags(TABLE *sfils );
Bool sfil_is_removed(Sfil *sfil);
int sfil_count_unaligned(TABLE *sfils );
void sfil_print_table(TABLE *sfils, int indent);
void sfil_print(Sfil *sfil, int indent);

/* sfil_pair.c */
Sfil_pair *sfil_pair_create(Sfil *key_sfil, Sfil *response_sfil, char status);
void sfil_pair_release(char *elt);
int sfil_pair_sum_tallies(Tallies *tallies, TABLE *sfil_pairs );
int sfil_pair_f_compare(const void *elt1, const void *elt2);
int sfil_pair_wrong_compare(const void *elt1, const void *elt2);

/* slot.c */
Slot *slot_create(char *name);
void slot_release(char *c_ptr);
int slot_compare(char *elt1, char *elt2);
int slot_clear_aligned_flags(TABLE *slots );
Slot *slot_find(TABLE *slots, char *sname);
Bool slot_is_removed(Slot *slot);
Bool slot_is_unscored(Slot *slot);
int slot_count_unaligned(TABLE *slots );
void slot_print_table(TABLE *slots, int indent);
void slot_print(Slot *slot, int indent);

/* slot_def.c */
Slot_def *slot_def_create(char *name, char *classname);
void slot_def_release(char *elt);
int slot_def_compare(char *elt1, char *elt2);
Slot_def *slot_def_find(TABLE *slot_defs, char *name);
void slot_def_print_table(TABLE *slot_def_table, int indent);
void slot_def_print(Slot_def *sd, int indent);

/* slot_pair.c */
Slot_pair *slot_pair_create(Slot *key_slot, Slot *response_slot, char status);
void slot_pair_release(char *elt);
Slot_pair *slot_pair_find(TABLE *slot_pairs, char *name);
int slot_pair_sum_tallies(Tallies *tallies, TABLE *slot_pairs );
float slot_pair_sum_weighted_scores(TABLE *slot_pairs );
int slot_pair_f_compare(const void *elt1, const void *elt2);
int slot_pair_wrong_compare(const void *elt1, const void *elt2);

/* slot_score.c */
Slot_score *slot_score_create(char *name, char *docnum);
void slot_score_release(char *elt);
int slot_score_compare(char *elt1, char *elt2);
Slot_score *slot_score_find(TABLE *scores, char *sname);
void slot_score_init_table(TABLE *slot_scores, 
			   Class_def *class_def, 
			   char *docnum);


/* stringfill.c */
Stringfill *stringfill_create(void);
void stringfill_release(char *c_ptr);
char stringfill_match(Stringfill *key_stringfill,
		      Stringfill *response_stringfill,
		      Tallies *tallies);
void stringfill_print(Stringfill *stringfill, int indent);

/* tallies.c */
Tallies *tallies_create(void);
void tallies_release(char *c);
void tallies_copy(Tallies *t1, Tallies *t2);
void tallies_combine(Tallies *t1, Tallies *t2);
void tallies_subtract(Tallies *t1, Tallies *t2);
void tallies_clear(Tallies *t);
void tallies_print(Tallies *t);
void tally_correct(Tallies *tallies, int count);
void tally_incorrect(Tallies *tallies, int count);
void tally_partial(Tallies *tallies, int count);
void tally_missing(Tallies *tallies, int count);
void tally_spurious(Tallies *tallies, int count);
void tally_noncommittal(Tallies *tallies, int count);
void tally1_correct(Tallies *tallies);
void tally1_partial(Tallies *tallies);
void tally1_incorrect(Tallies *tallies);
void tally1_missing(Tallies *tallies);
void tally1_spurious(Tallies *tallies);
void tally1_noncommittal(Tallies *tallies);

/*************************/
/* useful function stuff */
/*************************/

/* utils.c */
void non_release(char *elt);
void breaker();
void table_concat(TABLE *t1, TABLE *t2);
void muc_memmove(char *s, char *ct, int n);
float compute_tallies_f(Tallies *tallies, int which_f);
float compute_f(float beta, float precision, float recall);
float compute_error_per_response_fill(Tallies *tallies);
float compute_overgeneration(Tallies *tallies);
float compute_precision(Tallies *tallies);
float compute_recall(Tallies *tallies);
float compute_substitution(Tallies *tallies);
float compute_undergeneration(Tallies *tallies);

/* muc_strings.c */
Bool case_is_prefix(char *s, char *c);
Bool case_is_suffix(char *whole, char *string);
char *alloc_between( char *firstchar, char *afterlast );
char *trim_quotes(char *qstr);
char *trim_whitespace(char *str);
char *trim_intertoken_whitespace(char *str);
char *string_lower(char *str);
char *nice_docnum(char *docnum);
int  muc_strcasecmp(char *str1, char *str2);
char muc_tolower(char c);
Bool case_eq(char s, char c);


#endif   /* _MUC_PROTOS_H_ */
