/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	named_tallies.c
 *
 * SYNOPSIS
 *      named_tallies_create
 *      named_tallies_release
 *      named_tallies_compare
 *      named_tallies_table_combine
 *      named_tallies_table_create
 *      named_tallies_find
 *      named_tallies_print_table
 *      named_tallies_print
 *
 * DESCRIPTION
 *	support routines for Named_tallies data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)named_tallies.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  named_tallies_create
 *									*
 ***********************************************************************/

Named_tallies *
named_tallies_create(char *name)
{
    Named_tallies *named_tallies;
    
    if((named_tallies = UALLOC(Named_tallies, 1)) == (Named_tallies *)NULL)
    {
	fprintf(stderr, "Error: [named_tallies_create]: Allocation failed.\n");
	exit(1);
    }
    
    named_tallies->name = STRALLOC(name);
    named_tallies->tallies = tallies_create();
    return named_tallies;
}

/************************************************************************
 *									*
 *  named_tallies_release
 *									*
 ***********************************************************************/

void
named_tallies_release(char *elt)
{
    Named_tallies *named_tallies = (Named_tallies *)elt;
    
    if(named_tallies == (Named_tallies *)NULL)
	return;

    UFREE(named_tallies->name);
    UFREE(named_tallies->tallies);
    
    UFREE(named_tallies);
}


/************************************************************************
 *									*
 *  named_tallies_compare							*
 *									*
 ***********************************************************************/

int
named_tallies_compare(char *elt1, char *elt2)
{
    Named_tallies *named_tallies1 = (Named_tallies *)elt1;
    Named_tallies *named_tallies2 = (Named_tallies *)elt2;
    
    return(STRCASEEQ(named_tallies1->name, named_tallies2->name));
}


/************************************************************************
 *									*
 *  named_tallies_find
 *									*
 ***********************************************************************/

Named_tallies *
named_tallies_find(TABLE *named_tallies_table, char *name)
{
    Named_tallies  *named_tallies;
    
    /* search the table for a named_tallies having the same name as optname */

    table_first(named_tallies_table);
    for(named_tallies = (Named_tallies *)table_next(named_tallies_table);
	named_tallies;
	named_tallies = (Named_tallies *)table_next(named_tallies_table))
      if( STRCASEEQ(name, named_tallies->name) )
	break;

    return(named_tallies);
}


/************************************************************************
 *									*
 *  named_tallies_table_combine
 *           -for every tallies in t1, find a tallies in t2 with the
 *            same name, and add t2's tallies to t1's tallies
 *									*
 ***********************************************************************/

void
named_tallies_table_combine(TABLE *t1, TABLE *t2)
{
	Named_tallies *nt1;
	Named_tallies *nt2;
	
	if( t1 == NULL || t2 == NULL ) return;
	
	table_first(t1);
	for(nt1 = (Named_tallies *)table_next(t1);
	    nt1;
	    nt1 = (Named_tallies *)table_next(t1))
	{
		nt2 = named_tallies_find(t2, nt1->name);
		if( nt2 )
			tallies_combine(nt1->tallies, nt2->tallies);
	}
}

/************************************************************************
 *									*
 *  named_tallies_table_create
 *          - given a table of strings (like an option, for example),
 *            create a table of named_tallies, with empty tallies, and
 *            names the same as the given table
 *									*
 ***********************************************************************/

TABLE *
named_tallies_table_create(TABLE *name_table)
{
	TABLE *named_tallies_table;
	Named_tallies *named_tallies;
	char *name;

	named_tallies_table = table_list_create(named_tallies_compare,
						named_tallies_release,
						default_destroy);

	if( name_table == NULL ) return named_tallies_table;
	
	table_first(name_table);
	for(name = (char *)table_next(name_table);
	    name;
	    name = (char *)table_next(name_table))
	{
		named_tallies = named_tallies_create(name);
		table_append((char *)named_tallies, named_tallies_table);
	}
	
	return named_tallies_table;
}


/************************************************************************
 *									*
 * named_tallies_print_table
 *									*
 ***********************************************************************/

void
named_tallies_print_table(TABLE *named_tallies_table, int indent)
{
  Named_tallies *named_tallies;
  table_first( named_tallies_table );
  for(named_tallies = (Named_tallies *)table_next(named_tallies_table);
      named_tallies;
      named_tallies = (Named_tallies *)table_next(named_tallies_table))
    named_tallies_print( named_tallies, indent );
}


/************************************************************************
 *									*
 * named_tallies_print
 *									*
 ***********************************************************************/

void
named_tallies_print(Named_tallies *named_tallies, int indent)
{
  char *value;

  printf("%*.s%s\t", indent, "", named_tallies->name );
  
  tallies_print( named_tallies->tallies );
}

