/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *      @(#)parsers.c	6.1\t09/08/97
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	entry functions for the various parsers in the scorer program
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)parsers.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/* local functions */

static void parse_markup(void);

/*****************************************/
/*****************************************/
/*****************************************/

void
parse_config(char *fname)
{
	FILE	*fopen();
	extern FILE	*configin;   /* yyin --> configin   */
	extern configlineno;     /* defined in config.l */
	
	configlineno = 1;
	
	if ((configin = fopen(fname, "r")) == (FILE *)NULL)
	{
		fprintf(stderr, "Could not open configuration file '%s'.\n",
			fname);
		exit(1);
	}
	
	while (!feof(configin))
	{
		/* yyparse() --> configparse() returns 0 or 1 */
		if (configparse())
		{
			printf("parse of configuration '%s' failed.\n", fname);
			exit(1);
		}
	}
	
	fclose(configin);
}

/*****************************************/
/*****************************************/
/*****************************************/


void
parse_keys(void)
{
	MUC_key_or_response = KEY;
	
	if ( IS_IE )
	{
		printf("\nParsing keys:\n");
		parse_templates(KEY);
		printf("\ndone.\n");
	}
	else 
	{
		printf("\nParsing keys:\n");
		parse_markup();
		printf("\ndone.\n");
	}
}

/*****************************************/
/*****************************************/
/*****************************************/


void
parse_responses(void)
{
	MUC_key_or_response = RESPONSE;
	
	if ( IS_IE )
	{
		printf("\nParsing responses:\n");
		parse_templates(RESPONSE);
		printf("\ndone.\n");
	}
	else 
	{
		printf("\nParsing responses:\n");
		parse_markup();
		printf("\ndone.\n");
	}
}

/*****************************************/
/*****************************************/
/*****************************************/
/*****************************************/
/*****************************************/
/*****************************************/


static
void
parse_markup()
{
	FILE	 *fopen();
	
	/* yyin --> markup_in */
	/* defined in markup.l */
	extern FILE	 *markup_in;
	
	char *fname;
	
	if (MUC_key_or_response == KEY)
		fname = option_find_first_value(MUC_Options, ":key_file");
	else
		fname = option_find_first_value(MUC_Options, ":response_file");
	
	if (fname == NULL)
	{
		fprintf(stderr,
			"[parse_markup]: %s NE/CO file name not found.\n",
			( (MUC_key_or_response == KEY) ? "key" : "response"));
		exit(1);
	}
	
	if ((markup_in = fopen(fname, "r")) == (FILE *)NULL)
	{
		fprintf(stderr, "Could not open NE/CO document file '%s'.\n",
			fname);
		exit(1);
	}
	
	MUC_cur_lineno = 1;
	MUC_cur_text_charno = 0;
	MUC_cur_task_charno = 0;

	while (!feof(markup_in))
	{
		/* yylex() --> markup_lex() returns 0 or 1 */
		if(markup_lex())
		{
			printf("parse of NE/CO document file '%s' failed.\n", fname);
			exit(1);
		}
	}
	
	fclose(markup_in);
}
