/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	pointerfill.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for pointerfill data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)pointerfill.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  pointerfill_create
 *									*
 ***********************************************************************/

Pointerfill *
pointerfill_create(void)
{
    Pointerfill *new_pointerfill;
    
    if((new_pointerfill = UALLOC(Pointerfill, 1)) == (Pointerfill *)NULL)
    {
	fprintf(stderr, "Error: [pointerfill_create]: Allocation failed.\n");
	exit(1);
    }

    new_pointerfill->fill = NULL;
    new_pointerfill->string_rep = NULL;
    new_pointerfill->start_offset = 0;
    new_pointerfill->end_offset = 0;

    return(new_pointerfill);
}


/************************************************************************
 *									*
 *  pointerfill_release							*
 *									*
 ***********************************************************************/

void
pointerfill_release(char *c_ptr)
{
    Pointerfill *pointerfill = (Pointerfill *)c_ptr;
    
    if(pointerfill == (Pointerfill *)NULL)
	return;
    
    UFREE(pointerfill->fill);
    UFREE(pointerfill->string_rep);
    UFREE(pointerfill);
}

/*********************************************************************
 * 
 * pointerfill_match -- originally, two pointerfills were correct
 *   if what the key pointed to was aligned ("correctly") with what
 *   the response pointed to.  However, there may be "identical 
 *   instances" in the scenario template task;  the key pointerfill
 *   may point to one instance, and the response pointerfill
 *   may point to an instance which is aligned with another, 
 *   identical, key instance.  To get around this, we have every 
 *   instance point to its "representative" instance.  Then we can
 *   compare pointerfills by checking to see if the key pointerfill
 *   points to an instance whose representative is the same 
 *   as the representative of the instance aligned with what the 
 *   response pointerfill points to.  Is that clear?
 *
 ********************************************************************
 */

char
pointerfill_match(Pointerfill *key_pointerfill, 
		  Pointerfill *response_pointerfill,
		  Tallies *tallies)
{
  if(key_pointerfill
     &&
     key_pointerfill->fill
     &&
     response_pointerfill
     &&
     response_pointerfill->fill
     &&
     response_pointerfill->fill->its_pair
     &&
     response_pointerfill->fill->its_pair->key_inst
     &&
     (key_pointerfill->fill->its_rep_inst 
      == 
      response_pointerfill->fill->its_pair->key_inst->its_rep_inst)
     &&
     (response_pointerfill->fill->its_pair->status
      ==
      COR) )
    {
      tally1_correct( tallies );
      return COR;
    }
  tally1_incorrect( tallies );
  return INC;
}



/************************************************************************
 *									*
 * pointerfill_print
 *									*
 ***********************************************************************/

void
pointerfill_print(Pointerfill *pointerfill, int indent)
{
  printf("%s\n", pointerfill->string_rep );
}
