/*
 *  Copyright 1997 Science Applications International Corporation.
 */

/*
 * FILE 
 *	refmap.c
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *	support routines for Refmap data structure.
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)refmap.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  refmap_create
 *									*
 ***********************************************************************/

Refmap *
refmap_create(char *ref, Inst *inst)
{
    Refmap *refmap;
    
    if((refmap = UALLOC(Refmap, 1)) == (Refmap *)NULL)
    {
	fprintf(stderr, "Error: [refmap_create]: Allocation failed.\n");
	exit(1);
    }

    refmap->ref = STRALLOC(ref);
    refmap->inst = inst;
    return refmap;
}

/************************************************************************
 *									*
 *  refmap_release 
 *									*
 ***********************************************************************/

void
refmap_release( char *elt )
{
	Refmap *refmap = (Refmap *)elt;
	
	UFREE(refmap->ref);
	UFREE(refmap);
}


/************************************************************************
 *									*
 *  refmap_compare -- cast refmap->inst to int and return difference
 *									*
 ***********************************************************************/

int
refmap_compare(char *elt1, char *elt2)
{
    Refmap *refmap1 = (Refmap *)elt1;
    Refmap *refmap2 = (Refmap *)elt2;
    
    return muc_strcasecmp(refmap1->ref, refmap2->ref);
}


/************************************************************************
 *									*
 *  refmap_hash
 *									*
 ***********************************************************************/

long
refmap_hash( char *elt )
{
	Refmap *refmap = (Refmap *) elt;
	if( elt == NULL )
		return 0;
	else
		return cstring_case_hash(refmap->ref);
}


/************************************************************************
 *									*
 *  refmap_find_inst
 *									*
 ***********************************************************************/

Inst *
refmap_find_inst(TABLE *refmaps, char *ref)
{
	Refmap *refmap;
	Refmap *result;
	Inst *inst;
	
	refmap = refmap_create(ref, NULL);
	result = (Refmap *)table_find((char *) refmap, refmaps); 
	refmap_release((char *)refmap);
	if(result)
		return result->inst;
	return NULL;
}

void
refmap_print_table(TABLE *refmaps, int indent)
{
	Refmap *refmap;
	
	printf("%*.sRefmaps:\n", indent, "");
	table_first(refmaps);
	for(refmap=(Refmap *)table_next(refmaps);
	    refmap;
	    refmap=(Refmap *)table_next(refmaps))
		refmap_print(refmap, indent+2);
}

void
refmap_print(Refmap *refmap, int indent)	
{
	if( refmap == NULL )	
		printf("NULL refmap\n");
	else if( refmap->ref == NULL )
		printf("NULL refmap->ref\n");
	else
	{
		printf("%*.sREF: %s\n", indent, "", refmap->ref );
		inst_print(refmap->inst, indent+2);
	}
}

	
	
