/*
 * NAME
 * 
 * FILE 
 *     @(#)report_summary_IE.c	6.1\t09/08/97
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * SEE ALSO
 *
 * AUTHOR
 * 
 */

#ifdef SCCSID
static char SccsId[] = "@(#)report_summary_IE.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void display_clas_pairs(TABLE *clas_pairs, char *docnum);
static void display_inst_pairs(TABLE *inst_pairs,
			       char *class_name,
			       char *docnum);
static void display_inst_pair(Inst_pair *inst_pair,
			      char *class_name,
			      char *docnum);
static void build_inst_pair_status_string(char *status_string,
					  char status);
static char *build_inst_string(Inst *inst, 
			       char *class_name, 
			       char *docnum);
static void display_slot_pairs(TABLE *slot_pairs);
static void display_slot_pair(Slot_pair *slot_pair);
static char *build_1st_mfil_1st_sfil_line(Slot_pair *slot_pair,
					  Sfil_pair *sfil_pair);
static char *build_next_mfil_1st_sfil_line(Slot_pair *slot_pair,
					   Sfil_pair *sfil_pair);
static char *build_next_mfil_next_sfil_line(Slot_pair *slot_pair,
					    Sfil_pair *sfil_pair);
static void build_sfil_pair_status(char *status_string,
				   char status);
static char *get_sfil_fill(Sfil *sfil);

void
display_report_summary_IE(Catalog_entry *entry)
{
	display_clas_pairs(entry->clas_pairs, entry->docnum);
}

static
void
display_clas_pairs(TABLE *clas_pairs, char *docnum)
{
	Clas_pair *clas_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
		display_inst_pairs(clas_pair->inst_pairs,
				   clas_pair->name,
				   docnum);
}

static
void
display_inst_pairs(TABLE *inst_pairs,
		   char *class_name,
		   char *docnum)
{
	Inst_pair *inst_pair;
	
	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
		display_inst_pair(inst_pair,
				  class_name,
				  docnum);
}

static
void
display_inst_pair(Inst_pair *inst_pair,
		  char *class_name,
		  char *docnum)
{
	char status_string[5];
	char *key_inst_string = NULL;
	char *rsp_inst_string = NULL;
	
	build_inst_pair_status_string(status_string,
				      inst_pair->status);

	key_inst_string = build_inst_string(inst_pair->key_inst,
					    class_name,
					    docnum);

	rsp_inst_string = build_inst_string(inst_pair->response_inst,
					    class_name,
					    docnum);
	
	fprintf(MUC_report_summary_file,
		"%s %s %15s %s %-42s %s %s\n",
		status_string,
		MUC_report_field_separator,
		"",
		MUC_report_field_separator,
		key_inst_string,
		MUC_report_field_separator,
		rsp_inst_string
		);

	UFREE(key_inst_string);
	UFREE(rsp_inst_string);
	display_slot_pairs(inst_pair->slot_pairs);
}

static
void
build_inst_pair_status_string(char *status_string,
			      char status)
{
	switch(status)
	{
	case COR:
		sprintf(status_string,
			"COR");
		break;
	case PAR:
		sprintf(status_string,
			"PAR");
		break;
	case INC:
		sprintf(status_string,
			"INC");
		break;
	case MIS:
		sprintf(status_string,
			"MIS");
		break;
	case SPU:
		sprintf(status_string,
			"SPU");
		break;
	case OPT:
		sprintf(status_string,
			"OPT");
		break;
	case UNSCORED:
		sprintf(status_string,
			"UNS");
		break;
	case REMOVED:
		sprintf(status_string,
			"REM");
		break;
	case NON:
		sprintf(status_string,
			"NON");
		break;
	default:
		sprintf(status_string,
			"???");
		break;
	}
}

static
char *
build_inst_string(Inst *inst,
		  char *class_name,
		  char *docnum)
{
	char *inst_string;
	
	if( inst == NULL )
		inst_string = STRALLOC("");
	else
	{
		inst_string = (char *)calloc(5 +
					     strlen(class_name) +
					     strlen(docnum) +
					     strlen(inst->id),
					     sizeof(char *));
		if( inst_string==NULL )
			inst_string = STRALLOC("");
		else
			sprintf(inst_string,
				"<%s-%s-%s>",
				class_name,
				docnum,
				inst->id);
	}
	return inst_string;
}

static
void
display_slot_pairs(TABLE *slot_pairs)
{
	Slot_pair *slot_pair;
	
	table_first(slot_pairs);
	for(slot_pair=(Slot_pair *)table_next(slot_pairs);
	    slot_pair;
	    slot_pair=(Slot_pair *)table_next(slot_pairs))
		display_slot_pair(slot_pair);
}

static
void
display_slot_pair(Slot_pair *slot_pair)
{
	TABLE *mfil_pairs;
	Mfil_pair *mfil_pair;
	TABLE *sfil_pairs;
	Sfil_pair *sfil_pair;

	char *output_line;

	Bool first_mfil;
	Bool first_sfil;
	
	mfil_pairs = slot_pair->mfil_pairs;
	first_mfil=True;
	table_first(mfil_pairs);
	for(mfil_pair=(Mfil_pair *)table_next(mfil_pairs);
	    mfil_pair;
	    mfil_pair=(Mfil_pair *)table_next(mfil_pairs))
	{
		sfil_pairs=mfil_pair->sfil_pairs;
		first_sfil = True;
		table_first(sfil_pairs);
		for(sfil_pair=(Sfil_pair *)table_next(sfil_pairs);
		    sfil_pair;
		    sfil_pair=(Sfil_pair *)table_next(sfil_pairs))
		{
			if(first_mfil
			   &&
			   first_sfil)
				output_line = 
					build_1st_mfil_1st_sfil_line(slot_pair,
								     sfil_pair);
			else if( first_sfil )
				output_line = 
					build_next_mfil_1st_sfil_line(slot_pair,
								      sfil_pair);
			else
				output_line =
					build_next_mfil_next_sfil_line(slot_pair,
								       sfil_pair);
			fprintf(MUC_report_summary_file,
				"%s\n",
				output_line);
			UFREE(output_line);
			first_sfil = False;
		}
		first_mfil = False;
	}
}
		
static
char *
build_1st_mfil_1st_sfil_line(Slot_pair *slot_pair,
			     Sfil_pair *sfil_pair)
{
	char pair_status[5];
	char *slotname;
	char key_optmark;
	char rsp_optmark;
	char *key_fill;
	char *rsp_fill;
	long output_line_length;
	long key_length;
	long rsp_length;
	char *output_line;
	
	/* how did sfils compare? */
	build_sfil_pair_status(pair_status,
			       sfil_pair->status);

	/* get key and response fill */
	key_fill = get_sfil_fill(sfil_pair->key_sfil);
	rsp_fill = get_sfil_fill(sfil_pair->response_sfil);

	slotname = (char *)calloc(strlen(slot_pair->name) + 2,
				  sizeof(char *));
	sprintf(slotname, "%s:", slot_pair->name);
	
	/* if key says optional slot, print slash */
	if(slot_pair->key_slot
	   &&
	   slot_pair->key_slot->is_optional)
		key_optmark = '/';
	else
		key_optmark = ' ';

	/* if response says optional print slash */
	if(slot_pair->response_slot
	   &&
	   slot_pair->response_slot->is_optional)
		rsp_optmark = '/';		
	else
		rsp_optmark = ' ';	

	/* calculate output line length */
	output_line_length = 25;
	key_length = (key_fill?strlen(key_fill):0);
	rsp_length = (rsp_fill?strlen(rsp_fill):0);
	if( 52 > strlen(slotname) + key_length + rsp_length)
		output_line_length += 52;
	else
		output_line_length += (strlen(slotname) 
				       + 
				       key_length
				       +
				       rsp_length);
	
	/* allocate output line */
	output_line = (char *)calloc(output_line_length,
				     sizeof(char *));

	sprintf(output_line, 
		"%s %s %-15s %s     %c%-37s %s     %c%s",
		pair_status,
		MUC_report_field_separator,
		slotname,
		MUC_report_field_separator,
		key_optmark,
		(key_fill?key_fill:""),
		MUC_report_field_separator,
		rsp_optmark,
		(rsp_fill?rsp_fill:""));

	UFREE(slotname);
	return output_line;
}

static
char *
build_next_mfil_1st_sfil_line(Slot_pair *slot_pair,
			      Sfil_pair *sfil_pair)
{
	char pair_status[5];
	char *slotname;
	char key_optmark;
	char rsp_optmark;
	char *key_fill;
	char *rsp_fill;
	long output_line_length;
	long key_length;
	long rsp_length;
	char *output_line;
	
	/* how did sfils compare? */
	build_sfil_pair_status(pair_status,
			       sfil_pair->status);

	/* get key and response fill */
	key_fill = get_sfil_fill(sfil_pair->key_sfil);
	rsp_fill = get_sfil_fill(sfil_pair->response_sfil);

	/* calculate output line length */
	output_line_length = 25;
	key_length = (key_fill?strlen(key_fill):0);
	rsp_length = (rsp_fill?strlen(rsp_fill):0);
	if( 67 > key_length + rsp_length)
		output_line_length += 67;
	else
		output_line_length += (key_length
				       +
				       rsp_length);
	
	/* allocate output line */
	output_line = (char *)calloc(output_line_length,
				     sizeof(char *));

	/* put it all together */
	sprintf(output_line, 
		"%s %s %15s %s     %c%-37s %s     %c%s",
		pair_status,
		MUC_report_field_separator,
		"",
		MUC_report_field_separator,
		(key_fill?'/':' '),
		(key_fill?key_fill:""),
		MUC_report_field_separator,
		(rsp_fill?'/':' '),
		(rsp_fill?rsp_fill:""));

	return output_line;
}

static
char *
build_next_mfil_next_sfil_line(Slot_pair *slot_pair,
			       Sfil_pair *sfil_pair)
{
	char pair_status[5];
	char *slotname;
	char key_optmark;
	char rsp_optmark;
	char *key_fill;
	char *rsp_fill;
	long output_line_length;
	long key_length;
	long rsp_length;
	char *output_line;
	
	/* how did sfils compare? */
	build_sfil_pair_status(pair_status,
			       sfil_pair->status);

	/* get key and response fill */
	key_fill = get_sfil_fill(sfil_pair->key_sfil);
	rsp_fill = get_sfil_fill(sfil_pair->response_sfil);

	/* calculate output line length */
	output_line_length = 25;
	key_length = (key_fill?strlen(key_fill):0);
	rsp_length = (rsp_fill?strlen(rsp_fill):0);
	if( 67 > key_length + rsp_length)
		output_line_length += 67;
	else
		output_line_length += (key_length
				       +
				       rsp_length);
	
	/* allocate output line */
	output_line = (char *)calloc(output_line_length,
				     sizeof(char *));

	/* put it all together */
	sprintf(output_line, 
		"%s %s %15s %s      %-37s %s      %s",
		pair_status,
		MUC_report_field_separator,
		"",
		MUC_report_field_separator,
		(key_fill?key_fill:""),
		MUC_report_field_separator,
		(rsp_fill?rsp_fill:""));

	return output_line;
}

static
void
build_sfil_pair_status(char *status_string, char status)
{
	switch(status)
	{
	case COR:
		sprintf(status_string,
			"cor");
		break;
	case PAR:
		sprintf(status_string,
			"par");
		break;
	case INC:
		sprintf(status_string,
			"inc");
		break;
	case MIS:
		sprintf(status_string,
			"mis");
		break;
	case SPU:
		sprintf(status_string,
			"spu");
		break;
	case OPT:
		sprintf(status_string,
			"opt");
		break;
	case UNSCORED:
		sprintf(status_string,
			"uns");
		break;
	case REMOVED:
		sprintf(status_string,
			"rem");
		break;
	case NON:
		sprintf(status_string,
			"non");
		break;
	default:
		sprintf(status_string,
			"???");
		break;
	}
}


static
char *
get_sfil_fill(Sfil *sfil)
{
	if( sfil == NULL )
		return NULL;
	switch(sfil->type)
	{
	case POINTER_FILL:
		if( sfil->fill.ptr )
			return sfil->fill.ptr->string_rep;
		else
			return NULL;
	case STRING_FILL:
		if( sfil->fill.str )
			return sfil->fill.str->fill;
		else
			return NULL;
	case SET_FILL:
		if( sfil->fill.set )
			return sfil->fill.set->fill;
		else
			return NULL;
	}
}

