/*
 * NAME
 * 
 * FILE 
 *     @(#)report_summary_NE.c	6.1	09/08/97
 *
 * SYNOPSIS
 *
 * DESCRIPTION
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * SEE ALSO
 *
 * AUTHOR
 * 
 */

#ifdef SCCSID
static char SccsId[] = "@(#)report_summary_NE.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void get_COR_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs);
static void get_MIS_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs);
static void get_SPU_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs);
static void get_OPT_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs);
static void display_inst_pairs(TABLE *inst_pairs);
static void display_inst_pair(Inst_pair *inst_pair);
static Slot_pair *get_TEXT_slot_pair(TABLE *slot_pairs);
static Slot_pair *get_TYPE_slot_pair(TABLE *slot_pairs);
static Sfil_pair *get_scored_sfil_pair(TABLE *mfil_pairs);
static void display_status(char status);
static void display_tag_name(Inst_pair *inst_pair);


void
display_report_summary_NE(Catalog_entry *entry)
{
	TABLE *all_inst_pairs;
	
	fprintf(MUC_report_summary_file,
		"---------------------------------------------------------------------------------------------------------\n");
	fprintf(MUC_report_summary_file,
		"Document %s\n",
		entry->docnum);
	fprintf(MUC_report_summary_file,
		"%s%s",
		"TAG        TYPE TEXT KEY_TYPE     RSP_TYPE     KEY_TEXT                                          RSP_TEXT\n",
		"---------------------------------------------------------------------------------------------------------\n");

	all_inst_pairs = table_list_create(default_compar,
					   NULL,
					   default_destroy);
	get_COR_inst_pairs(all_inst_pairs, entry->clas_pairs);
	get_MIS_inst_pairs(all_inst_pairs, entry->clas_pairs);
	get_SPU_inst_pairs(all_inst_pairs, entry->clas_pairs);
	get_OPT_inst_pairs(all_inst_pairs, entry->clas_pairs);
	
	display_inst_pairs(all_inst_pairs);

	table_destroy(all_inst_pairs);
}

static
void
get_COR_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	TABLE *inst_pairs;
	Inst_pair *inst_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair=(Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair=(Inst_pair *)table_next(inst_pairs))
			if(inst_pair->status == COR)
				table_append((char *)inst_pair, 
					     all_inst_pairs);
	}
}

static
void
get_MIS_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	TABLE *inst_pairs;
	Inst_pair *inst_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair=(Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair=(Inst_pair *)table_next(inst_pairs))
			if(inst_pair->status == MIS)
				table_append((char *)inst_pair, 
					     all_inst_pairs);
	}
}

static
void
get_SPU_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	TABLE *inst_pairs;
	Inst_pair *inst_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair=(Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair=(Inst_pair *)table_next(inst_pairs))
			if(inst_pair->status == SPU)
				table_append((char *)inst_pair, 
					     all_inst_pairs);
	}
}

static
void
get_OPT_inst_pairs(TABLE *all_inst_pairs, TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	TABLE *inst_pairs;
	Inst_pair *inst_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair=(Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair=(Inst_pair *)table_next(inst_pairs))
			if(inst_pair->status == OPT)
				table_append((char *)inst_pair, 
					     all_inst_pairs);
	}
}

static
void
display_inst_pairs(TABLE *inst_pairs)
{
	Inst_pair *inst_pair;

	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
		display_inst_pair(inst_pair);
}


static
void
display_inst_pair(Inst_pair *inst_pair)
{
	TABLE *slot_pairs;
	Slot_pair *text_slot_pair;
	Slot_pair *type_slot_pair;
	
	Sfil_pair *type_sfil_pair;
	Sfil_pair *text_sfil_pair;
	
	char text_status = NON;
	char type_status = NON;
	
	char *key_type;
	char *resp_type;
	
	char *key_text;
	char *resp_text;
	
	char *text_buffer;
	
	text_slot_pair = get_TEXT_slot_pair(inst_pair->slot_pairs);
	type_slot_pair = get_TYPE_slot_pair(inst_pair->slot_pairs);
	
	if( text_slot_pair )
		text_sfil_pair = 
			get_scored_sfil_pair(text_slot_pair->mfil_pairs);
	else
		text_sfil_pair = NULL;
	
	if( type_slot_pair )
		type_sfil_pair = 
			get_scored_sfil_pair(type_slot_pair->mfil_pairs);
	else
		type_sfil_pair = NULL;
	
	
	if( text_sfil_pair )
	{
		text_status = text_sfil_pair->status;
		
		if(text_sfil_pair->key_sfil
		   &&
		   text_sfil_pair->key_sfil->fill.str)
			key_text = text_sfil_pair->key_sfil->fill.str->fill;
		else
			key_text = NULL;
		
		if(text_sfil_pair->response_sfil
		   &&
		   text_sfil_pair->response_sfil->fill.str)
			resp_text = text_sfil_pair->response_sfil->fill.str->fill;
		else
			resp_text = NULL;
	}
	else
	{
		key_text = NULL;
		resp_text = NULL;
		text_status = NON;
	}
	
	if( type_sfil_pair )
	{
		type_status = type_sfil_pair->status;
		
		if(type_sfil_pair->key_sfil
		   &&
		   type_sfil_pair->key_sfil->fill.set)
			key_type = type_sfil_pair->key_sfil->fill.set->fill;
		else
			key_type = NULL;
		
		if(type_sfil_pair->response_sfil
		   &&
		   type_sfil_pair->response_sfil->fill.set)
			resp_type = type_sfil_pair->response_sfil->fill.set->fill;
		else
			resp_type = NULL;
	}
	else
	{
		key_type = NULL;
		resp_type = NULL;
		type_status = NON;
	}

	display_tag_name(inst_pair);
	
	display_status(type_status);
	fprintf(MUC_report_summary_file, "  ");
	display_status(text_status);
	fprintf(MUC_report_summary_file, "  ");
	
	fprintf(MUC_report_summary_file, "%-13s", ( key_type ? key_type : ""));
	fprintf(MUC_report_summary_file, "%-13s", ( resp_type ? resp_type: ""));
	
	if( key_text )
	{
		text_buffer = (char *)calloc(2 + strlen(key_text), sizeof(char *));
		sprintf(text_buffer, "\"");
		strcat(text_buffer, key_text);
		strcat(text_buffer, "\"");		
		fprintf(MUC_report_summary_file, "%-50s", text_buffer);
		UFREE(text_buffer);
	}
	else
		fprintf(MUC_report_summary_file, "%-50s", "\"\"");
	
	if( resp_text )
	{
		text_buffer = (char *)calloc(2 + strlen(resp_text), sizeof(char *));
		sprintf(text_buffer, "\"");
		strcat(text_buffer, resp_text);
		strcat(text_buffer, "\"");		
		fprintf(MUC_report_summary_file, "%s", text_buffer);
		UFREE(text_buffer);
	}
	else
		fprintf(MUC_report_summary_file, "%s", "\"\"");
	fprintf(MUC_report_summary_file, "\n");
}	

static
Slot_pair *
get_TEXT_slot_pair(TABLE *slot_pairs)
{
	Slot_pair *slot_pair;
	char *slotname;
	
	slotname = option_find_first_value(MUC_Options, ":sgml_TEXT_slot");
	if(slotname == NULL)
		return NULL;
	else
		return slot_pair_find(slot_pairs, slotname);
}

static
Slot_pair *
get_TYPE_slot_pair(TABLE *slot_pairs)
{
	Slot_pair *slot_pair;
	char *slotname;
	
	slotname = option_find_first_value(MUC_Options, ":sgml_TYPE_slot");
	if(slotname == NULL)
		return NULL;
	else
		return slot_pair_find(slot_pairs, slotname);
}

/* returns the first sfil pair with a status flag that
   indicates it contributed to the POS/ACT/COR score */

static
Sfil_pair *
get_scored_sfil_pair(TABLE *mfil_pairs)
{
	Mfil_pair *mfil_pair;
	TABLE *sfil_pairs;
	Sfil_pair *sfil_pair;
	
	table_first(mfil_pairs);
	for(mfil_pair=(Mfil_pair *)table_next(mfil_pairs);
	    mfil_pair;
	    mfil_pair=(Mfil_pair *)table_next(mfil_pairs))
	{
		sfil_pairs = mfil_pair->sfil_pairs;
		table_first(sfil_pairs);
		for(sfil_pair=(Sfil_pair *)table_next(sfil_pairs);
		    sfil_pair;
		    sfil_pair=(Sfil_pair *)table_next(sfil_pairs))
			if((sfil_pair->status==COR)
			   ||
			   (sfil_pair->status==INC)
			   ||
			   (sfil_pair->status==PAR)
			   ||
			   (sfil_pair->status==MIS)
			   ||
			   (sfil_pair->status==SPU)
			   ||
			   (sfil_pair->status==OPT))
				return sfil_pair;
	}
	return NULL;
}

static
void
display_status(char status)
{
	char *s;
	
	switch(status)
	{
	case COR:
		fprintf(MUC_report_summary_file,
			"cor");
		break;
	case INC:
		fprintf(MUC_report_summary_file,
			"inc");
		break;
	case PAR:
		fprintf(MUC_report_summary_file,
			"par");
		break;
	case MIS:
		fprintf(MUC_report_summary_file,
			"mis");
		break;
	case SPU:
		fprintf(MUC_report_summary_file,
			"spu");
		break;
	case OPT:
		fprintf(MUC_report_summary_file,
			"opt");
		break;
	case UNSCORED:
		fprintf(MUC_report_summary_file,
			"uns");
		break;
	case NON:
		fprintf(MUC_report_summary_file,
			"non");
		break;
	default:
		fprintf(MUC_report_summary_file,
			"   ");
		break;
	}
}

static
void
display_tag_name(Inst_pair *inst_pair)
{
	char *name;
	
	if( inst_pair->key_inst )
		name = inst_pair->key_inst->name;
	else if( inst_pair->response_inst )
		name = inst_pair->response_inst->name;
	else
		name = NULL;
	
	fprintf(MUC_report_summary_file,
		"%-10s ",
		(name ? name : ""));
}

