/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	score.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	scoring functions for the MUC scorer.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)score.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/*  internal function prototypes */

static void score_entry_class_scores(TABLE* clas_pairs, TABLE* class_scores);
static void mark_unscored_objs_classes(TABLE *clas_pairs);
static void mark_unscored_objs_insts(TABLE *inst_pairs);
static void score_obj_level(TABLE *inst_pairs, Tallies *tallies);
static void score_entry_slot_scores(TABLE* inst_pairs, TABLE* slot_scores);
static void score_entry_text_filtering(TABLE* clas_pairs, 
				       Tallies* text_filtering);
static void score_entry_ne_subtasks(Catalog_entry *entry);
static void add_slot_tallies_to_entry_subtask_tallies(Catalog_entry *entry,
						      Clas_pair *clas_pair, 
						      Slot_pair *slot_pair);
static void score_entry_ne_doc_sections(Catalog_entry *entry);
static void score_entry_dsgs(Catalog_entry *entry);
static void score_clas_dsgs(TABLE *clas_pairs,
			    TABLE *dsg_tallies);
static void score_inst_dsgs(TABLE *inst_pairs,
			    TABLE *dsg_tallies);
static void add_entry_to_totals(Catalog_entry* entry);
static void add_entry_class_scores_to_totals(Catalog_entry* entry);
static void add_entry_slot_scores_to_totals(TABLE *total_slot_scores, 
					    TABLE *entry_slot_scores);
static void add_entry_total_slot_scores_to_totals(Catalog_entry *entry);
static void add_entry_text_filtering_to_totals(Catalog_entry *entry);
static void add_entry_ne_subtasks_to_totals(Catalog_entry *entry);
static void add_entry_ne_doc_sections_to_totals(Catalog_entry *entry);
static void add_entry_dsgs_to_totals(Catalog_entry *entry);


void
score_entry_IE(Catalog_entry *entry)
{
	score_entry_class_scores(entry->clas_pairs, 
				 entry->class_scores);
	if( IS_ST )	
		score_entry_text_filtering(entry->clas_pairs, 
					   entry->text_filtering);
	if( IS_NE )
	{
		score_entry_ne_subtasks(entry);
		score_entry_ne_doc_sections(entry);
		score_entry_dsgs(entry);
	}
	add_entry_to_totals(entry);
}

/*
 *  score_entry_class_scores -- add up the COR, INC, etc *insts*
 *  of each class, and 
 */


static
void
score_entry_class_scores(TABLE* clas_pairs, TABLE* class_scores)
{
	Class_score *class_score;
	Clas_pair *clas_pair;

	mark_unscored_objs_classes(clas_pairs);

	table_first(class_scores);
	for(class_score = (Class_score*)table_next(class_scores);
	    class_score;
	    class_score = (Class_score*)table_next(class_scores))
	{	
		clas_pair = clas_pair_find(clas_pairs, class_score->name);

		/* get "object level" tallies */
		score_obj_level(clas_pair->inst_pairs,
				class_score->tallies);

		/* get tallies for each slot in a clas */
		score_entry_slot_scores(clas_pair->inst_pairs,
					class_score->slot_scores);
	}
}

/*************************************************************/

static
void
mark_unscored_objs_classes(TABLE *clas_pairs)
{
	TABLE *unscored_obj_names;
	char *unscored_obj_name;
	Clas_pair *clas_pair;
	
	unscored_obj_names = option_find_values(MUC_Options,
						":unscored_objects");
	if( unscored_obj_names )
	{
		table_first(unscored_obj_names);
		for(unscored_obj_name=table_next(unscored_obj_names);
		    unscored_obj_name;
		    unscored_obj_name=table_next(unscored_obj_names))
		{
			clas_pair = clas_pair_find(clas_pairs,
						   unscored_obj_name);
			if( clas_pair )
				mark_unscored_objs_insts(clas_pair->inst_pairs);
		}
	}
}

/*************************************************************/

static
void
mark_unscored_objs_insts(TABLE *inst_pairs)
{
	Inst_pair *inst_pair;
	
	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
		inst_pair->status = UNSCORED;
}

/*************************************************************/

static
void
score_obj_level(TABLE *inst_pairs, Tallies *tallies)
{
	Inst_pair *inst_pair;
	
	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
		switch( inst_pair->status )
		{
		case COR:
			tally1_correct(tallies);
			break;
		case PAR:
			tally1_partial(tallies);
			break;
		case INC:
			tally1_incorrect(tallies);
			break;
		case MIS:
			tally1_missing(tallies);
			break;
		case SPU:
			tally1_spurious(tallies);
			break;
		default:
			tally1_noncommittal(tallies);
			break;
		}
}

/*************************************************************/

static
void
score_entry_slot_scores(TABLE* inst_pairs, TABLE* slot_scores)
{
	Slot_score *slot_score;
	Inst_pair  *inst_pair;
	Slot_pair  *slot_pair;

	table_first(slot_scores);
	for(slot_score = (Slot_score*)table_next(slot_scores);
	    slot_score;
	    slot_score = (Slot_score*)table_next(slot_scores))
	{	
		tallies_clear(slot_score->tallies);
		table_first(inst_pairs);
		for(inst_pair = (Inst_pair*)table_next(inst_pairs);
		    inst_pair;
		    inst_pair = (Inst_pair*)table_next(inst_pairs))
		{	
			slot_pair = slot_pair_find(inst_pair->slot_pairs,
						   slot_score->name);
			if(slot_pair)
				tallies_combine(slot_score->tallies,
						slot_pair->tallies);
		}
	}
}

/*************************************************************/

static
void
score_entry_text_filtering(TABLE* clas_pairs, Tallies* text_filtering)
{
	Clas_pair *clas_pair;
	Inst_pair *inst_pair;
	Slot_pair *slot_pair;
	int       key_content_size;
	int       response_content_size;
	char      *template_name;
	char      *content_name;

	template_name = option_find_first_value(MUC_Options, 
						":template_name");
	content_name = option_find_first_value(MUC_Options,
					       ":content_name");

	clas_pair = clas_pair_find(clas_pairs, template_name);
	if( clas_pair == NULL )
	{
		tally1_correct(text_filtering);
		return;
	}
	
	table_first(clas_pair->inst_pairs);
	inst_pair = (Inst_pair*)table_next(clas_pair->inst_pairs);
	if( inst_pair == NULL )
	{
		tally1_correct(text_filtering);
		return;
	}
	
	slot_pair = slot_pair_find(inst_pair->slot_pairs, content_name);
	if( slot_pair == NULL )
	{
		tally1_correct(text_filtering);
		return;
	}
	
	
	if( slot_pair->key_slot == NULL )
		key_content_size = 0;
	else
		key_content_size = 
			table_count(slot_pair->key_slot->mfils);

	if( slot_pair->response_slot == NULL )
		response_content_size = 0;
	else
		response_content_size = 
			table_count(slot_pair->response_slot->mfils);

	if ((key_content_size == 0)
	    &&
	    (response_content_size == 0))
		tally1_correct(text_filtering);

	else if((key_content_size > 0)
		&&	
		(response_content_size == 0))
		tally1_incorrect(text_filtering);

	else if ((key_content_size == 0)
		 &&	
		 (response_content_size > 0))
		tally1_incorrect(text_filtering);
	else
		tally1_correct(text_filtering);
}

/*************************************************************/

static
void
score_entry_ne_subtasks(Catalog_entry *entry)
{
	TABLE *clas_pairs;
	Clas_pair *clas_pair;
	TABLE *inst_pairs;
	Inst_pair *inst_pair;
	TABLE *slot_pairs;
	Slot_pair *slot_pair;
	
	clas_pairs = entry->clas_pairs;
	table_first(clas_pairs);
	for(clas_pair = (Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair = (Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair = (Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair = (Inst_pair *)table_next(inst_pairs))
		{
			slot_pairs = inst_pair->slot_pairs;
			table_first(slot_pairs);
			for(slot_pair = (Slot_pair *)table_next(slot_pairs);
			    slot_pair;
			    slot_pair = (Slot_pair *)table_next(slot_pairs))
                                add_slot_tallies_to_entry_subtask_tallies(entry, 
									  clas_pair,
									  slot_pair);
		}	
	}
}

/*************************************************************/

static
void
add_slot_tallies_to_entry_subtask_tallies(Catalog_entry *entry,
					  Clas_pair *clas_pair, 
					  Slot_pair *slot_pair)
{
	char *subtask_string;
	TABLE *mfil_pairs;
	Mfil_pair *mfil_pair;
	TABLE *sfil_pairs;
	Sfil_pair *sfil_pair;
	Sfil *sfil;
	char *fill;
	Named_tallies *named_tallies;

	mfil_pairs = slot_pair->mfil_pairs;
	table_first(mfil_pairs);
	for(mfil_pair=(Mfil_pair *)table_next(mfil_pairs);
	    mfil_pair;
	    mfil_pair=(Mfil_pair *)table_next(mfil_pairs))
	{
		sfil_pairs = mfil_pair->sfil_pairs;
		table_first(sfil_pairs);
		for(sfil_pair=(Sfil_pair *)table_next(sfil_pairs);
		    sfil_pair;
		    sfil_pair=(Sfil_pair *)table_next(sfil_pairs))
		{
			if( sfil_pair->key_sfil )
				sfil = sfil_pair->key_sfil;
			else if( sfil_pair->response_sfil )
				sfil = sfil_pair->response_sfil;
			else
				continue;

			if( sfil->fill.set == NULL )
				continue;
			
			switch(sfil->type)
			{
			case SET_FILL:
				fill=sfil->fill.set->fill;
				break;
			case STRING_FILL:
				fill=sfil->fill.str->fill;
				break;
			case POINTER_FILL:
				fill=sfil->fill.ptr->string_rep;
				break;
			default:
				continue;
			}
	
			subtask_string=(char *)malloc(strlen(clas_pair->name) +
						      strlen(slot_pair->name) + 
						      strlen(fill) +
						      3);
			strcpy(subtask_string, clas_pair->name);
			strcat(subtask_string, " ");
			strcat(subtask_string, slot_pair->name);
			strcat(subtask_string, " ");
			strcat(subtask_string, fill);

			named_tallies = 
				named_tallies_find(entry->ne_subtask_tallies,
						   subtask_string);

			UFREE( subtask_string );

			if( named_tallies )
				tallies_combine(named_tallies->tallies,
						sfil_pair->tallies);
		}
	}
}	
	

/*************************************************************/

static
void
score_entry_ne_doc_sections(Catalog_entry *entry)
{
	TABLE *clas_pairs;
	TABLE *inst_pairs;
	Clas_pair *clas_pair;
	Inst_pair *inst_pair;
	Inst *inst;
	char *doc_section;
	Named_tallies *doc_section_tallies;
	
	if( entry == NULL ) return;

	clas_pairs = entry->clas_pairs;
	table_first(clas_pairs);
	for(clas_pair = (Clas_pair *)table_next(clas_pairs);
	    clas_pair;
	    clas_pair = (Clas_pair *)table_next(clas_pairs))
	{
		inst_pairs = clas_pair->inst_pairs;
		table_first(inst_pairs);
		for(inst_pair = (Inst_pair *)table_next(inst_pairs);
		    inst_pair;
		    inst_pair = (Inst_pair *)table_next(inst_pairs))
		{
			if( inst_pair->key_inst )
				inst = inst_pair->key_inst;
			else if ( inst_pair->response_inst )
				inst = inst_pair->response_inst;
			else
				continue;
			
			doc_section = inst->doc_section;
			if(doc_section == NULL) 
				continue;
			
			doc_section_tallies = 
				named_tallies_find(entry->ne_doc_section_tallies,
						   doc_section);
			if( doc_section_tallies )
				tallies_combine(doc_section_tallies->tallies,
						inst_pair->tallies);
		}
	}
}

/*************************************************************/

/* doc section groups */

static
void
score_entry_dsgs(Catalog_entry *entry)
{
	if( entry->dsg_tallies )
		score_clas_dsgs(entry->clas_pairs, 
				entry->dsg_tallies);
}

static
void
score_clas_dsgs(TABLE *clas_pairs,
		TABLE *dsg_tallies)
{
	Clas_pair *clas_pair;
	
	table_first(clas_pairs);
	for(clas_pair=(Clas_pair *)table_next(clas_pairs);	
	    clas_pair;
	    clas_pair=(Clas_pair *)table_next(clas_pairs))
		score_inst_dsgs(clas_pair->inst_pairs,
				dsg_tallies);
}

static
void
score_inst_dsgs(TABLE *inst_pairs,
		TABLE *dsg_tallies)
{
	Inst_pair *inst_pair;
	Inst *inst;
	char *doc_section;
	Named_tallies *dsg_tally;
	char *p;
	

	table_first(inst_pairs);
	for(inst_pair=(Inst_pair *)table_next(inst_pairs);
	    inst_pair;
	    inst_pair=(Inst_pair *)table_next(inst_pairs))
	{
		if(inst_pair->key_inst )
			inst = inst_pair->key_inst;
		else if(inst_pair->response_inst)
			inst = inst_pair->response_inst;
		else
			continue;

		doc_section = inst->doc_section;
		if( doc_section == NULL )
			continue;
		
		table_first(dsg_tallies);
		for(dsg_tally=(Named_tallies *)table_next(dsg_tallies);
		    dsg_tally;
		    dsg_tally=(Named_tallies *)table_next(dsg_tallies))
		{
			p = dsg_tally->name;
			/* skip over group's name */
			p += strcspn(p, " "); 
			if( case_strstr(p, doc_section) )
				tallies_combine(dsg_tally->tallies,
						inst_pair->tallies);
		}
	}
}

/*************************************************************/

static
void
add_entry_to_totals(Catalog_entry* entry)
{
		add_entry_class_scores_to_totals(entry);
		add_entry_total_slot_scores_to_totals(entry);
		if( IS_ST )
			add_entry_text_filtering_to_totals(entry);
		if( IS_NE )
		{
			add_entry_ne_subtasks_to_totals(entry);
			add_entry_ne_doc_sections_to_totals(entry);
			add_entry_dsgs_to_totals(entry);
		}
}

/*************************************************************/

static
void
add_entry_class_scores_to_totals(Catalog_entry* entry)
{
	TABLE *entry_class_scores;
	Class_score *total_class_score;
	Class_score *entry_class_score;
	
	entry_class_scores = entry->class_scores;

	table_first(entry_class_scores);
	for(entry_class_score = (Class_score *)table_next(entry_class_scores);
	    entry_class_score;
	    entry_class_score = (Class_score *)table_next(entry_class_scores))
	{
		total_class_score = class_score_find(MUC_Summary_Class_scores, 
						     entry_class_score->name);
		tallies_combine(total_class_score->tallies,
				entry_class_score->tallies);
		add_entry_slot_scores_to_totals(total_class_score->slot_scores,
					        entry_class_score->slot_scores);
	}
}

/*************************************************************/

static
void
add_entry_slot_scores_to_totals(TABLE *total_slot_scores, 
				TABLE *entry_slot_scores)
{
	Slot_score *total_slot_score;
	Slot_score *entry_slot_score;
	
	table_first(entry_slot_scores);
	for(entry_slot_score = (Slot_score *)table_next(entry_slot_scores);
	    entry_slot_score;
	    entry_slot_score = (Slot_score *)table_next(entry_slot_scores))
	{
		total_slot_score = slot_score_find(total_slot_scores, 
						   entry_slot_score->name);
		tallies_combine(total_slot_score->tallies,
				entry_slot_score->tallies);
	}
}


/*************************************************************/
	
static
void
add_entry_total_slot_scores_to_totals(Catalog_entry *entry)
{
	tallies_combine(MUC_Total_Slot_tallies, entry->total_slot_tallies);
}

/*************************************************************/

static
void
add_entry_text_filtering_to_totals(Catalog_entry *entry)
{
	tallies_combine(MUC_Total_Text_Filtering, entry->text_filtering);
}

/*************************************************************/

static
void
add_entry_ne_subtasks_to_totals(Catalog_entry *entry)
{
	if(entry==NULL) return;
	named_tallies_table_combine(MUC_ne_subtask_tallies,
				    entry->ne_subtask_tallies);
}


/*************************************************************/

static
void
add_entry_ne_doc_sections_to_totals(Catalog_entry *entry)
{
	if(entry==NULL) return;
	named_tallies_table_combine(MUC_ne_doc_section_tallies,
				    entry->ne_doc_section_tallies);
}
	

/*************************************************************/

static
void
add_entry_dsgs_to_totals(Catalog_entry *entry)
{
	if(entry==NULL) return;
	if(entry->dsg_tallies && MUC_dsg_tallies)
		named_tallies_table_combine(MUC_dsg_tallies,
					    entry->dsg_tallies);
}
	

/*************************************************************/

	

