/*
 * FILE 
 *	@(#)score_coref.c	6.1\t09/08/97
 *     
 * SYNOPSIS
 *
 * DESCRIPTION
 *
 * DIAGNOSTICS
 *
 * FILES
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)score_coref.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

static void add_entry_to_totals(Catalog_entry *entry);
static TABLE *get_coref_insts(TABLE *classes);
static TABLE *get_coref_inst_pairs(TABLE *clas_pairs);
static Eqclass *establish_new_eqclass(Inst *first_member, 
				    Eqclasset *eqclasset );
static void add_eqclass_member(Inst *new_member,
			       Eqclass *eqclass,
			       Eqclasset *eqclasset);
static void make_equivalent(Inst *inst1, Inst *inst2);
static Inst * find_eqclass_rep(Inst *inst);
static Eqclasset *compute_equivalence_classes(TABLE *insts);
static Eqclasset *fix_key_eqclasset( Eqclasset *eqclasset );
static Eqclasset *fix_response_eqclasset( Eqclasset *eqclasset );
static Bool only_inst_is_not_aligned(TABLE *insts);
static void make_new_eqclass_from_insts(TABLE *insts, 
					Eqclasset *eqclasset);


void
score_entry_coref(Catalog_entry *entry)
{
	TABLE *key_insts;
	TABLE *response_insts;
	TABLE *inst_pairs;
	Eqclasset *fixed_key_eqclasset;
	Eqclasset *fixed_response_eqclasset;
	
	key_insts = get_coref_insts(entry->key_classes);
	response_insts = get_coref_insts(entry->response_classes);
	inst_pairs = get_coref_inst_pairs(entry->clas_pairs);
	
	entry->key_eqclasset = compute_equivalence_classes(key_insts);
	entry->response_eqclasset = compute_equivalence_classes(response_insts);
	
	fixed_key_eqclasset = fix_key_eqclasset(entry->key_eqclasset);
	fixed_response_eqclasset = fix_response_eqclasset(entry->response_eqclasset);
	
	entry->partitionset = partitionset_create(fixed_key_eqclasset,	
						  fixed_response_eqclasset,
						  inst_pairs);
	
	partitionset_score_recall(entry->partitionset,
				  &(entry->recall_numerator),
				  &(entry->recall_denominator));
	
	partitionset_score_precision(entry->partitionset,
				     &(entry->precision_numerator),
				     &(entry->precision_denominator));

	eqclasset_release((char *)fixed_key_eqclasset);
	eqclasset_release((char *)fixed_response_eqclasset);
	
	add_entry_to_totals(entry);
}

static
void
add_entry_to_totals(Catalog_entry *entry)
{
	MUC_recall_numerator += entry->recall_numerator;
	MUC_recall_denominator += entry->recall_denominator;
	MUC_precision_numerator += entry->precision_numerator;
	MUC_precision_denominator += entry->precision_denominator;

	MUC_total_key_eqclasses += entry->partitionset->num_key_classes;
	MUC_total_response_eqclasses += entry->partitionset->num_resp_classes;
	tallies_combine(MUC_Total_Slot_tallies, entry->total_slot_tallies);
}


static 
TABLE *
get_coref_insts(TABLE *classes)
{
	Clas *clas;
	
	if( classes == NULL ) return NULL;
	table_first(classes);
	clas = (Clas *)table_next(classes);
	
	if(clas == NULL) return NULL;
	return clas->insts;
}

static 
TABLE *
get_coref_inst_pairs(TABLE *clas_pairs)
{
	Clas_pair *clas_pair;
	
	if( clas_pairs == NULL ) return NULL;
	table_first(clas_pairs);
	clas_pair = (Clas_pair *)table_next(clas_pairs);
	
	if(clas_pair == NULL) return NULL;
	return clas_pair->inst_pairs;
}

static
Eqclass *
establish_new_eqclass( Inst *first_member, Eqclasset *eqclasset )
{
	Eqclass *eqclass;
	Eqmap *eqmap;
	
	eqclass = eqclass_create();

	eqclass_add_member(first_member, eqclass);
	table_append((char *)eqclass, eqclasset->eqclasses);
	
	eqmap = eqmap_create(first_member, eqclass);
	table_add((char *)eqmap, eqclasset->eqmaps);
	
	return eqclass;
}

static
void
add_eqclass_member(Inst *new_member,
		   Eqclass *eqclass,
		   Eqclasset *eqclasset)
{
	Eqmap *eqmap;

	eqclass_add_member(new_member, eqclass);

	eqmap = eqmap_create(new_member, eqclass);
	table_add((char *)eqmap, eqclasset->eqmaps);
}

Eqclass *
lookup_eqclass(Inst *member, Eqclasset *eqclasset)
{
	Eqmap *eqmap_dummy;
	Eqmap *eqmap;
	
	eqmap_dummy = eqmap_create(member, NULL);
	eqmap = (Eqmap *) table_find((char *) eqmap_dummy,
				     eqclasset->eqmaps);
	if( eqmap == NULL ) 
		return NULL;
	else
		return eqmap->eqclass;
}

static
Eqclasset *
compute_equivalence_classes(TABLE *insts)
{	
	Eqclasset *eqclasset;
	Eqclass *eqclass;
	Inst *inst1;
	Inst *inst2;
	long i;

	eqclasset = eqclasset_create();
	
	/* each inst's eqclass_index and equivalence pointer */
	table_first(insts);
	for(i=0, inst1=(Inst *)table_next(insts);
	    inst1;
	    i++, inst1=(Inst *)table_next(insts))
	{
		inst1->eqclass_index = i;
		inst1->eqclass_rep = inst1;
	}

	/* now, go back and use REF links to update eqclass_index's */
	table_first(insts);
	for(inst1 = (Inst *)table_next(insts);
	    inst1;
	    inst1 = (Inst *)table_next(insts))
	{
		inst2 = inst_follow_REF_link(inst1);	
		if( inst2 )
			make_equivalent(inst1, inst2);
	}

	/* now, use equivalence class pointers to compute classes */
	table_first(insts);
	for(inst1 = (Inst *)table_next(insts);
	    inst1;
	    inst1 = (Inst *)table_next(insts))
	{
		/* get the "root" member of inst1's class */
		inst2 = find_eqclass_rep(inst1);
		if( inst1 == inst2  )
			/* inst1 is the "root" member of its  */
			/* class, and should be first in its  */
			/* class in the table                 */
			establish_new_eqclass(inst1,
					      eqclasset);
		else
		{       
			/* add inst1 to the root member's class */
			eqclass = lookup_eqclass(inst2, eqclasset);
			add_eqclass_member(inst1,
					   eqclass,
					   eqclasset);
		}
	}
	return eqclasset;
}

static
void
make_equivalent(Inst *inst1, Inst *inst2)
{
	Inst *inst1_rep;
	Inst *inst2_rep;

	/* inst1 is equivalent to inst2.      */
	/* Find the "root" members of their   */
	/* classes, and make whichever root   */
	/* that comes second in the table     */
	/* point to the root that comes first */
	/* in the table.                      */

	inst1_rep = find_eqclass_rep(inst1);
	inst2_rep = find_eqclass_rep(inst2);
	
	if( inst1_rep->eqclass_index < inst2_rep->eqclass_index )
		inst2_rep->eqclass_rep = inst1_rep;
	
	else if( inst2_rep->eqclass_index < inst1_rep->eqclass_index )
		inst1_rep->eqclass_rep = inst2_rep;
}

static
Inst *
find_eqclass_rep(Inst *inst)
{
	if( inst->eqclass_rep == inst )
		;   /* found */
	else
		inst->eqclass_rep = 
			find_eqclass_rep(inst->eqclass_rep);
					 
	return inst->eqclass_rep;
}

static
Eqclasset *
fix_key_eqclasset( Eqclasset *eqclasset )
{
	TABLE *eqclasses;
	Eqclass *eqclass;
	TABLE *insts;
	Inst *inst;
	
	Eqclasset *new_eqclasset;
	TABLE *new_insts;

	if( eqclasset == NULL ) return NULL;
	
	new_eqclasset = eqclasset_create();
	
	eqclasses = eqclasset->eqclasses;
	
	table_first(eqclasses);
	for(eqclass=(Eqclass *)table_next(eqclasses);
	    eqclass;
	    eqclass=(Eqclass *)table_next(eqclasses))
	{
		new_insts = table_list_create(inst_compare, 
					      NULL, 
					      default_destroy );
		insts = eqclass->insts;
		if( IS_K2R )
		{
			table_first(insts);
			for(inst=(Inst *)table_next(insts);
			    inst;
			    inst=(Inst *)table_next(insts))
				if((inst->its_pair)
				   &&
				   (inst->its_pair->status == OPT))
					;
				else
					table_append((char *)inst, 
						     new_insts);
		}
		else /* key 2 key */
			table_concat(new_insts, insts);
		
		if( table_count(new_insts) == 0 )
			; /* shouldn't ever happen */
		else if((table_count(new_insts) == 1 )
			&&
			only_inst_is_not_aligned(new_insts))
		{
			table_first(new_insts);
			inst = (Inst *)table_next(new_insts);
			printf("%s%s%s",
			       "\n            Ignoring singleton eq class {",
			       inst->id,
			       "} in key");
		}
		else
			make_new_eqclass_from_insts(new_insts, new_eqclasset);
		table_destroy(new_insts);
	}
	return new_eqclasset;
}

static
Eqclasset *
fix_response_eqclasset( Eqclasset *eqclasset )
{
	TABLE *eqclasses;
	Eqclass *eqclass;
	TABLE *insts;
	Inst *inst;
	
	Eqclasset *new_eqclasset;
	TABLE *new_insts;

	if( eqclasset == NULL ) return NULL;
	
	new_eqclasset = eqclasset_create();
	
	eqclasses = eqclasset->eqclasses;
	
	table_first(eqclasses);
	for(eqclass=(Eqclass *)table_next(eqclasses);
	    eqclass;
	    eqclass=(Eqclass *)table_next(eqclasses))
	{
		new_insts = table_list_create(inst_compare, 
					      NULL, 
					      default_destroy );
		table_concat(new_insts, eqclass->insts);

		if( table_count(new_insts) == 0 )
			;
		else if((table_count(new_insts) == 1 )
			&&
			only_inst_is_not_aligned(new_insts))
		{
			table_first(new_insts);
			inst = (Inst *)table_next(new_insts);
			printf("%s%s%s",
			       "\n  WARNING:  Ignoring singleton eq class {",
			       inst->id,
			       "} in response");
		}
		else
			make_new_eqclass_from_insts(new_insts, new_eqclasset);
		table_destroy(new_insts);
	}
	return new_eqclasset;
}

static
Bool
only_inst_is_not_aligned(TABLE *insts)
{
	Inst *inst;
	
	if( insts == NULL )
		return False;
	
	table_first(insts);
	inst = (Inst *)table_next(insts);
	
	if(inst
	   &&
	   inst->its_pair
	   &&
	   ((inst->its_pair->status == COR)))
		return False;
	else
		return True;
}	
     
static
void
make_new_eqclass_from_insts(TABLE *insts, Eqclasset *eqclasset)
{
	Inst *first_inst, *inst;
	Eqclass *eqclass;
	
	if( insts == NULL || eqclasset == NULL ) return;
	
	table_first(insts);
	first_inst = (Inst *)table_next(insts);
	eqclass = establish_new_eqclass(first_inst, eqclasset);
	
	for(inst=(Inst *)table_next(insts);
	    inst;
	    inst=(Inst *)table_next(insts))
		add_eqclass_member(inst, eqclass, eqclasset);
}
