/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	sfil.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for sfil (single fill) data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)sfil.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  sfil_create
 *									*
 ***********************************************************************/

Sfil *
sfil_create(int type)
{
    Sfil *new_sfil;
    
    if((new_sfil = UALLOC(Sfil, 1)) == NULL)
    {
	fprintf(stderr, "Error: [sfil_create]: Allocation failed.\n");
	exit(1);
    }

    new_sfil->type = type;  
    switch(type)
      {
      case SET_FILL:
	new_sfil->fill.set = setfill_create();
	break;
      case POINTER_FILL:
	new_sfil->fill.ptr = pointerfill_create();
	break;
      case STRING_FILL:
	new_sfil->fill.str = stringfill_create();
	break;
      default:
	fprintf( stderr, "[sfil_create]: Unknown fill type\n" );
	exit(1);
	break;
      }

    return(new_sfil);
}


/************************************************************************
 *									*
 *  sfil_release							*
 *									*
 ***********************************************************************/

void
sfil_release(char *c_ptr)
{
    Sfil *sfil = (Sfil *)c_ptr;
    
    if(sfil == (Sfil *)NULL)
	return;
    
    switch(sfil->type)
      {
      case SET_FILL:
	setfill_release((char *)sfil->fill.set);
	break;
      case POINTER_FILL:
	pointerfill_release((char *)sfil->fill.ptr);
	break;
      case STRING_FILL:
	stringfill_release((char *)sfil->fill.str);
	break;
      default:
	fprintf(stderr, "[sfil_release]: Unknown fill type\n" );
	exit(1);
	break;
      }
    UFREE(sfil);
}

/************************************************************************
 *									*
 *  sfil_compare							*
 *									*
 ***********************************************************************/

int
sfil_compare(char *item1, char *item2)
{
    return ((int)((char *) item1 - (char *) item2));
}


/************************************************************************
 *									*
 * sfil_clear_aligned_flags
 *									*
 ***********************************************************************/

int
sfil_clear_aligned_flags( TABLE *sfils )
{
  Sfil *sfil;
  
  table_first(sfils);
  for(sfil = (Sfil *)table_next(sfils);
      sfil;
      sfil = (Sfil *)table_next(sfils))
    sfil->is_aligned = False;
}


/************************************************************************
 *									*
 *  sfil_is_removed -- if a single fill is a pointer fill, and the 
 *                     instance it points to is an unaligned optional
 *                     instance, then the single fill shouldn't be there
 *                     and will be "removed"
 *									*
 ***********************************************************************/


Bool
sfil_is_removed(Sfil *sfil)
{
  if(   sfil 
     && sfil->type == POINTER_FILL
     && sfil->fill.ptr
     && sfil->fill.ptr->fill
     && sfil->fill.ptr->fill->its_pair
     && sfil->fill.ptr->fill->its_pair->status == OPT )
    return True;
  else
    return False;
}


/************************************************************************
 *									*
 * sfil_count_unaligned
 *									*
 ***********************************************************************/

int
sfil_count_unaligned( TABLE *sfils )
{
  Sfil *sfil;
  int num_unaligned = 0;
  
  table_first(sfils);
  for(sfil = (Sfil *)table_next(sfils);
      sfil;
      sfil = (Sfil *)table_next(sfils))
    if( !sfil->is_aligned )
      num_unaligned++;
  return num_unaligned;

}



/************************************************************************
 *									*
 * sfil_print_table
 *									*
 ***********************************************************************/

void
sfil_print_table(TABLE *sfils, int indent)
{
  Sfil *sfil;
  table_first( sfils );
  for(sfil = (Sfil *)table_next(sfils);
      sfil;
      sfil = (Sfil *)table_next(sfils))
    sfil_print( sfil, indent );
}


/************************************************************************
 *									*
 * sfil_print
 *									*
 ***********************************************************************/

void
sfil_print(Sfil *sfil, int indent)
{
  switch(sfil->type)
    {
    case SET_FILL:
      /*
      setfill_print(sfil->fill.set, indent);
      */
      break;
    case POINTER_FILL:
      /*
      pointerfill_print(sfil->fill.ptr, indent);
      */
      break;
    case STRING_FILL:
      stringfill_print(sfil->fill.str, indent);
      break;
    default:
      fprintf( stderr, "[sfil_print]: Unknown fill type: %d\n", sfil->type );
      break;
    }
}
