/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 * FILE 
 *	sfil_pair.c
 *
 * SYNOPSIS
 * DESCRIPTION
 *	support routines for Sfil_pair data structure.
 *
 * DIAGNOSTICS
 * FILES
 * NOTES
 *
 * SEE ALSO
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)sfil_pair.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  sfil_pair_create							*
 *									*
 ***********************************************************************/

Sfil_pair *
sfil_pair_create(Sfil *key_sfil, Sfil *response_sfil, char status)
{
    Sfil_pair *sfil_pair;
    
    if((sfil_pair = UALLOC(Sfil_pair, 1)) == (Sfil_pair *) NULL)
    {
	fprintf(stderr, "Error: allocation for sfil_pair failed.\n");
	exit(1);
    }

    sfil_pair->key_sfil      = key_sfil;
    sfil_pair->response_sfil = response_sfil;
    sfil_pair->status        = status;
    sfil_pair->tallies       = tallies_create();
    sfil_pair->score         = 0.0 ;

    return(sfil_pair);
}


/************************************************************************
 *									*
 *  sfil_pair_release						*
 *									*
 ***********************************************************************/

void
sfil_pair_release(char *elt)
{
    Sfil_pair *ptr = (Sfil_pair *) elt;

    UFREE(ptr->tallies);
    UFREE(ptr);
    return;
}


/************************************************************************
 *									*
 *  sfil_pair_sum_tallies
 *									*
 ***********************************************************************/

int
sfil_pair_sum_tallies(Tallies *tallies, TABLE *sfil_pairs )
{
  Sfil_pair *sfil_pair;
  
  table_first(sfil_pairs);
  for(sfil_pair=(Sfil_pair *)table_next(sfil_pairs);
      sfil_pair;
      sfil_pair=(Sfil_pair *)table_next(sfil_pairs))
    tallies_combine( tallies, sfil_pair->tallies );
}



/************************************************************************
 *									*
 *  sfil_pair_f_compare
 *									*
 *  used to compare sfil_pair structures during "qsort"-ing.
 *       of "correct" sfils.
 *									*
 ***********************************************************************/

int
sfil_pair_f_compare(const void *elt1, const void *elt2)
{
    Sfil_pair *ptr1 = *(Sfil_pair **)elt1;
    Sfil_pair *ptr2 = *(Sfil_pair **)elt2;

    int dif;
    
    /* should sort in descending order by score,
     * so add a minus 
     */

    dif = (int) -(ptr1->score - ptr2->score);

    return(dif);
}


/************************************************************************
 *									*
 *  sfil_pair_wrong_compare
 *									*
 *  used to compare sfil_pair structures during "qsort"-ing of
 *       "incorrect" sfils -- we want incorrect sfil pairs sorted
 *       in increasing order by the number of wrong tallies
 *       (incorrect + missing + spurious).
 *									*
 ***********************************************************************/

int
sfil_pair_wrong_compare(const void *elt1, const void *elt2)
{
    Sfil_pair *ptr1 = *(Sfil_pair **)elt1;
    Sfil_pair *ptr2 = *(Sfil_pair **)elt2;

    int dif;
    
    dif = (int) (  ptr1->tallies->incorrect 
		 + ptr1->tallies->missing
		 + ptr1->tallies->spurious
		 - ptr2->tallies->incorrect 
		 - ptr2->tallies->missing
		 - ptr2->tallies->spurious);

    return(dif);
}
