/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	slot.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Slot data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)slot.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

/************************************************************************
 *									*
 *  slot_create
 *									*
 ***********************************************************************/

Slot *
slot_create(char *name)
{
  Slot *slot;
  
  if((slot = UALLOC(Slot, 1)) == (Slot *)NULL)
    {
      fprintf(stderr, "Error: [slot_create]: Allocation failed.\n");
      exit(1);
    }
  
  slot->name = STRALLOC(name);
  slot->is_optional = False;
  slot->mfils = table_list_create(mfil_compare,
				  mfil_release,
				  default_destroy);
  
  
  return(slot);
}


/************************************************************************
 *									*
 *  slot_release							*
 *									*
 ***********************************************************************/

void
slot_release(char *c_ptr)
{
    Slot *slot = (Slot *)c_ptr;
    
    if(slot == (Slot *)NULL)
	return;
    
    UFREE(slot->name);
    table_destroy( slot->mfils );
    UFREE(slot);
}


/************************************************************************
 *									*
 *  slot_compare							*
 *									*
 ***********************************************************************/

int
slot_compare(char *elt1, char *elt2)
{
    Slot *slot1 = (Slot *)elt1;
    Slot *slot2 = (Slot *)elt2;
    
    return(muc_strcasecmp(slot1->name, slot2->name));
}

/************************************************************************
 *									*
 * slot_clear_aligned_flags
 *									*
 ***********************************************************************/

int
slot_clear_aligned_flags( TABLE *slots )
{
  Slot *slot;
  
  table_first(slots);
  for(slot = (Slot *)table_next(slots);
      slot;
      slot = (Slot *)table_next(slots))
    slot->is_aligned = False;
}

/************************************************************************
 *									*
 *  slot_find
 *									*
 *  Returns the slot named "sname" in a table of slots.
 *									*
 ***********************************************************************/

Slot *
slot_find(TABLE *slots, char *sname)
{
    Slot  *slot;
    
    /* search the table for a slot having the same name as sname */

    table_first(slots);
    for(slot = (Slot *)table_next(slots);
	slot;
	slot = (Slot *)table_next(slots))
      if( STRCASEEQ(sname, slot->name) )
	break;

    return(slot);
}


/************************************************************************
 *									*
 *  slot_is_removed -- if every multi-fill of a slot is "removed"
 *                     (ie, all its single fills point to an unaligned 
 *                     optional instance), then the slot is also removed.
 *                     Note that the "content" slots of ST are present
 *                     but empty if the message is deemed irrevalent 
 *                     to the scenario task.  That's why empty slots
 *                     are not "removed".
 *									*
 ***********************************************************************/

Bool
slot_is_removed(Slot *slot)
{
	Mfil *mfil;
	TABLE *mfils;
	
	if((slot == NULL)
	   || 
	   (slot->mfils == NULL ))  /* should never happen */
		return False;

	mfils = slot->mfils;

	if( table_count(mfils) == 0 )  /* screwy, but true */
		return False;
	
	table_first( mfils );
	for(mfil=(Mfil *)table_next(mfils);
	    mfil;
	    mfil=(Mfil *)table_next(mfils))
		if( !mfil_is_removed(mfil) )
			return False;
	return True;
}


/************************************************************************
 *									*
 *  slot_is_unscored
 *									*
 ***********************************************************************/

Bool
slot_is_unscored(Slot *slot)
{
  if(slot
     && slot->its_slot_def )
    return( !(slot->its_slot_def->is_scored_slot) );
  return True;
}


/************************************************************************
 *									*
 * slot_count_unaligned
 *									*
 ***********************************************************************/

int
slot_count_unaligned( TABLE *slots )
{
  Slot *slot;
  int num_unaligned = 0;
  
  table_first(slots);
  for(slot = (Slot *)table_next(slots);
      slot;
      slot = (Slot *)table_next(slots))
    if( !slot->is_aligned )
      num_unaligned++;
  return num_unaligned;

}


/************************************************************************
 *									*
 * slot_print_table
 *									*
 ***********************************************************************/

void
slot_print_table(TABLE *slots, int indent)
{
  Slot *slot;
  table_first( slots );
  for(slot = (Slot *)table_next(slots);
      slot;
      slot = (Slot *)table_next(slots))
    slot_print( slot, indent );
}


/************************************************************************
 *									*
 * slot_print
 *									*
 ***********************************************************************/

void
slot_print(Slot *slot, int indent)
{

  printf("%*.s%s%s:\n", indent, "", slot->name,
	 (slot->is_optional ? " (optional)" : ""));
  
  mfil_print_table( slot->mfils, indent+2 );
}

