/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	slot_def.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Slot_def data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)slot_def.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  slot_def_create							*
 *									*
 *  Initialize a new slot_def with NAME as key, and search the table	*
 *  for a matching class_def.  If it exists, give warning about a	*
 *  re-definition.  On the assumption that a re-definition is legal,	*
 *  free the existing slot_def, then insert the new entry into the	*
 *  table.								*
 *  									*
 *  returns a pointer to a slot_def.					*
 *  									*
 ***********************************************************************/

Slot_def *
slot_def_create(char *name, char *classname)
{
    Slot_def *slot_def;
    
    if((slot_def = UALLOC(Slot_def, 1)) == (Slot_def *)NULL)
    {
	fprintf(stderr, "Error: [slot_def_create]: Allocation failed.\n");
	exit(1);
    }

    slot_def->name = STRALLOC(name);
    slot_def->score_report_name = NULL;
    slot_def->fill_types = table_list_create(cstring_compare,
					    default_release,
					    default_destroy);
    slot_def->set_fill_values = table_list_create(cstring_compare,
						  default_release,
						  default_destroy);
    slot_def->map_weight = 0;
    slot_def->is_scored_slot = False;
    slot_def->total_tallies = tallies_create();

    return(slot_def);
}


/************************************************************************
 *  									*
 *  slot_def_release							*
 *  									*
 ***********************************************************************/

void
slot_def_release(char *elt)
{
    Slot_def *slot_def = (Slot_def *)elt;

    if(slot_def == NULL)
	return;

    UFREE(slot_def->name);
    UFREE(slot_def->score_report_name);

    table_destroy(slot_def->fill_types);
    table_destroy(slot_def->set_fill_values);
    tallies_release((char *)slot_def->total_tallies);

    UFREE(slot_def);
}


/************************************************************************
 *  									*
 *  slot_def_compare							*
 *  									*
 ***********************************************************************/

int
slot_def_compare(char *elt1, char *elt2)
{
    Slot_def *ptr1 = (Slot_def *)elt1;
    Slot_def *ptr2 = (Slot_def *)elt2;
    
    return(muc_strcasecmp(ptr1->name, ptr2->name));
}

/************************************************************************
 *									*
 *  slot_def_find
 *									*
 ***********************************************************************/


Slot_def *
slot_def_find(TABLE *slot_defs, char *name)
{
    Slot_def  *thisdef;
    
    /* search the table for a slot_def having the same
       name as sname */

    table_first(slot_defs);
    for(thisdef = (Slot_def *)table_next(slot_defs);
	thisdef && !(STRCASEEQ(name, thisdef->name));
	thisdef = (Slot_def *)table_next(slot_defs))
	;
    
    return(thisdef);
}

/************************************************************************
 *									*
 *  slot_def_which_type
 *									*
 ***********************************************************************/


char
slot_def_which_type(Slot_def *slot_def)
{
	char *fill_type;
	TABLE *fill_types;
	
	if( (slot_def == NULL) || (slot_def->fill_types == NULL) )
		return UNKNOWN;
	fill_types = slot_def->fill_types;
	table_first(fill_types);
	fill_type = table_next(fill_types);
	if( fill_type == NULL ) return UNKNOWN;

	if( STRCASEEQ(fill_type, "set") )
		return SET_FILL;
	if( STRCASEEQ(fill_type, "string") )
		return STRING_FILL;
	return POINTER_FILL;
}

/************************************************************************
 *									*
 * slot_def_print_table
 *									*
 ***********************************************************************/

void
slot_def_print_table(TABLE *slot_def_table, int indent)
{
  Slot_def *slot_def;
  table_first( slot_def_table );
  for(slot_def = (Slot_def *)table_next(slot_def_table);
      slot_def;
      slot_def = (Slot_def *)table_next(slot_def_table))
    slot_def_print( slot_def, indent );
}


/************************************************************************
 *									*
 *  slot_def_print
 *									*
 ***********************************************************************/

void
slot_def_print(Slot_def *sd, int indent)
{
    printf("%*.s%s =\n", indent, "", sd->name);
    printf("%*.s%s\n", indent, "", sd->score_report_name);
    cstring_print_table( sd->fill_types, indent + 2 );
}

