/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 * FILE 
 *	slot_pair.c
 *
 * SYNOPSIS
 * DESCRIPTION
 *	support routines for Slot_pair data structure.
 *
 * DIAGNOSTICS
 * FILES
 * NOTES
 *
 * SEE ALSO
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */


#ifdef SCCSID
static char SccsId[] = "@(#)slot_pair.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  slot_pair_create							*
 *									*
 ***********************************************************************/

Slot_pair *
slot_pair_create(Slot *key_slot, Slot *response_slot, char status)
{
    Slot_pair *slot_pair;
    
    if((slot_pair = UALLOC(Slot_pair, 1)) == (Slot_pair *) NULL)
    {
	fprintf(stderr, "Error: allocation for slot_pair failed.\n");
	exit(1);
    }

    if( key_slot )
	    slot_pair->name  = STRALLOC(key_slot->name);
    else if( response_slot )
	    slot_pair->name  = STRALLOC(response_slot->name);
    else
	    slot_pair->name  = NULL;
    slot_pair->key_slot      = key_slot;
    slot_pair->response_slot = response_slot;
    slot_pair->status        = status;
    slot_pair->mfil_pairs    = table_list_create(default_compar,
						 mfil_pair_release,
						 default_destroy );
    slot_pair->tallies =  tallies_create();
    slot_pair->score =    0.0 ;
    
    return(slot_pair);
}


/************************************************************************
 *									*
 *  slot_pair_release						*
 *									*
 ***********************************************************************/

void
slot_pair_release(char *elt)
{
    Slot_pair *ptr = (Slot_pair *) elt;

    UFREE(ptr->tallies);
    UFREE(ptr->name);
    table_destroy( ptr->mfil_pairs );
    UFREE(ptr);

    return;
}

Slot_pair *
slot_pair_find(TABLE *slot_pairs, char *name)
{
  Slot_pair *slot_pair;
  
  table_first(slot_pairs);
  for(slot_pair = (Slot_pair *)table_next(slot_pairs);
      slot_pair && !(STRCASEEQ(name, slot_pair->name));
      slot_pair = (Slot_pair *)table_next(slot_pairs))
    ;
  
  return(slot_pair);
}



/************************************************************************
 *									*
 *  slot_pair_sum_tallies
 *									*
 ***********************************************************************/

int
slot_pair_sum_tallies(Tallies *tallies, TABLE *slot_pairs )
{
  Slot_pair *slot_pair;
  
  table_first(slot_pairs);
  for(slot_pair=(Slot_pair *)table_next(slot_pairs);
      slot_pair;
      slot_pair=(Slot_pair *)table_next(slot_pairs))
    tallies_combine( tallies, slot_pair->tallies );
}

/************************************************************************
 *									*
 *  slot_pair_sum_weighted_scores
 *      for every slot_pair in the table, multiply the slot pair's
 *      score by the slot definition's map_weight, then add the 
 *      product to the result
 *									*
 ***********************************************************************/

float
slot_pair_sum_weighted_scores( TABLE *slot_pairs )
{
  Slot_pair *slot_pair;
  Slot_def  *slot_def;
  float     result = 0.0;
  
  table_first(slot_pairs);
  for(slot_pair=(Slot_pair *)table_next(slot_pairs);
      slot_pair;
      slot_pair=(Slot_pair *)table_next(slot_pairs))
    {
      slot_def = NULL;       /* if can't find slot_def, don't use a */
                             /* different slot's!                   */
      if( slot_pair->key_slot )
	slot_def = slot_pair->key_slot->its_slot_def;
      else if( slot_pair->response_slot )
	slot_def = slot_pair->response_slot->its_slot_def;
      if( slot_def && (slot_def->is_scored_slot) )
	result += (slot_def->map_weight * slot_pair->score);
    }
  return result;
}



/************************************************************************
 *									*
 *  slot_pair_f_compare
 *									*
 *  used to compare slot_pair structures during "qsort"-ing.
 *       of "correct" slots.
 *									*
 ***********************************************************************/

int
slot_pair_f_compare(const void *elt1, const void *elt2)
{
    Slot_pair *ptr1 = *(Slot_pair **)elt1;
    Slot_pair *ptr2 = *(Slot_pair **)elt2;

    int dif;
    
    /* should sort in descending order by score,
     * so add a minus 
     */

    dif = (int) -(ptr1->score - ptr2->score);

    return(dif);
}


/************************************************************************
 *									*
 *  slot_pair_wrong_compare
 *									*
 *  used to compare slot_pair structures during "qsort"-ing of
 *       "incorrect" slots -- we want incorrect slot pairs sorted
 *       in increasing order by the number of wrong tallies
 *       (incorrect + missing + spurious).
 *									*
 ***********************************************************************/

int
slot_pair_wrong_compare(const void *elt1, const void *elt2)
{
    Slot_pair *ptr1 = *(Slot_pair **)elt1;
    Slot_pair *ptr2 = *(Slot_pair **)elt2;

    int dif;
    
    dif = (int) (  ptr1->tallies->incorrect 
		 + ptr1->tallies->missing
		 + ptr1->tallies->spurious
		 - ptr2->tallies->incorrect 
		 - ptr2->tallies->missing
		 - ptr2->tallies->spurious);

    return(dif);
}




