/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	slot_score.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Slot_score data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)slot_score.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  slot_score_create							*
 *									*
 ***********************************************************************/

Slot_score *
slot_score_create(char *name, char *docnum)
{
    Slot_score *newscore;
    
    if((newscore = UALLOC(Slot_score, 1)) == (Slot_score *)NULL)
    {
	fprintf(stderr, "Error: [slot_score_create]: Allocation failed.\n");
	exit(1);
    }
    
    newscore->name = STRALLOC(name);
    newscore->docnum = STRALLOC(docnum);
    newscore->tallies = tallies_create();
    
    return(newscore);
}


/************************************************************************
 *									*
 *  slot_score_release							*
 *									*
 ***********************************************************************/

void
slot_score_release(char *elt)
{
    Slot_score *ptr = (Slot_score *)elt;
    
    UFREE(ptr->name);
    UFREE(ptr->docnum);
    tallies_release((char *) ptr->tallies);
    
    UFREE(ptr);
}


/************************************************************************
 *									*
 *  slot_score_compare							*
 *									*
 ***********************************************************************/

int
slot_score_compare(char *elt1, char *elt2)
{
    Slot_score *ptr1 = (Slot_score *)elt1;
    Slot_score *ptr2 = (Slot_score *)elt2;
    
    return(muc_strcasecmp(ptr1->name, ptr2->name));
}

/************************************************************************
 *									*
 *  slot_score_find							*
 *									*
 ***********************************************************************/

Slot_score *
slot_score_find(TABLE *scores, char *sname)
{
    Slot_score  *this_score;
    
    /* search the table for a slot_score having the same
       name as sname. */
    table_first(scores);
    for(this_score = (Slot_score *)table_next(scores);
	this_score && !(STRCASEEQ(sname, this_score->name));
	this_score = (Slot_score *)table_next(scores))
	;
    
    return(this_score);
}

void
slot_score_init_table(TABLE *slot_scores, Class_def *class_def, char *docnum)
{
	TABLE *slot_defs;
	Slot_def *slot_def;
	Slot_score *slot_score;
	
	if( slot_scores == NULL || class_def == NULL || docnum == NULL )
	{
		fprintf(stderr,
			"[slot_score_init_table]: parameter(s) NULL\n");
		exit(1);
	}
	slot_defs = class_def->slot_defs;
	table_first(slot_defs);
	for(slot_def=(Slot_def *)table_next(slot_defs);
	    slot_def;
	    slot_def=(Slot_def *)table_next(slot_defs))
	{
		slot_score = slot_score_create(slot_def->name, docnum);
		table_append((char *)slot_score, slot_scores);
	}
}

			
	
