/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	stringfill.c
 *
 * SYNOPSIS
 *      stringfill_create
 *      stringfill_release
 *      stringfill_print
 *
 * DESCRIPTION
 *	support routines for stringfill data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	October 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)stringfill.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  stringfill_create							*
 *									*
 ***********************************************************************/

Stringfill *
stringfill_create(void)
{
    Stringfill *newfill;
    
    if((newfill = UALLOC(Stringfill, 1)) == (Stringfill *) NULL)
    {
	fprintf(stderr, "Error: [stringfill_create]: Allocation failed.\n");
	exit(1);
    }

    newfill->fill = (char *) NULL;
    newfill->straight_fill = (char *) NULL;
    newfill->clean_fill = (char *) NULL;
    
    newfill->start_offset = 0;
    newfill->end_offset = 0;

    return(newfill);
}


/************************************************************************
 *									*
 *  stringfill_release							*
 *									*
 ***********************************************************************/

void
stringfill_release(char *c_ptr)
{
    Stringfill *stringfill = (Stringfill *) c_ptr;
    
    if(stringfill == (Stringfill *)NULL)
	return;

    /* free the pointer to the allocated text string */
    UFREE(stringfill->fill);
    UFREE(stringfill->straight_fill);
    UFREE(stringfill->clean_fill);
    UFREE(stringfill);
}

/*********************************************************************
 * 
 * stringfill_match -- comparees two stringfills, and returns COR, INC
 *                  or PAR -- also increments the tallies data 
 *                  structure
 * 
 *    this function uses the values of the options 
 *           :stringfill_correct_comparison
 *           :stringfill_partial_comparison
 *    First it looks to see if the two stringfills match "correctly"
 *    If they do, or if the option stringfill_correct_comparison is 
 *         NONE (in which case they'll be incorrect), we're done.
 *    If they don't match correctly, the function looks for a partial
 *         match. 
 *    If they don't match that either, it's incorrect.
 *
 ********************************************************************
 */

char
stringfill_match(Stringfill *key_stringfill, 
		 Stringfill *response_stringfill,
		 Tallies *tallies)
{
	char result = INC;
	Bool done = False;
	
	switch( MUC_stringfill_correct_comparison )
	{
	case ORIG:
		if(strcmp(key_stringfill->fill,
			  response_stringfill->fill) == 0)
		{
			tally1_correct( tallies );
			done=True;
			result = COR;
		}
		break;
	case STRAIGHTENED:
		if(strcmp(key_stringfill->straight_fill,
			  response_stringfill->straight_fill) == 0)
		{
			tally1_correct( tallies );
			done=True;
			result = COR;
		}
		break;
	case CLEAN:
		if(strcmp(key_stringfill->clean_fill,
			  response_stringfill->clean_fill) == 0)
		{
			tally1_correct( tallies );
			done=True;
			result = COR;
		}
		break;
	case NONE:
	default:
		{
			tally1_incorrect( tallies );
			done=True;
			result = INC;
		}
		break;
	}
	if( ! done )
		switch( MUC_stringfill_partial_comparison )
		{
		case ORIG:
			if(strcmp(key_stringfill->fill,
				  response_stringfill->fill) == 0)
			{	
				tally1_partial( tallies );
				done = True;
				result = PAR;
			}
			break;
		case STRAIGHTENED:
			if(strcmp(key_stringfill->straight_fill,
				  response_stringfill->straight_fill) == 0)
			{
				tally1_partial( tallies );
				done = True;
				result = PAR;
			}
			break;
		case CLEAN:
			if(strcmp(key_stringfill->clean_fill,
				  response_stringfill->clean_fill) == 0)
			{
				tally1_partial( tallies );
				done = True;
				result = PAR;
			}
			break;
		case NONE:
		default:
			tally1_incorrect( tallies );
			done = True;
			result = INC;
			break;
		}
	if( ! done )
	{
		tally1_incorrect( tallies );
		result = INC;
	}
		
	return result;
}


/************************************************************************
 *									*
 *  stringfill_print							*
 *									*
 ***********************************************************************/

void
stringfill_print(Stringfill *stringfill, int indent)
{
	if( stringfill )
	{
		if( stringfill->fill )
			printf("%s\n",
			       stringfill->fill); 
		if( stringfill->clean_fill )
			printf("%s\n", 
			       stringfill->clean_fill);
		printf("\n");
	}
}


