/*
 *  Copyright 1995 Science Applications International Corporation.
 */

/*
 * NAME
 *
 * 
 * FILE 
 *	tallies.c
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	support routines for Tallies data structure.
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 *	Scratch pad - modification history, plans for enhancements.
 * 
 * SEE ALSO
 *
 * AUTHOR
 *	Gary Dungca	August 1995
 *
 */

#ifdef SCCSID
static char SccsId[] = "@(#)tallies.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"


/************************************************************************
 *									*
 *  tallies_create							*
 *									*
 ***********************************************************************/

Tallies *
tallies_create(void)
{
    Tallies *newtallies;
    
    if((newtallies = UALLOC(Tallies, 1)) == (Tallies *)NULL)
    {
	fprintf(stderr, "Error: [tallies_create]: Allocation failed.\n");
	exit(1);
    }

    newtallies->possible = 0;
    newtallies->actual = 0;
    newtallies->correct = 0;
    newtallies->partial = 0;
    newtallies->incorrect = 0;
    newtallies->missing = 0;
    newtallies->spurious = 0;
    newtallies->noncommittal = 0;
    
    return(newtallies);
}


/************************************************************************
 *									*
 *  tallies_release							*
 *									*
 *  since there are no dynamically allocated blocks within the current	*
 *  structure definition, we could simply call UFREE on the tallies	*
 *  ptr instead of calling this routine.  however, in the event that	*
 *  the structure definition changes to include allocated memory, then	*
 *  it will be simpler to change the freeing process in this routine	*
 *  than changing all the instances of UFREE calls on this pointer.	*
 *									*
 ***********************************************************************/


void
tallies_release(char  *c)
{
	Tallies *t = (Tallies *) c;
	
	UFREE(t);
}


/************************************************************************
 *									*
 *									*
 *									*
 ***********************************************************************/

void
tallies_copy(Tallies *t1, Tallies *t2)
{
    t1->possible = t2->possible;
    t1->actual = t2->actual;
    t1->correct = t2->correct;
    t1->partial = t2->partial;
    t1->incorrect = t2->incorrect;
    t1->missing = t2->missing;
    t1->spurious = t2->spurious;
    t1->noncommittal = t2->noncommittal;

    return;
}

void
tallies_combine(Tallies *t1, Tallies *t2)
{
    t1->possible += t2->possible;
    t1->actual += t2->actual;
    t1->correct += t2->correct;
    t1->partial += t2->partial;
    t1->incorrect += t2->incorrect;
    t1->missing += t2->missing;
    t1->spurious += t2->spurious;
    t1->noncommittal += t2->noncommittal;

    return;
}


void
tallies_subtract(Tallies *t1, Tallies *t2)
{
    t1->possible -= t2->possible;
    t1->actual -= t2->actual;
    t1->correct -= t2->correct;
    t1->partial -= t2->partial;
    t1->incorrect -= t2->incorrect;
    t1->missing -= t2->missing;
    t1->spurious -= t2->spurious;
    t1->noncommittal -= t2->noncommittal;

    return;
}


void
tallies_clear(Tallies *t)
{
    t->possible = 0;
    t->actual = 0;
    t->correct = 0;
    t->partial = 0;
    t->incorrect = 0;
    t->missing = 0;
    t->spurious = 0;
    t->noncommittal = 0;

    return;
}

void
tallies_print(Tallies *t)
{
	printf("(%d %d %d %d %d %d %d %d)\n",
	       t->possible,
	       t->actual,
	       t->correct,
	       t->partial,
	       t->incorrect,
	       t->missing,
	       t->spurious,
	       t->noncommittal);
}



/************************************************************************
 *									*
 *  tally shortcuts
 *									*
 ***********************************************************************/


void
tally_correct(Tallies* tallies, int count)
{
	((tallies)->correct += (count));
	((tallies)->possible += (count));
	((tallies)->actual += (count));
}

void
tally_incorrect(Tallies* tallies, int count)
{
	((tallies)->incorrect += (count));
	((tallies)->possible += (count));
	((tallies)->actual += (count));
}

void
tally_partial(Tallies* tallies, int count)
{
	((tallies)->partial += (count));
	((tallies)->possible += (count));
	((tallies)->actual += (count));
}

void
tally_missing(Tallies* tallies, int count)
{
	((tallies)->missing += (count));
	((tallies)->possible += (count));
}

void
tally_spurious(Tallies* tallies, int count)
{
	((tallies)->spurious += (count));
	((tallies)->actual += (count));
}

void
tally_noncommittal(Tallies* tallies, int count)
{
	((tallies)->noncommittal += (count));
}

void
tally1_correct(Tallies* tallies)
{
	((tallies)->correct++);
	((tallies)->possible++);
	((tallies)->actual++);
}

void
tally1_partial(Tallies* tallies)
{
	((tallies)->partial++);
	((tallies)->possible++);
	((tallies)->actual++);
}

void
tally1_incorrect(Tallies* tallies)
{
	((tallies)->incorrect++);
	((tallies)->possible++);
	((tallies)->actual++);
}

void
tally1_missing(Tallies* tallies)
{
	((tallies)->missing++);
	((tallies)->possible++);
}

void
tally1_spurious(Tallies* tallies)
{
	((tallies)->spurious++);
	((tallies)->actual++);}

void
tally1_noncommittal(Tallies* tallies)
{
	((tallies)->noncommittal++);
}


