/*
 * FILE 
 *	@(#)utils.c	6.1
 *
 * SYNOPSIS
 *
 *
 * DESCRIPTION
 *	utility functions
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *
 * NOTES
 * 
 * SEE ALSO
 *
 * AUTHOR
 */

#ifdef SCCSID
static char SccsId[] = "@(#)utils.c	6.1\t09/08/97\tSAIC";
#endif

#undef SCCSID
#include  "muc_defs.h"
#include  "muc.h"
#include  "muc_protos.h"
#include  "muc_globals.h"

void
breaker()
{
	return;
}


void
non_release(char *elt)
{
	return;
}


void
table_concat( TABLE *t1, TABLE *t2 )
{
	char *c;
	table_first(t2);
	for(c=table_next(t2);
	    c;
	    c=table_next(t2))
		table_append(c, t1);
}

void
table_reverse( TABLE *t1, TABLE *t2 )
{
	char *c;
	table_first(t2);
	for(c=table_next(t2);
	    c;
	    c=table_next(t2))
		table_prepend(c, t1);
}

void
muc_memmove(char *s, char *ct, int n)
{
	char *buf;
	int i;
	
	buf = (char *)malloc(n);
	for(i=0; i<n; i++)
		buf[i] = ct[i];
	for(i=0; i<n; i++)
		s[i] = buf[i];
	UFREE( buf );
}


/************************************************************************
 *									*
 *  compute_tallies_f							*
 *									*
 *  compute an f-measure based on the scoring information in tallies.	*
 *  recall = (#correct + (0.5 * #partial)) / #possible			*
 *  precision = (#actual + (0.5 * #partial) / #actual			*
 *  return -1.0 if either #pos or #act = 0				*
 *									*
 ***********************************************************************/

float
compute_tallies_f(Tallies *tallies, int which_f)
{
    static float f_pr = 0.5;
    static float f_2pr = 0.8;
    static float f_p2r = 0.2;
    
    float fpos, fact, fcor, fpar, frecall, fprecision, beta, result;
    
    fpos = (float) tallies->possible;
    fact = (float) tallies->actual;
    fcor = (float) tallies->correct;
    fpar = (float) tallies->partial;
    
    switch(which_f)
    {
    case F_PR:
	beta = f_pr;
	break;
    case F_2PR:
	beta = f_2pr;
	break;
    case F_P2R:
	beta = f_p2r;
	break;
    default:
	beta = f_pr;
    }

    /* get recall and precision percentages before calling
       compute_f to get result.  if either fpos or fact = 0.0,
       then this falls out, and the return value is -1.0 */
    if ((fpos > 0.0) && (fact > 0.0))
    {
	frecall = compute_recall(tallies);
	fprecision = compute_precision(tallies);

	result = compute_f(beta, fprecision, frecall);
	return(result);
    }
    else
	return(-1.0);
}


/************************************************************************
 *									*
 *  compute_f								*
 *									*
 *  = (	1.0 / ( (b * (1.0 / p)) +					*
 *		((1.0 - b) * (1.0 / r)) ) )				*
 *									*
 ***********************************************************************/

float
compute_f(float beta, float precision, float recall)
{
    float fval = 0.0;

    if ((precision == 0.0) || (recall == 0.0))
	return(fval);
    
    fval = (1.0 / ((beta * (1.0 / precision)) +
		   ((1.0 - beta) * (1.0 / recall))));
    
    return(fval);
}


/************************************************************************
 *									*
 *  compute_error_per_response_fill					*
 *									*
 *  = ( ((partial / 2) + incorrect + missing + spurious)		*
 *      / (correct + partial + incorrect + missing + spurious))		*
 *    * 100.0								*
 *									*
 ***********************************************************************/

float
compute_error_per_response_fill(Tallies *tallies)
{
    float correct = (float) tallies->correct;
    float partial = (float) tallies->partial;
    float incorrect = (float) tallies->incorrect;
    float missing = (float) tallies->missing;
    float spurious = (float) tallies->spurious;
    float num, denom, errprf;
    
    num = (partial / 2.0) + incorrect + missing + spurious;
    denom = correct + partial + incorrect + missing + spurious;
    
    if (denom == 0.0)
	errprf = -1.0;
    else
	errprf = (num / denom) * 100.0;
    
    return(errprf);
}


/************************************************************************
 *									*
 *  compute_overgeneration						*
 *									*
 *  = (spurious / actual) * 100.0					*
 *									*
 ***********************************************************************/

float
compute_overgeneration(Tallies *tallies)
{
    float actual = (float) tallies->actual;
    float spurious = (float) tallies->spurious;
    float overgeneration;
    
    if (actual == 0.0)
	overgeneration = -1.0;
    else
	overgeneration = (spurious / actual) * 100.0;
    
    return(overgeneration);
}


/************************************************************************
 *									*
 *  compute_precision							*
 *									*
 *  = ((correct + (partial / 2)) / actual) * 100			*
 *									*
 ***********************************************************************/

float
compute_precision(Tallies *tallies)
{
    float actual = (float) tallies->actual;
    float correct = (float) tallies->correct;
    float partial = (float) tallies->partial;
    float precision;
    
    if (actual == 0.0)
	precision = -1.0;
    else
	precision = ((correct + (partial / 2.0)) / actual) * 100.0;
    
    return(precision);
}


/************************************************************************
 *									*
 *  compute_recall							*
 *									*
 *  = ((correct + (partial / 2)) / possible) * 100			*
 *									*
 ***********************************************************************/

float
compute_recall(Tallies *tallies)
{
    float possible = (float) tallies->possible;
    float correct = (float) tallies->correct;
    float partial = (float) tallies->partial;
    float recall;
    
    if (possible == 0.0)
	recall = -1.0;
    else
	recall = ((correct + (float)(partial / 2.0)) / possible) * 100.0;
    
    return(recall);
}


/************************************************************************
 *									*
 *  compute_substitution						*
 *									*
 *  = ( (incorrect + (partial / 2))					*
 *      / (correct + partial + incorrect))				*
 *    * 100.0								*
 *									*
 ***********************************************************************/

float
compute_substitution(Tallies *tallies)
{
    float correct = (float) tallies->correct;
    float partial = (float) tallies->partial;
    float incorrect = (float) tallies->incorrect;
    float denom, substitution;
    
    denom = correct + partial + incorrect;
    
    if (denom == 0.0)
	substitution = -1.0;
    else
	substitution = ((incorrect + (partial / 2.0)) / denom) * 100.0;
    
    return(substitution);
}


/************************************************************************
 *									*
 *  compute_undergeneration						*
 *									*
 *  = (missing / possible) * 100.0					*
 *									*
 ***********************************************************************/

float
compute_undergeneration(Tallies *tallies)
{
    float possible = (float) tallies->possible;
    float missing = (float) tallies->missing;
    float undergeneration;
    
    if (possible == 0.0)
	undergeneration = -1.0;
    else
	undergeneration = (missing / possible) * 100.0;
    
    return(undergeneration);
}
